package applications;

/*
 *  Copyright (C) 2014 Juergen Platzer 
 * 
 *  This file is part of JFXLibrary.
 *  
 *  JFXLibrary is free software: you can redistribute it and/or modify
 *  it under the terms of the GNU Lesser General Public License as published by
 *  the Free Software Foundation, either version 3 of the License, or
 *  (at your option) any later version.
 *  
 *  JFXLibrary is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU Lesser General Public License for more details.
 *  
 *  You should have received a copy of the GNU Lesser General Public License
 *  along with JFXLibrary.  If not, see <http://www.gnu.org/licenses/>.
 */

import java.io.File;
import java.io.BufferedInputStream;
import java.io.FileInputStream;

import java.util.Properties;

/**
 * An entity that holds the language properties, for example 
 * all label names for the specified language. 
 **/

public class LanguageProperties {


  /*** CONSTANTS ***/
  /** The string constant for the language English. **/
  public static final String ENGLISH = "ENGLISH";
  /** The string constant for the language German. **/
  public static final String GERMAN = "GERMAN";
  /** The string constant for the language Austrian. **/
  public static final String AUSTRIAN = "AUSTRIAN";
  
  /*** member variables ***/
  private Properties m_pProperties; 
  private DefaultLanguageProperties m_pFallbackProperties;

  ////////////////////////////////
  // CONSTRUCTOR                //
  ////////////////////////////////
  /**
   *  Creates an object with the labels of the specified language. If the specified language configuration 
   *  cannot be loaded, the {@code DefaultLanguageProperties} are used to return the English default labels. 
   *
   *  @param pLanguage The language of the properties as string. 
   *
   **/
  public LanguageProperties(String pLanguage) {
    
    
    //1. fetch the file path
    String pFileName = getFileName(pLanguage);
    
    //2. load the properties
    try {
      m_pProperties = new Properties();
      BufferedInputStream pStream = new BufferedInputStream(new FileInputStream(pFileName));
      m_pProperties.load(pStream);
      pStream.close();
    }
    catch (Exception pException) {
      System.out.println("The property file " + pFileName + 
      " could not be loaded! " + pException.getMessage());
    }
    
    //3. fallback properties
    m_pFallbackProperties = new DefaultLanguageProperties();
    
  }
  
  
  ////////////////////////////////
  // GET METHODS                //
  ////////////////////////////////
  /**
   *  Returns the value of the specified property.
   *
   *  @param pProperty A string indicating the name of the property.
   *
   *  @return The value of the specified property as string.
   **/
  public String getProperty(String pProperty) {
    //1. make a simple check
    if (pProperty == null) return null;
    if (pProperty.trim().equals("")) return null;
  
    //2. return the language property
    if (m_pProperties != null) {
      String pValue = m_pProperties.getProperty(pProperty);
      if (pValue != null) return pValue;
    }
    
    //3. if not available return the value from the fallback properties
    return m_pFallbackProperties.getProperty(pProperty);
  }
  
  
  
  //////////////////////////////
  // STATIC HELPER METHODS    //
  //////////////////////////////
  private static String getFileName(String pLanguage) {
    //1. make an initial check
    if ((pLanguage == null) || (pLanguage.trim().equals(""))) pLanguage = ENGLISH;
    
    //2. return the path for the given language
    String pPrefix = "props" + File.separator + "languages" + File.separator;
    if (pLanguage.equalsIgnoreCase(GERMAN)) {
      return pPrefix + "german.properties";
    }
    else if (pLanguage.equalsIgnoreCase(AUSTRIAN)) {
      return pPrefix + "austrian.properties";
    }
    
    return pPrefix + "english.properties";
  }

}