package applications;

/*
 *  Copyright (C) 2014 Juergen Platzer 
 * 
 *  This file is part of JFXLibrary.
 *  
 *  JFXLibrary is free software: you can redistribute it and/or modify
 *  it under the terms of the GNU Lesser General Public License as published by
 *  the Free Software Foundation, either version 3 of the License, or
 *  (at your option) any later version.
 *  
 *  JFXLibrary is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU Lesser General Public License for more details.
 *  
 *  You should have received a copy of the GNU Lesser General Public License
 *  along with JFXLibrary.  If not, see <http://www.gnu.org/licenses/>.
 */

import java.util.Hashtable;
import java.util.Properties;

import java.io.FileOutputStream;

/**
 * An entity that holds the settings of the gui application. 
 **/
public class Settings {
  
  /** member variables **/
  private Hashtable<String, String> m_pDefaultProperties;
  private Properties m_pSpecificProperties;
  
  /////////////////////////////////
  // CONSTRUCTOR                 //
  /////////////////////////////////
  
  /**
   *  Creates the default settings.
   **/
  public Settings() {
    m_pSpecificProperties = null;
  
    createDefaultSettings();

  }
  
  
  /**
   *   Loads the settings saved in the specified properties object.
   *
   *   @param pProperties The properties object from which the settings should be taken.
   **/ 
   
  public Settings(Properties pProperties) {
    m_pSpecificProperties = pProperties;
    
    createDefaultSettings();
    
  }
  
  
  //////////////////////////////////
  // GET METHODS                  //
  //////////////////////////////////
  /**
   *  Returns the value of the specified property.
   *
   *  @param pProperty A string indicating the name of the property.
   *
   *  @return The value of the specified property as string.
   **/
  public String getProperty(String pProperty) {
    //1. make an initial check
    if (pProperty == null) return null;
    if (pProperty.trim().equals("")) return null;
    
    //2. return the default value
    if (m_pSpecificProperties == null) {
      return m_pDefaultProperties.get(pProperty);
    }
    
    //3. specific value
    String pValue = m_pSpecificProperties.getProperty(pProperty);
    if (pValue == null) {
      return m_pDefaultProperties.get(pProperty);
    } 
    
    return pValue;
    
    
  }
  
  /////////////////////////////////
  // SET METHODS                 //
  /////////////////////////////////
  /**
   *  Sets the value of the specified property.
   *
   *  @param pProperty A string indicating the name of the property.
   *  @param pValue The value of the specified property as string.
   *
   **/
  public void setProperty(String pProperty, String pValue) {
    //1. make an initial check
    if (pProperty == null) return;
    if (pProperty.trim().equals("")) return;
    if (m_pSpecificProperties == null) return;
    
    
    //3. specific value
    m_pSpecificProperties.setProperty(pProperty, pValue);
    
  }
  
  
  /////////////////////////////////
  // STORE METHODS               //
  /////////////////////////////////
   /**
   *  Stores the current settings to a Java properties file with the specified file name.
   *
   *  @param pFileName The file name of the java properties file that should be stored.
   *
   *  @throws Exception If the file name is not defined, if there are no values to store or if the store procedure has IO exceptions.
   *
   **/
  public void store(String pFileName) throws Exception {
    //1. make some funny checks
    if (pFileName == null) throw new NullPointerException("No filename was specified!");
    if (pFileName.trim().equals("")) throw new Exception("No filename was specified!");
    if (m_pSpecificProperties == null) throw new Exception("No properties were loaded!");
    
    //2. store it
    FileOutputStream pFileOutputStream = new FileOutputStream(pFileName);
    m_pSpecificProperties.store(pFileOutputStream, "");
    pFileOutputStream.close();
  }
  
  
  
  
  ////////////////////////////////////
  // PRIVATE HELPER METHODS         //
  ////////////////////////////////////
  private void createDefaultSettings() {
    m_pDefaultProperties = new Hashtable<String, String>();
    
    m_pDefaultProperties.put("Language", "English");

  }
  
}