package color;

/*
 *  Copyright (C) 2014 Juergen Platzer 
 * 
 *  This file is part of JFXLibrary.
 *  
 *  JFXLibrary is free software: you can redistribute it and/or modify
 *  it under the terms of the GNU Lesser General Public License as published by
 *  the Free Software Foundation, either version 3 of the License, or
 *  (at your option) any later version.
 *  
 *  JFXLibrary is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU Lesser General Public License for more details.
 *  
 *  You should have received a copy of the GNU Lesser General Public License
 *  along with JFXLibrary.  If not, see <http://www.gnu.org/licenses/>.
 */

//standard imports
import java.awt.image.BufferedImage;
import java.awt.Color;
import java.awt.Graphics2D;

//library imports
import standard.ImageEffect;
import standard.EffectParameters;
import util.image.ImageTransformations;

import static color.ColorTransitionParameters.*;

/**
 *  Implements a color transition effect.
 **/
public class ColorTransition extends ImageEffect {

  /*** private member variables ***/
  private ColorTransitionParameters m_pParameters;
  
  private BufferedImage m_pColorGradientImage;
  private BufferedImage m_pImage;
  private Color m_pTransitionColor;
  private int m_iTransitionLength;
  private boolean m_bTransitionTop, m_bTransitionBottom;

  
  //////////////////////////////
  // CONSTRUCTOR              //
  //////////////////////////////
  /**
   * Creates a color transition effect object with the default effect settings.
   **/
  public ColorTransition() {
    this(new ColorTransitionParameters());
  }
  

  /**
   *   Creates a ColorTransition object with the specified parameters.
   *
   *   @param pParameters The parameters that configure the effect.
   **/
  public ColorTransition(ColorTransitionParameters pParameters) {
    //1. Make an initial check
    if (pParameters == null) pParameters = new ColorTransitionParameters();
    
    //2. set the member variables
    m_pParameters = pParameters;
    fetchParameters();
  }
  
  
  //////////////////////////////
  // SET METHODS              //
  //////////////////////////////
  /**
   *   Sets the parameters for the color transition effect.
   *
   *   @param pParameters The parameters that configure the effect.
   **/
  public void setColorTransitionParameters(ColorTransitionParameters pParameters) {
    //1. Make an initial check
    if (pParameters == null) return;
    
    
    //2. set the member variables
    m_pParameters = pParameters;
    fetchParameters();
     
  }
  
  ////////////////////////////////
  // SET METHODS                //
  ////////////////////////////////
  /**
   *   Sets the parameters for the color transition effect. The parameters are only set, 
   *   if the object is of the type {@code ColorTransitionParameters}.
   *
   *   @param pEffectParameters The parameters that configure the effect. 
   **/
  public void setEffectParameters(EffectParameters pEffectParameters) {
    if (pEffectParameters == null) return;
    
    if (pEffectParameters instanceof ColorTransitionParameters) {
      setColorTransitionParameters((ColorTransitionParameters)pEffectParameters);
    }
  }
  
  
  //////////////////////////////
  // GET METHODS              //
  //////////////////////////////
  /**
   *   Returns the parameters that are currently used for the grayscale reduction effect.
   *
   *   @return The parameters that are currently used for the effect.
   **/
  public EffectParameters getEffectParameters() {
    return m_pParameters;
  }
  
  
    
  
  //////////////////////////////
  // PROCESS METHODS          //
  //////////////////////////////
  /**
   *  Returns an image processed with the current color transition.
   *
   *  @return An image processed with the current color transition.
   *
   */
  public BufferedImage process() throws Exception {
   
    
    
    
    //1. copy the image 
    int iWidth = m_pImage.getWidth();
    int iHeight = m_pImage.getHeight();
    m_pColorGradientImage = new BufferedImage(iWidth, iHeight, BufferedImage.TYPE_INT_RGB);
    Graphics2D pGraphics = m_pColorGradientImage.createGraphics();
    pGraphics.drawImage(m_pImage, 0, 0, null);
    
    
    //2. add the color gradient
    if (m_bTransitionTop) {
      System.out.println("Transition top");
      int iStartHeight = m_iTransitionLength;
      int iEndHeight = 0;
      createVerticalTransition(iStartHeight, iEndHeight);
    }
    if (m_bTransitionBottom) {
      System.out.println("Transition bottom");
      int iStartHeight = iHeight-m_iTransitionLength;
      int iEndHeight = iHeight;
      createVerticalTransition(iStartHeight, iEndHeight);
    }
    
    
    
    
    
    
    
    return m_pColorGradientImage;
    
    
    
  }
  
  /////////////////////////////////////////
  // PRIVATE HELPER METHODS              //
  /////////////////////////////////////////
  private void createVerticalTransition(int iStartHeight, int iEndHeight) {
  
    double dLambda = 0;
    double dLambdaIncrease = 1/(double)(iEndHeight-iStartHeight);
    int iYIncrease = 1;
    if (iEndHeight < iStartHeight) {
      dLambdaIncrease *= -1;
      iYIncrease = -1;
    }
    
    int iRedComponent = m_pTransitionColor.getRed();
    int iGreenComponent = m_pTransitionColor.getGreen();
    int iBlueComponent = m_pTransitionColor.getBlue();
    
    int iImageHeight = m_pColorGradientImage.getHeight();
    int iImageWidth = m_pColorGradientImage.getWidth();
  
    int y = iStartHeight;
    boolean bNotFinished = true;
    
    while (bNotFinished) {
    
      if (y == iEndHeight) bNotFinished = false; //when the end height is reached, stop after this loop
    
      dLambda += dLambdaIncrease;
      
      if ((y < 0) || (y >= iImageHeight)) {
        y += iYIncrease;
        continue;
      }
      
      for (int x = 0; x < iImageWidth; x++) {
        Color pCol = new Color(m_pColorGradientImage.getRGB(x, y));
        
        int iRed = (int)((1-dLambda)*pCol.getRed() + dLambda*iRedComponent);//50
        int iGreen = (int)((1-dLambda)*pCol.getGreen() + dLambda*iGreenComponent);//50
        int iBlue = (int)((1-dLambda)*pCol.getBlue() + dLambda*iBlueComponent);//255
        
        if (iRed < 0) iRed = 0;
        if (iRed > 255) iRed = 255;
        if (iGreen < 0) iGreen = 0;
        if (iGreen > 255) iGreen = 255;
        if (iBlue < 0) iBlue = 0;
        if (iBlue > 255) iBlue = 255;
        
        m_pColorGradientImage.setRGB(x, y, (new Color(iRed, iGreen, iBlue)).getRGB());
      }
      
      
      y += iYIncrease;
    } 
  
  
    for (int i = iStartHeight; i != iEndHeight; i += iYIncrease) {
      
      
      if (i < 0) continue;
      if (i >= iImageHeight) continue;
      
      
    }
  }
  
  
  
  private void fetchParameters() {
    
    m_pImage = (BufferedImage)m_pParameters.getObjectParameterValue(PARAM_OBJECT_IMAGE);
    m_pTransitionColor = m_pParameters.getColorParameterValue(PARAM_COLOR_TRANSITION_COLOR);
    m_iTransitionLength = m_pParameters.getIntegerParameterValue(PARAM_INT_TRANSITION_LENGTH);
    
    m_bTransitionTop = m_pParameters.getBooleanParameterValue(PARAM_BOOL_TRANSITION_TOP);
    m_bTransitionBottom = m_pParameters.getBooleanParameterValue(PARAM_BOOL_TRANSITION_BOTTOM);
    
    
    
  }
  
  /*private void compareAndSetIntegerParameters(String pIteratorParameterName, String pMandelbrotSetParameterName) {
    int iOldValue = m_pIteratorParameters.getIntegerParameterValue(pIteratorParameterName);
    int iNewValue = m_pParameters.getIntegerParameterValue(pMandelbrotSetParameterName);
    if (iOldValue != iNewValue) {
      m_pIteratorParameters.setIntegerParameterValue(pIteratorParameterName, iNewValue);
      m_bRecalculateRaster = true;
    }
  }*/
  
  
  
  

}
