package drawing;

/*
 *  Copyright (C) 2014 Juergen Platzer 
 * 
 *  This file is part of JFXLibrary.
 *  
 *  JFXLibrary is free software: you can redistribute it and/or modify
 *  it under the terms of the GNU Lesser General Public License as published by
 *  the Free Software Foundation, either version 3 of the License, or
 *  (at your option) any later version.
 *  
 *  JFXLibrary is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU Lesser General Public License for more details.
 *  
 *  You should have received a copy of the GNU Lesser General Public License
 *  along with JFXLibrary.  If not, see <http://www.gnu.org/licenses/>.
 */

//java imports
import java.util.Hashtable;

import java.awt.Color;

//library imports
import standard.EffectParameters;
import standard.BooleanParameter;
import standard.IntegerParameter;
import standard.DoubleParameter;
import standard.ColorParameter;



/**
 *   Encapsulates the parameters that can be specified for the Network effect. 
 **/
public class NetworkParameters extends EffectParameters {

  /*** PUBLIC PARAMETER KEYS ***/

  /*** KEY for the integer parameter: number of points  ***/
  public static final String PARAM_INT_POINT_NUM = "POINT_NUM";
  
  /*** KEY for the integer parameter: maximum number of starting connections per point  ***/
  public static final String PARAM_INT_MAX_CONNECTION_NUM = "MAX_CONNECTION_NUM";
  
  /*** KEY for the integer parameter: maximum size of the points  ***/
  public static final String PARAM_INT_MAX_POINT_DRAWING_SIZE = "MAX_POINT_DRAWING_SIZE";
  
  
  /*** KEY for the integer parameter: width ***/
  public static final String PARAM_INT_WIDTH = "WIDTH";

  /*** KEY for the integer parameter: height ***/
  public static final String PARAM_INT_HEIGHT = "HEIGHT";
  
  /*** KEY for the integer parameter: point intend ***/
  public static final String PARAM_INT_POINT_INTEND = "POINT_INTEND";
  
  /*** KEY for the double parameter: Point check percentage ***/
  public static final String PARAM_DOUBLE_POINT_CHECK_PERCENTAGE = "POINT_CHECK_PERCENTAGE ";


  /*** KEY for the color parameter: line color ***/
  public static final String PARAM_COLOR_LINE_COLOR = "LINE_COLOR";
  
  /*** KEY for the color parameter: background color ***/
  public static final String PARAM_COLOR_BACKGROUND_COLOR = "BACKGROUND_COLOR";
  
  /*** KEY for the color parameter: point color 1 ***/
  public static final String PARAM_COLOR_POINT_COLOR1 = "POINT_COLOR1";
  
  /*** KEY for the color parameter: point color 2 ***/
  public static final String PARAM_COLOR_POINT_COLOR2 = "POINT_COLOR2";
  
  /*** KEY for the color parameter: point color 3 ***/
  public static final String PARAM_COLOR_POINT_COLOR3 = "POINT_COLOR3";
  
  /*** KEY for the color parameter: point color 4 ***/
  public static final String PARAM_COLOR_POINT_COLOR4 = "POINT_COLOR4";
  



  /***
   *   Creates a default parameter set for the Network effect. The following settings are used:
   *
   *   <ul>
   *     <li>An image of 400 x 400 pixels is created.</li>
   *     <li>200 points are drawn.</li>
   *     <li>The maximum drawing size of the point is set to 30 pixels.</li>
   *     <li>The point positions have a minimum distance of 10 pixels to the image borders.</li>
   *     <li>Per point a maximum starting connection number of 10 is set.</li>
   *     <li>The background color is white.</li>
   *     <li>The line color of the connections is set to black with opacity 0.1.</li>
   *     <li>The four point colors are set to black, blue, rgb(170, 0, 170) and red.</li>
   *     <li>40 percents of the points are checked for the nearest connection</li>
   *   </ul>
   *
   **/
  public NetworkParameters() {

    //1. create the parameters and set them
    setBooleanParameters(createBooleanParameters());
    setIntegerParameters(createIntegerParameters());
    setDoubleParameters(createDoubleParameters());
    setColorParameters(createColorParameters());

  }


  ////////////////////////////////////////////
  // PRIVATE METHODS FOR PARAMETER CREATION //
  ////////////////////////////////////////////
  private Hashtable<String, BooleanParameter> createBooleanParameters() {

    //1. create the hashtable
    Hashtable<String, BooleanParameter> pParameters = new Hashtable<String, BooleanParameter>();

    //2. fill the hashtable
    
    //3. return the hashtable
    return pParameters;
  }




  private Hashtable<String, IntegerParameter> createIntegerParameters() {
    //1. create the hashtable
    Hashtable<String, IntegerParameter> pParameters = new Hashtable<String, IntegerParameter>();

    //2. fill the hashtable
    pParameters.put(PARAM_INT_POINT_NUM, new IntegerParameter(2, 200, 10000));
    pParameters.put(PARAM_INT_MAX_CONNECTION_NUM, new IntegerParameter(1, 10, 1000));
    pParameters.put(PARAM_INT_MAX_POINT_DRAWING_SIZE, new IntegerParameter(30, 1, 100));
    pParameters.put(PARAM_INT_WIDTH, new IntegerParameter(400, 10, 10000));
    pParameters.put(PARAM_INT_HEIGHT, new IntegerParameter(400, 10, 10000));
    pParameters.put(PARAM_INT_POINT_INTEND, new IntegerParameter(10, 0, 100));
    
    //3. return the hashtable
    return pParameters;
  }
  
  
  private Hashtable<String, ColorParameter> createColorParameters() {
    //1. create the hashtable
    Hashtable<String, ColorParameter> pParameters = new Hashtable<String, ColorParameter>();

    //2. fill the hashtable
    pParameters.put(PARAM_COLOR_LINE_COLOR, new ColorParameter(new Color(0.0f, 0.0f, 0.0f, 0.1f)));
    pParameters.put(PARAM_COLOR_BACKGROUND_COLOR, new ColorParameter(Color.WHITE));
    pParameters.put(PARAM_COLOR_POINT_COLOR1, new ColorParameter(Color.BLACK));
    pParameters.put(PARAM_COLOR_POINT_COLOR2, new ColorParameter(Color.BLUE));
    pParameters.put(PARAM_COLOR_POINT_COLOR3, new ColorParameter(new Color(170, 0, 170)));
    pParameters.put(PARAM_COLOR_POINT_COLOR4, new ColorParameter(Color.RED));
    
    //3. return the hashtable
    return pParameters;
  
  }
  
  private Hashtable<String, DoubleParameter> createDoubleParameters() {
    //1. create the hashtable
    Hashtable<String, DoubleParameter> pParameters = new Hashtable<String, DoubleParameter>();

    //2. fill the hashtable
    pParameters.put(PARAM_DOUBLE_POINT_CHECK_PERCENTAGE, new DoubleParameter(0, 0.4, 1));
    
    //3. return the hashtable
    return pParameters;
  }
  



}
