package drawing;

/*
 *  Copyright (C) 2014 Juergen Platzer 
 * 
 *  This file is part of JFXLibrary.
 *  
 *  JFXLibrary is free software: you can redistribute it and/or modify
 *  it under the terms of the GNU Lesser General Public License as published by
 *  the Free Software Foundation, either version 3 of the License, or
 *  (at your option) any later version.
 *  
 *  JFXLibrary is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU Lesser General Public License for more details.
 *  
 *  You should have received a copy of the GNU Lesser General Public License
 *  along with JFXLibrary.  If not, see <http://www.gnu.org/licenses/>.
 */

//java imports
import java.util.Hashtable;

import java.awt.Color;

//library imports
import standard.EffectParameters;
import standard.BooleanParameter;
import standard.IntegerParameter;
import standard.ColorParameter;



/**
 *   Encapsulates the parameters that can be specified for the Stars effect. 
 **/
public class StarsParameters extends EffectParameters {

  /*** PUBLIC PARAMETER KEYS ***/

  /*** KEY for the integer parameter: number of spikes  ***/
  public static final String PARAM_INT_SPIKE_NUM = "SPIKE_NUM";
  
  /*** KEY for the integer parameter: length of spikes  ***/
  public static final String PARAM_INT_SPIKE_LENGTH = "SPIKE_LENGTH";

  /*** KEY for the integer parameter: number of lines ***/
  public static final String PARAM_INT_LINE_NUM = "LINE_NUM";
  
  /*** KEY for the integer parameter: horizontal translation ***/
  public static final String PARAM_INT_HORIZONTAL_TRANSLATION = "HORIZONTAL_TRANSLATION";
  
  /*** KEY for the integer parameter: horizontal translation ***/
  public static final String PARAM_INT_VERTICAL_TRANSLATION = "VERTICAL_TRANSLATION";
  
  /*** KEY for the integer parameter: rotation ***/
  public static final String PARAM_INT_ROTATION = "ROTATION";
  
  /*** KEY for the integer parameter: width ***/
  public static final String PARAM_INT_WIDTH = "WIDTH";

  /*** KEY for the integer parameter: height ***/
  public static final String PARAM_INT_HEIGHT = "HEIGHT";


  /*** KEY for the color parameter: line color ***/
  public static final String PARAM_COLOR_LINE_COLOR = "LINE_COLOR";
  
  /*** KEY for the color parameter: background color ***/
  public static final String PARAM_COLOR_BACKGROUND = "BACKGROUND";
  



  /***
   *   Creates a default parameter set for the stars effect. The following settings are used:
   *
   *   <ul>
   *     <li>An image of 400 x 400 pixels is created.</li>
   *     <li>4 spikes are drawn.</li>
   *     <li>The spike length is set to 180 pixels.</li>
   *     <li>Per spike 10 lines are drawn.</li>
   *     <li>There is no rotation, no vertical and no horizontal translation.</li>
   *     <li>The line color is set to black.</li>
   *     <li>The background color is set to white.</li>
   *     <li>There is no parameter for the background so far. (TO DO!!!)</li>
   *   </ul>
   *
   **/
  public StarsParameters() {

    //1. create the parameters and set them
    setBooleanParameters(createBooleanParameters());
    setIntegerParameters(createIntegerParameters());
    setColorParameters(createColorParameters());

  }


  ////////////////////////////////////////////
  // PRIVATE METHODS FOR PARAMETER CREATION //
  ////////////////////////////////////////////
  private Hashtable<String, BooleanParameter> createBooleanParameters() {

    //1. create the hashtable
    Hashtable<String, BooleanParameter> pParameters = new Hashtable<String, BooleanParameter>();

    //2. fill the hashtable
    
    //3. return the hashtable
    return pParameters;
  }




  private Hashtable<String, IntegerParameter> createIntegerParameters() {
    //1. create the hashtable
    Hashtable<String, IntegerParameter> pParameters = new Hashtable<String, IntegerParameter>();

    //2. fill the hashtable
    pParameters.put(PARAM_INT_SPIKE_NUM, new IntegerParameter(4, 3, 255));
    pParameters.put(PARAM_INT_SPIKE_LENGTH, new IntegerParameter(180, 10, 10000));
    pParameters.put(PARAM_INT_LINE_NUM, new IntegerParameter(10, 1, 1000));
    pParameters.put(PARAM_INT_HORIZONTAL_TRANSLATION, new IntegerParameter(0, -1000, 1000));
    pParameters.put(PARAM_INT_VERTICAL_TRANSLATION, new IntegerParameter(0, -1000, 1000));
    pParameters.put(PARAM_INT_ROTATION, new IntegerParameter(0, 0, 360));
    pParameters.put(PARAM_INT_WIDTH, new IntegerParameter(400, 10, 10000));
    pParameters.put(PARAM_INT_HEIGHT, new IntegerParameter(400, 10, 10000));
    
    //3. return the hashtable
    return pParameters;
  }
  
  
  private Hashtable<String, ColorParameter> createColorParameters() {
    //1. create the hashtable
    Hashtable<String, ColorParameter> pParameters = new Hashtable<String, ColorParameter>();

    //2. fill the hashtable
    pParameters.put(PARAM_COLOR_LINE_COLOR, new ColorParameter(Color.BLACK));
    pParameters.put(PARAM_COLOR_BACKGROUND, new ColorParameter(Color.WHITE));
    
    //3. return the hashtable
    return pParameters;
  
  }



}
