package fractals;

/*
 *  Copyright (C) 2014 Juergen Platzer 
 * 
 *  This file is part of JFXLibrary.
 *  
 *  JFXLibrary is free software: you can redistribute it and/or modify
 *  it under the terms of the GNU Lesser General Public License as published by
 *  the Free Software Foundation, either version 3 of the License, or
 *  (at your option) any later version.
 *  
 *  JFXLibrary is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU Lesser General Public License for more details.
 *  
 *  You should have received a copy of the GNU Lesser General Public License
 *  along with JFXLibrary.  If not, see <http://www.gnu.org/licenses/>.
 */

//standard imports
import java.awt.Color;
import java.awt.image.BufferedImage;

//library imports
import standard.ImageEffect;
import standard.EffectParameters;
import util.color.ColorGradient;
import util.math.ComplexNumber;

import static fractals.MandelbrotSetParameters.*;

/**
 *  Implements the Mandelbrot set effect.
 **/
public class MandelbrotSet extends ImageEffect {

  /*** private member variables ***/
  private MandelbrotSetParameters m_pParameters;
  private IteratorParameters m_pIteratorParameters;
  private Iterator m_pIterator;
  private Raster m_pRaster;
  private boolean m_bFetchParameters;
  private boolean m_bRecalculateRaster;

  
  //////////////////////////////
  // CONSTRUCTOR              //
  //////////////////////////////
  /**
   *   Creates a {@code MandelbrotSet} object with the default parameters.
   *
   **/
  public MandelbrotSet() {
    this(new MandelbrotSetParameters());
  }
  

  /**
   *   Creates a {@code MandelbrotSet} object that draws a part of the mandelbrot set according to the specified parameters.
   *
   *   @param pParameters The parameters that configure the Mandelbrot set image.
   **/
  public MandelbrotSet(MandelbrotSetParameters pParameters) {
    //1. Make an initial check
    if (pParameters == null) pParameters = new MandelbrotSetParameters();
    
    //2. set the member variables
    m_pParameters = pParameters;
    m_pIteratorParameters = null;
    m_pIterator = null;
    m_pRaster = null;
    m_bFetchParameters = true;
    m_bRecalculateRaster = true;
  }
  
  
  //////////////////////////////
  // SET METHODS              //
  //////////////////////////////
  /**
   *   Sets the parameters for the Mandelbrot set effect. 
   *
   *   @param pParameters The parameters that configure the effect. 
   **/
  public void setMandelbrotSetParameters(MandelbrotSetParameters pParameters) {
    //1. Make an initial check
    if (pParameters == null) return;
    
    //2. set the member variables
    m_pParameters = pParameters;
    m_bFetchParameters = true; 
    
     
  }
  
  /**
   *   Sets the parameters for the Mandelbrot set effect. The parameters are only set, 
   *   if the object is of the type {@code MandelbrotSetParameters}.
   *
   *   @param pEffectParameters The parameters that configure the effect. 
   **/
  public void setEffectParameters(EffectParameters pEffectParameters) {
    if (pEffectParameters == null) return;
    
    if (pEffectParameters instanceof MandelbrotSetParameters) {
      setMandelbrotSetParameters((MandelbrotSetParameters)pEffectParameters);
    }
  }
  
  
  //////////////////////////////
  // GET METHODS              //
  //////////////////////////////
  /**
   *    Returns the complex number that it is represented by the specified position in the image.
   *    
   *    @param iXPos The horizontal position as integer.
   *    @param iYPos The vertical position as integer.
   *
   *    @return The {@code ComplexNumber} object representing the complex number of the position in the image.
   **/
  public ComplexNumber getComplexNumberAt(int iXPos, int iYPos) {
    if (m_pIterator == null) return null;
    return m_pIterator.getComplexNumberAt(iXPos, iYPos);
  }
  
  /**
   *   Returns the parameters that are currently used for the Mandelbrot set effect.
   *
   *   @return The parameters that are currently used for the effect.
   **/
  public EffectParameters getEffectParameters() {
    return m_pParameters;
  }
  
  
    
  
  //////////////////////////////
  // PROCESS METHODS          //
  //////////////////////////////
  /**
   *  Returns an image of the Mandelbrot set according to the parameters that were set.
   *
   *  @return An image of the Mandelbrot set as BufferedImage.
   *
   */
  public BufferedImage process() throws Exception {
    //1. check if the parameters have to be reset
    if (m_bFetchParameters || (m_pIteratorParameters == null) || (m_pIterator == null)) {
      fetchParameters();
    }
    
    //2. if necessary recalculate the raster
    if (m_bRecalculateRaster || (m_pRaster == null)) {
      m_pRaster = m_pIterator.getRaster(this);
      if (m_pRaster == null) return null;
      m_bRecalculateRaster = false;
    }
    
    //3. create the image
    ColorGradient pColorGradient = null;
    try {
      pColorGradient = (ColorGradient)m_pParameters.getObjectParameterValue(PARAM_OBJECT_COLOR_GRADIENT);
    }
    catch (Exception pException) {
      pColorGradient = new ColorGradient();  
    }
    
    BufferedImage pImage = m_pRaster.getImage(this, pColorGradient, Color.BLACK);
    
    
    return pImage;
    
  }
  
  /////////////////////////////////////////
  // PRIVATE HELPER METHODS              //
  /////////////////////////////////////////
  private void fetchParameters() {
    //1. make some checks
    if (m_pIteratorParameters == null) m_pIteratorParameters = new IteratorParameters();
    
    //2. pass the relevant parameters of the mandelbrot set parameters to the iterator parameters
    //   and set the flag if the raster has to be recalculated
    
    //2.1. the integer parameters
    compareAndSetIntegerParameters(IteratorParameters.PARAM_INT_ITERATION_DEPTH, PARAM_INT_ITERATION_DEPTH);
    compareAndSetIntegerParameters(IteratorParameters.PARAM_INT_WIDTH, PARAM_INT_WIDTH);
    compareAndSetIntegerParameters(IteratorParameters.PARAM_INT_HEIGHT, PARAM_INT_HEIGHT);
    
    //2.2. the double parameters
    compareAndSetDoubleParameters(IteratorParameters.PARAM_DOUBLE_DECISION_LIMIT, PARAM_DOUBLE_DECISION_LIMIT);
    
    //2.3. the complex number parameters
    compareAndSetComplexNumberParameters(IteratorParameters.PARAM_OBJECT_UPPER_LEFT_WINDOW_CORNER, 
    PARAM_OBJECT_UPPER_LEFT_WINDOW_CORNER);
    compareAndSetComplexNumberParameters(IteratorParameters.PARAM_OBJECT_LOWER_RIGHT_WINDOW_CORNER, 
    PARAM_OBJECT_LOWER_RIGHT_WINDOW_CORNER);
    
    m_pIteratorParameters.setObjectParameterValue(IteratorParameters.PARAM_OBJECT_COMPLEX_NUMBER_Z, 
    new ComplexNumber(0,0));
    
    m_pIteratorParameters.setObjectParameterValue(IteratorParameters.PARAM_OBJECT_COMPLEX_NUMBER_C, 
    null);
    
       
    //3. recreate the iterator if necessary
    if (m_pIterator == null) {
      m_pIterator = new Iterator(m_pIteratorParameters);
    }
    else if (m_bRecalculateRaster) {
      m_pIterator.setIteratorParameters(m_pIteratorParameters);
    }
    
    //4. set the flags
    m_bFetchParameters = false;
    
    
    
  }
  
  private void compareAndSetIntegerParameters(String pIteratorParameterName, String pMandelbrotSetParameterName) {
    int iOldValue = m_pIteratorParameters.getIntegerParameterValue(pIteratorParameterName);
    int iNewValue = m_pParameters.getIntegerParameterValue(pMandelbrotSetParameterName);
    if (iOldValue != iNewValue) {
      m_pIteratorParameters.setIntegerParameterValue(pIteratorParameterName, iNewValue);
      m_bRecalculateRaster = true;
    }
  }
  
  private void compareAndSetDoubleParameters(String pIteratorParameterName, String pMandelbrotSetParameterName) {
    double dOldValue = m_pIteratorParameters.getDoubleParameterValue(pIteratorParameterName);
    double dNewValue = m_pParameters.getDoubleParameterValue(pMandelbrotSetParameterName);
    if (dOldValue != dNewValue) {
      m_pIteratorParameters.setDoubleParameterValue(pIteratorParameterName, dNewValue);
      m_bRecalculateRaster = true;
    }
  }
  
  private void compareAndSetComplexNumberParameters(String pIteratorParameterName, String pMandelbrotSetParameterName) {
    ComplexNumber pOldValue = (ComplexNumber)m_pIteratorParameters.getObjectParameterValue(pIteratorParameterName);
    ComplexNumber pNewValue = (ComplexNumber)m_pParameters.getObjectParameterValue(pMandelbrotSetParameterName);
    
    if (!pOldValue.equals(pNewValue)) {
      m_pIteratorParameters.setObjectParameterValue(pIteratorParameterName, pNewValue);
      m_bRecalculateRaster = true;
    }
  }
  
  
  

}
