package fractals;

/*
 *  Copyright (C) 2014 Juergen Platzer 
 * 
 *  This file is part of JFXLibrary.
 *  
 *  JFXLibrary is free software: you can redistribute it and/or modify
 *  it under the terms of the GNU Lesser General Public License as published by
 *  the Free Software Foundation, either version 3 of the License, or
 *  (at your option) any later version.
 *  
 *  JFXLibrary is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU Lesser General Public License for more details.
 *  
 *  You should have received a copy of the GNU Lesser General Public License
 *  along with JFXLibrary.  If not, see <http://www.gnu.org/licenses/>.
 */

//standard imports
import java.util.Hashtable;

//library imports
import standard.EffectParameters;
import standard.DoubleParameter;
import standard.IntegerParameter;
import standard.ObjectParameter;

import util.color.ColorGradient;

import util.math.ComplexNumber;

/**
 *   Encapsulates the parameters that can be specified for the MandelbrotSet object. 
 **/
public class MandelbrotSetParameters extends EffectParameters {

  /*** PUBLIC PARAMETER KEYS ***/

  /*** KEY for the integer parameter: iteration depth  ***/
  public static final String PARAM_INT_ITERATION_DEPTH = "ITERATION_DEPTH";
  
  /*** KEY for the integer parameter: width ***/
  public static final String PARAM_INT_WIDTH = "WIDTH";

  /*** KEY for the int parameter: height ***/
  public static final String PARAM_INT_HEIGHT = "HEIGHT";
  
  /*** KEY for the double parameter: decision limit ***/
  public static final String PARAM_DOUBLE_DECISION_LIMIT = "DECISION_LIMIT";
  
  
  /*** KEY for the object parameter: left upper window corner (ComplexNumber) ***/
  public static final String PARAM_OBJECT_UPPER_LEFT_WINDOW_CORNER = "UPPER_LEFT_WINDOW_CORNER";
  
  /*** KEY for the object parameter: right lower window corner (ComplexNumber) ***/
  public static final String PARAM_OBJECT_LOWER_RIGHT_WINDOW_CORNER = "LOWER_RIGHT_WINDOW_CORNER";

  /*** KEY for the object parameter: right lower window corner (ColorGradient) ***/
  public static final String PARAM_OBJECT_COLOR_GRADIENT = "COLOR_GRADIENT";
    
   /***
   *   Creates a default parameter set for the fractals iterator so that it create a Mandelbrot set. The following settings are used:
   *
   *   <ul>
   *     <li>iteration depth: 100</li>
   *     <li>width x height: 400 x 400</li>
   *     <li>decision limit: 2</li>
   *     <li>upper left window corner: (-2, 1.5)</li>
   *     <li>lower right window corner: (1, -1.5)</li>
   *     <li>z: (0,0)</li>
   *     <li>Color gradient: From White to Black</li>
   *   </ul>
   *
   **/
  public MandelbrotSetParameters() {

    //1. create the parameters and set them
    setIntegerParameters(createIntegerParameters());
    setDoubleParameters(createDoubleParameters());
    setObjectParameters(createObjectParameters());

  }
  
  
  ////////////////////////////////////////////
  // PRIVATE METHODS FOR PARAMETER CREATION //
  ////////////////////////////////////////////
  private Hashtable<String, IntegerParameter> createIntegerParameters() {
    //1. create the hashtable
    Hashtable<String, IntegerParameter> pParameters = new Hashtable<String, IntegerParameter>();

    //2. fill the hashtable
    pParameters.put(PARAM_INT_ITERATION_DEPTH, new IntegerParameter(100, 5, 50000));
    pParameters.put(PARAM_INT_WIDTH, new IntegerParameter(400, 2, 50000));
    pParameters.put(PARAM_INT_HEIGHT, new IntegerParameter(400, 2, 50000));
    
    //3. return the hashtable
    return pParameters;
  }
  
  
  private Hashtable<String, DoubleParameter> createDoubleParameters() {
    //1. create the hashtable
    Hashtable<String, DoubleParameter> pParameters = new Hashtable<String, DoubleParameter>();

    //2. fill the hashtable
    pParameters.put(PARAM_DOUBLE_DECISION_LIMIT, new DoubleParameter(2, 0.01, 100));
    
    //3. return the hashtable
    return pParameters;
  }
  
  private Hashtable<String, ObjectParameter> createObjectParameters() {
    //1. create the hashtable
    Hashtable<String, ObjectParameter> pParameters = new Hashtable<String, ObjectParameter>();

    //2. fill the hashtable
    pParameters.put(PARAM_OBJECT_UPPER_LEFT_WINDOW_CORNER, new ObjectParameter(new ComplexNumber(-2, 1.5)));
    pParameters.put(PARAM_OBJECT_LOWER_RIGHT_WINDOW_CORNER, new ObjectParameter(new ComplexNumber(1, -1.5)));
    pParameters.put(PARAM_OBJECT_COLOR_GRADIENT, new ObjectParameter(new ColorGradient()));
  
    //3. return the hashtable
    return pParameters;
  }
  
}
  
  
  
