package fractals;

/*
 *  Copyright (C) 2014 Juergen Platzer 
 * 
 *  This file is part of JFXLibrary.
 *  
 *  JFXLibrary is free software: you can redistribute it and/or modify
 *  it under the terms of the GNU Lesser General Public License as published by
 *  the Free Software Foundation, either version 3 of the License, or
 *  (at your option) any later version.
 *  
 *  JFXLibrary is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU Lesser General Public License for more details.
 *  
 *  You should have received a copy of the GNU Lesser General Public License
 *  along with JFXLibrary.  If not, see <http://www.gnu.org/licenses/>.
 */

import java.awt.image.BufferedImage;
import java.awt.Color;

//library imports
import standard.ImageEffect;
import util.color.ColorGradient;

/**
 * Entity that encapsulates the data of an 2D integer array that was calculated by an Iterator and provides default functionalities for it.
 **/
public class Raster {

  /*** member variables ***/
  private int[][] m_pRasterValues;
  private int m_iMinimum;
  private int m_iMaximum;

  
  
  ///////////////////////////////
  // CONSTRUCTOR               //
  ///////////////////////////////
  /**
   *    Creates a two dimensional Raster object with the raster values. The minimum and maximum 
   *    properties are calculated from the raster values.
   *
   *    @param pRasterValues The raster values as two dimensional integer array.
   **/
  public Raster(int[][] pRasterValues) {
    m_pRasterValues = pRasterValues;
    recalculateMembers();
  }
  
  /**
   *    Creates a two dimensional Raster object with the raster values and the already known minimum and maximum 
   *    properties.
   *
   *    @param pRasterValues The raster values as two dimensional integer array.
   *    @param iMinValue The minimum integer value in the raster.
   *    @param iMaxValue The maximum integer value in the raster.
   **/
  public Raster(int[][] pRasterValues, int iMinValue, int iMaxValue) {
    //set the members
    m_pRasterValues = pRasterValues;
    
    if (iMinValue > iMaxValue) {
      m_iMinimum = iMaxValue;
      m_iMaximum = iMinValue;
    }
    else {
      m_iMaximum = iMaxValue;
      m_iMinimum = iMinValue;
    }
  }

  ////////////////////////////////
  // GET METHODS                //
  ////////////////////////////////
  /**
   *  Returns an image calculated from the raster values by using a color gradiant to map colors to the raster values.
   *
   *  @param pImageEffect The image effect that uses the Raster to create the effect image.
   *  @param pColorGradient The color gradient object that is used to map the integer values of the raster to a color.
   *  @param pUpperBorderColor The color object that is used for the maximum value in the raster. (This is needed for example to make the Mandelbrot set appear black, while its border still has a different color.)
   *
   *  @return An image of the raster object as {@code BufferedImage}.
   *
   */
  public BufferedImage getImage(ImageEffect pImageEffect, ColorGradient pColorGradient, Color pUpperBorderColor) {
    //1. make some initial checks
    if (pImageEffect == null) return null;
    if (pColorGradient == null) return null;
    
    int iWidth = getWidth();
    int iHeight = getHeight();
    if ((iWidth == 0) || (iHeight == 0)) return null;


    //2. fetch the lookuptable from the color gradient
    int[] pLookupTable = pColorGradient.getRGBLookupTable(m_iMaximum+1);
    if (pUpperBorderColor != null) pLookupTable[m_iMaximum] = pUpperBorderColor.getRGB();
    
    //3. create the image
    BufferedImage pImage = new BufferedImage(iWidth, iHeight, BufferedImage.TYPE_INT_RGB);
    for (int x = 0; x < iWidth; x++) {    
      for (int y = 0; y < iHeight; y++) {
        pImage.setRGB(x, y, pLookupTable[m_pRasterValues[x][y]]);
      }
      if (!pImageEffect.isRunning()) return null;
    }
    
    return pImage;
  }
  
  /**
   *  Returns the raster values as two dimensional Integer array.
   *
   *  @return The raster values as two dimensional Integer array.
   *
   */
  public int[][] getRasterValues() {
    return m_pRasterValues;
  }
  
  /**
   *  Returns the minimum raster value as integer.
   *
   *  @return The minimum raster value as integer.
   *
   */
  public int getMinimum() {
    return m_iMinimum;
  }
  
  /**
   *  Returns the maximum raster value as integer.
   *
   *  @return The maximum raster value as integer.
   *
   */
  public int getMaximum() {
    return m_iMaximum;
  }
  
  /**
   *  Returns the width of the raster.
   *
   *  @return The width of the raster as integer.
   *
   */
  public int getWidth() {
    if (m_pRasterValues == null) return 0;
    return m_pRasterValues.length;
  }
  
  /**
   *  Returns the height of the raster.
   *
   *  @return The height of the raster as integer.
   *
   */
  public int getHeight() {
    if (m_pRasterValues == null) return 0;
    if (m_pRasterValues.length == 0) return 0;
    return m_pRasterValues[0].length;
  }
  
  ///////////////////////////////
  // PRIVATE HELPER METHODS    //
  ///////////////////////////////
  private void recalculateMembers() {
    //1. make initial checks
    int iWidth = getWidth();
    int iHeight = getHeight();
    if ((iWidth == 0) || (iHeight == 0)) {
      m_iMinimum = 0;
      m_iMaximum = 0;
      return;
    }
    
    //2. calculate minimum and maximum
    int iMinimum = m_pRasterValues[0][0];
    int iMaximum = iMinimum;
    
    for (int x = 0; x < iWidth; x++) {
      for (int y = 0; y < iHeight; y++) {
        int iValue = m_pRasterValues[x][y];
        if (iValue < iMinimum) iMinimum = iValue;
        if (iValue > iMaximum) iMaximum = iValue;
      }
    }
    
    m_iMinimum = iMinimum;
    m_iMaximum = iMaximum;
  }

}
