package gui;

/*
 *  Copyright (C) 2014 Juergen Platzer 
 * 
 *  This file is part of JFXLibrary.
 *  
 *  JFXLibrary is free software: you can redistribute it and/or modify
 *  it under the terms of the GNU Lesser General Public License as published by
 *  the Free Software Foundation, either version 3 of the License, or
 *  (at your option) any later version.
 *  
 *  JFXLibrary is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU Lesser General Public License for more details.
 *  
 *  You should have received a copy of the GNU Lesser General Public License
 *  along with JFXLibrary.  If not, see <http://www.gnu.org/licenses/>.
 */

//gui imports
import javax.swing.JPanel;
import java.awt.image.BufferedImage;
import java.awt.Graphics2D;
import java.awt.Graphics;
import java.awt.Dimension;


//library imports
import standard.EffectParameters;

import applications.Settings;
import applications.ISettingsListener;

/**
 *  Represents a panel that shows the result of an image effect in the gui application.
 *
 */
public abstract class AbstractImageEffectPanel extends JPanel implements IPropertiesPanelListener, ISettingsListener {

  
  
  
  ////////////////////////////////
  // SET METHODS                //
  ////////////////////////////////
  /**
   *   Sets the parameters for the image effect that is shown in the panel. The parameters are only set, 
   *   if the object is of the correct parameter type.
   *
   *   @param pEffectParameters The parameters that configure the image effect. 
   **/
  public abstract void setEffectParameters(EffectParameters pEffectParameters);
  
  
  /**
   *   Sets the image size of the result image of the image effect.
   *
   *   @param iWidth The width in pixels of the result image.
   *   @param iHeight The width in pixels of the result image.
   **/
  public abstract void setEffectSize(int iWidth, int iHeight);
  
  
  ////////////////////////////////
  // GET METHODS                //
  ////////////////////////////////
  /**
   *   Returns the parameters that are currently used by the effect that is shown in this panel.
   *
   *   @return The parameters that are currently used for the effect that is shown in this panel.
   **/
  public abstract EffectParameters getEffectParameters();
  
  /**
   *   Returns the image that is created by the image effect that is shown in this panel.
   *
   *   @return The image that is created by the image effect that is shown in this panel.
   **/
  public abstract BufferedImage getEffectImage();
  
  /**
   *   Returns the properties panel that is appropriate for the image effect that is shown in this panel.
   *
   *   @return The properties panel that is appropriate for the image effect that is shown in this panel.
   **/
  public abstract AbstractPropertiesPanel getPropertiesPanel();
  
  
  
  
  //////////////////////////////////////
  // IPropertiesPanelListener METHODS //
  //////////////////////////////////////
  /**
   *  If a properties panel has adapted the image effect parameters those paremeters are passed on to the 
   *  image effect and a repainting is initiated.
   *
   *  @param pEffectParameters The effect parameters that have been sent by the properties panel.
   **/
  public void applyChanges(EffectParameters pEffectParameters) {
    //System.out.println("test");
    setEffectParameters(pEffectParameters);
    repaint();
  }
  
  
  //////////////////////////////// 
  // SETTINGS LISTENER METHODS  // 
  //////////////////////////////// 
  /**
   *  If the settings for the gui application have been changed, the changed settings are sent.
   *
   *  @param pSettings The modified settings of the gui application.
   **/
  public void applyNewSettings(Settings pSettings) { 
  
  }

  
  
 
}