package gui;

/*
 *  Copyright (C) 2014 Juergen Platzer 
 * 
 *  This file is part of JFXLibrary.
 *  
 *  JFXLibrary is free software: you can redistribute it and/or modify
 *  it under the terms of the GNU Lesser General Public License as published by
 *  the Free Software Foundation, either version 3 of the License, or
 *  (at your option) any later version.
 *  
 *  JFXLibrary is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU Lesser General Public License for more details.
 *  
 *  You should have received a copy of the GNU Lesser General Public License
 *  along with JFXLibrary.  If not, see <http://www.gnu.org/licenses/>.
 */

//gui imports
import javax.swing.JPanel;
import java.awt.image.BufferedImage;
import java.awt.Graphics2D;
import java.awt.Graphics;
import java.awt.Dimension;


//library imports
import standard.EffectParameters;
import standard.IImageEffectListener;


/**
 *  Represents a panel that shows the result of an image effect in the gui application and which allows interaction with the mouse.
 *
 */
public abstract class AbstractInteractionImageEffectPanel extends AbstractImageEffectPanel
  implements IImageEffectListener {

  /*** member variables ***/
  private int m_iWidth = 0;
  private int m_iHeight = 0;
  private BufferedImage m_pImage = null;
  
  
  
  
  
  //////////////////////////////////
  // DRAWING METHODS              //
  //////////////////////////////////
  /**
   *  Paints the content of the panel.
   *
   *  @param g The graphics context that is used to perform the drawings.
   **/
  protected void paintComponent(Graphics g) {
    //1. convert the graphics context
    Graphics2D pGraphics = (Graphics2D)g;  

    //2. check if the size has been changed
    Dimension pSize = getSize();
    boolean bSizeChanged = false;
    if (m_iWidth != pSize.width) {
      m_iWidth = pSize.width;
      bSizeChanged = true;
    }
    if (m_iHeight != pSize.height) {
      m_iHeight = pSize.height;
      bSizeChanged = true;
    }
    
    if (bSizeChanged) setEffectSize(m_iWidth, m_iHeight); //must init a recalculation of the image
    
    //System.out.println(m_iWidth + " x " + m_iHeight);
    
    //2. draw the effect
    try {
      pGraphics.clearRect(0, 0, m_iWidth, m_iHeight);
      int iXOffset = getImageXOffset();
      int iYOffset = getImageYOffset();
      //BufferedImage pImage = getEffectImage();
      //TO DO: center the image, if it is smaller than the panel. but what to do if the image is bigger -> scrolling
      pGraphics.drawImage(m_pImage, iXOffset, iYOffset, null);
    }
    catch (Exception pException) {
      pException.printStackTrace();
    }

    //3. draw the interaction if necessary
    drawInteraction(pGraphics);
  }
  
  ///////////////////////////
  // PROTECTED METHODS     //
  ///////////////////////////  
  /**
   *  Draws the interaction graphics on top of the result image. 
   *
   *  @param pGraphics The graphics context that is used to perform the drawings.
   **/
  protected abstract void drawInteraction(Graphics2D pGraphics);
  
  
  ///////////////////////////
  // IIMAGE EFFECT METHODS //
  ///////////////////////////
  /**
   *   The only method how the image that should be shown can be set. It triggers a repaint.
   *
   *   @param pImage    The rendered image.
   **/
  public void setEffectImage(BufferedImage pImage) {
    m_pImage = pImage;
    repaint();
  }
  
  
  ////////////////////////////
  // GET METHODS            //
  ////////////////////////////
  /**
   *   Returns the image that is created by the image effect that is shown in this panel.
   *
   *   @return The image that is created by the image effect that is shown in this panel.
   **/
  public BufferedImage getEffectImage() {
    return m_pImage;
  }
  
  /**
   *   Returns the horizontal offset of the image.
   *
   *   @return The horizontal offset of the image.
   **/
  public int getImageXOffset() {
    if (m_pImage == null) return 0;
    return (getWidth()-m_pImage.getWidth())/2;
  }
  
  /**
   *   Returns the vertical offset of the image.
   *
   *   @return The vertical offset of the image.
   **/
  public int getImageYOffset() {
    if (m_pImage == null) return 0;
    return (getHeight()-m_pImage.getHeight())/2;
  }
  
  
  

  
  
 
}