package gui;

/*
 *  Copyright (C) 2014 Juergen Platzer 
 * 
 *  This file is part of JFXLibrary.
 *  
 *  JFXLibrary is free software: you can redistribute it and/or modify
 *  it under the terms of the GNU Lesser General Public License as published by
 *  the Free Software Foundation, either version 3 of the License, or
 *  (at your option) any later version.
 *  
 *  JFXLibrary is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU Lesser General Public License for more details.
 *  
 *  You should have received a copy of the GNU Lesser General Public License
 *  along with JFXLibrary.  If not, see <http://www.gnu.org/licenses/>.
 */

import javax.swing.JPanel;
import javax.swing.JLabel;
import javax.swing.JComboBox;

import java.awt.FlowLayout;
import java.awt.Dimension;

import java.awt.event.ActionListener;
import java.awt.event.ActionEvent;

import java.util.Vector;

/**
 *  A panel with a label for an effect property and {@code ComboBox} managing the value of this effect property.
 **/
public class ComboBoxPanel extends JPanel implements ActionListener {
  
  /** private gui member variables **/
  private JLabel m_pLabel;
  private JComboBox<String> m_pComboBox;
  
  /** private listener variables **/
  Vector<IComboBoxPanelListener> m_pListeners = null;
  
  //////////////////////////
  // CONSTRUCTOR          //
  //////////////////////////
  /**
   *  Creates the {@code ComboBox} with a label text, the possible values for the {@code ComboBox} and the selected index to 
   *  indicate the current value of the {@code ComboBox}.
   *
   *  @param pText The text of the label. 
   *  @param pValueLabels The list of values for the {@code ComboBox}.
   *  @param iSelectedIndex The index of the currently selected value of the {@code ComboBox}.
   **/
  public ComboBoxPanel(String pText, Vector<String> pValueLabels, int iSelectedIndex) {
    setLayout(new FlowLayout());
  
    m_pLabel = new JLabel(pText);
    add(m_pLabel);
    
    m_pComboBox = new JComboBox<String>(pValueLabels);
    m_pComboBox.setSelectedIndex(iSelectedIndex);
    m_pComboBox.addActionListener(this);
    add(m_pComboBox);
    
    //setMaximumSize(new Dimension(300, 40));
    //setPreferredSize(new Dimension(200, 30));
    
  
  }
  
  ////////////////////////////
  // GET METHODS            //
  ////////////////////////////
  /**
   *  Returns the selected index of the {@code ComboBox}.
   *
   *  @return The selected index of the {@code ComboBox}.
   **/
  public int getSelectedIndex() {
    return m_pComboBox.getSelectedIndex();
  }
  
  /**
   *  Returns the selected value of the {@code ComboBox} as String.
   *
   *  @return The selected value of the {@code ComboBox} as String.
   **/
  public String getSelectedItem() {
    return m_pComboBox.getSelectedItem().toString();
  }
  
  
  ///////////////////////////
  // SET METHODS           //
  ///////////////////////////
  /**
   *  Sets the text of the label. 
   *
   *  @param pText The new text of the label as String.
   **/
  public void setText(String pText) {
    m_pLabel.setText(pText);
  }
  
  /**
   *  Sets a new list of values for the {@code ComboBox}. This is necessary for example, when the language of the GUI 
   *  application has been changed.
   *
   *  @param pValueLabels A new list of values for the {@code ComboBox}.
   **/
  public void setValueLabels(Vector<String> pValueLabels) {
    //1. make some checks
    if (pValueLabels == null) return;
    if (pValueLabels.size() == 0) return;
    
    //2. fetch the selected index and delete the entries
    int iSelectedIndex = m_pComboBox.getSelectedIndex();
    m_pComboBox.removeAllItems();
    
    
    //3. fill the combobox and set the index
    int iLabelNum = pValueLabels.size();
    for (int i = 0; i < iLabelNum; i++) {
      m_pComboBox.addItem(pValueLabels.get(i));
    }
    
    if (iSelectedIndex >= iLabelNum) iSelectedIndex = iLabelNum-1;
    m_pComboBox.setSelectedIndex(iSelectedIndex);
    
    
  }
  
  /**
   *  Sets the selected index of the {@code ComboBox}. 
   *
   *  @param iSelectedIndex The index of the {@code ComboBox} that should be selected.
   **/
  public void setSelectedIndex(int iSelectedIndex) {
    if (iSelectedIndex < 0) iSelectedIndex = 0;
    if (iSelectedIndex >= m_pComboBox.getItemCount()) iSelectedIndex = m_pComboBox.getItemCount()-1;
    
    m_pComboBox.setSelectedIndex(iSelectedIndex);
  }
  
  
  
  ////////////////////////////////
  // LISTENER METHODS           //
  ////////////////////////////////
  /**
   *  Adds a listener for this {@code ComboBoxPanel}. 
   *
   *  @param pListener The listener for this {@code ComboBoxPanel} that should be added. 
   **/
  public void addComboBoxPanelListener(IComboBoxPanelListener pListener) {
    if (m_pListeners == null) m_pListeners = new Vector<IComboBoxPanelListener>();
    m_pListeners.add(pListener);
  }
  
  /**
   *  Removes the specified listener from this {@code ComboBoxPanel}. 
   *
   *  @param pListener The listener that should be removed. 
   **/
  public void removeComboBoxPanelListener(IComboBoxPanelListener pListener) {
    if (m_pListeners == null) return;
    m_pListeners.remove(pListener);
  }
  
  
  /**
   *  Removes all listeners from this {@code ComboBoxPanel}. 
   *
   **/
  public void removeAllComboBoxPanelListeners() {
    if (m_pListeners == null) return;
    m_pListeners.clear();
  }
  
  ///////////////////////////////
  // ACTION LISTENER           //
  ///////////////////////////////
  /**
   *  When a change is made to the {@code ComboBox} this is communicated to all listeners.
   *
   *  @param pEvent The {@code ActionEvent} that specifies the change in the {@code ComboBox}.
   **/
  public void actionPerformed(ActionEvent pEvent) {
    if (m_pListeners == null) return;
    
    int iListenerNum = m_pListeners.size();
    int iSelectedIndex = m_pComboBox.getSelectedIndex();
    for (int i = 0; i < iListenerNum; i++) {
      m_pListeners.get(i).applyChanges(this, iSelectedIndex);
    }
  }
  
  
  
  
}