package gui;

/*
 *  Copyright (C) 2014 Juergen Platzer 
 * 
 *  This file is part of JFXLibrary.
 *  
 *  JFXLibrary is free software: you can redistribute it and/or modify
 *  it under the terms of the GNU Lesser General Public License as published by
 *  the Free Software Foundation, either version 3 of the License, or
 *  (at your option) any later version.
 *  
 *  JFXLibrary is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU Lesser General Public License for more details.
 *  
 *  You should have received a copy of the GNU Lesser General Public License
 *  along with JFXLibrary.  If not, see <http://www.gnu.org/licenses/>.
 */

import javax.swing.*;
import java.awt.Dimension;
import java.awt.image.BufferedImage;
import java.awt.BorderLayout;

import java.awt.event.FocusListener;
import java.awt.event.FocusEvent;

import javax.swing.event.InternalFrameListener;
import javax.swing.event.InternalFrameEvent;

//library imports
import standard.ImageEffect;
import applications.GuiApplication;

/**
 *  Represents an internal frame of an image effect that contains the image effect panel and its properties panel.
 *
 **/

public class ImageEffectInternalFrame extends JInternalFrame implements InternalFrameListener {

  /*** member variables **/
  private AbstractImageEffectPanel m_pPanel;
  private PropertiesInternalFrame m_pPropertiesFrame;
  private GuiApplication m_pGuiApplication;

  //////////////////////////
  // CONSTRUCTOR          //
  //////////////////////////
  /**
   *   Creates an {@code ImageEffectInternalFrame} object with a specified title, image effect panel, properties panel and a link to the 
   *   {@code GuiApplication} object.
   *
   *   @param pTitle The title of the frame as String.
   *   @param pPanel The image effect panel that should be shown.
   *   @param pPropertiesFrame The {@code PropertiesInternalFrame} object that shows the properties panel of the image effect.
   *   @param pApplication The {@code GuiApplication} object that represents the application that hosts this frame.
   **/
  public ImageEffectInternalFrame(  String pTitle, 
                                    AbstractImageEffectPanel pPanel, 
                                    PropertiesInternalFrame pPropertiesFrame, 
                                    GuiApplication pApplication) {
    //1. init the frame
    super(pTitle, true, true, true, true);
    
    //2. init the member variables
    setLayout(new BorderLayout());
    m_pPanel = pPanel;
    this.add(m_pPanel, BorderLayout.CENTER);
    
    m_pPropertiesFrame = pPropertiesFrame;
    m_pGuiApplication = pApplication;
    
    //3. add focus listener
    addInternalFrameListener(this);
    
    
    
  }
  
  
  ////////////////////////////////
  // GET METHODS                //
  ////////////////////////////////
  /**
   *   Returns the image that is created by the image effect that is shown in this frame.
   *
   *   @return The image that is created by the image effect that is shown in this frame.
   *
   *   @throws NullPointerException If no image effect panel is set.
   **/
  public BufferedImage getCurrentEffectImage() throws NullPointerException {
    if (m_pPanel == null) throw new NullPointerException("No effect panel was specified!");
    
    return m_pPanel.getEffectImage();
  }
  
  
  //////////////////////////////////////////
  // INTERNAL FRAME LISTENER METHODS      //
  //////////////////////////////////////////
  
  /**
   *  When this internal frame gets activated, the properties panel of the image effect is set again to the properties frame and 
   *  the gui application is notified, that this is the current effect frame.
   *
   *  @param e The {@code InternalFrameEvent} event that triggers the activation.
   **/
  public void internalFrameActivated(InternalFrameEvent e) {
    //1. make some checks
    if (m_pPanel == null) return;
    if (m_pPropertiesFrame == null) return;
    
    //2. give the properties panel of the effect to the properties frame
    m_pPropertiesFrame.setPropertiesPanel(m_pPanel.getPropertiesPanel());
    System.out.println("Effect panel activated!");
    
    //3. tell the gui application that this is the current effect panel
    m_pGuiApplication.setCurrentEffectPanel(this);
  }
  
  /**
   *  The close event is not used at the moment.
   *
   *  @param e The {@code InternalFrameEvent} event that triggers the closing.
   **/
  public void internalFrameClosed(InternalFrameEvent e){
  }
  
  /**
   *  The close event is not used at the moment.
   *
   *  @param e The {@code InternalFrameEvent} event that triggers the closing.
   **/
  public void internalFrameClosing(InternalFrameEvent e){
  }
  
  /**
   *  The deactivated event is not used at the moment.
   *
   *  @param e The {@code InternalFrameEvent} event that triggers the deactivation.
   **/
  public void internalFrameDeactivated(InternalFrameEvent e){
  }
  
  /**
   *  The Deiconified event is not used at the moment.
   *
   *  @param e The {@code InternalFrameEvent} event that triggers the deiconification.
   **/
  public void internalFrameDeiconified(InternalFrameEvent e){
  }
  
  /**
   *  The Iconified event is not used at the moment.
   *
   *  @param e The {@code InternalFrameEvent} event that triggers the iconification.
   **/
  public void internalFrameIconified(InternalFrameEvent e){
  }
  
  /**
   *  The open event is not used at the moment.
   *
   *  @param e The {@code InternalFrameEvent} event that triggers the opening.
   **/
  public void internalFrameOpened(InternalFrameEvent e) {
  }
  
  
  ////////////////////////////////
  // PRIVATE HELPER METHODS     //
  ////////////////////////////////
  
}

