package gui;

/*
 *  Copyright (C) 2014 Juergen Platzer 
 * 
 *  This file is part of JFXLibrary.
 *  
 *  JFXLibrary is free software: you can redistribute it and/or modify
 *  it under the terms of the GNU Lesser General Public License as published by
 *  the Free Software Foundation, either version 3 of the License, or
 *  (at your option) any later version.
 *  
 *  JFXLibrary is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU Lesser General Public License for more details.
 *  
 *  You should have received a copy of the GNU Lesser General Public License
 *  along with JFXLibrary.  If not, see <http://www.gnu.org/licenses/>.
 */

//gui imports
import javax.swing.JPanel;
import javax.swing.JLabel;
import javax.swing.JButton;
import javax.swing.JOptionPane;
import javax.swing.JFileChooser;
import javax.swing.filechooser.FileNameExtensionFilter;

import javax.imageio.ImageIO;

import java.awt.FlowLayout;
import java.awt.Dimension;
import java.awt.image.BufferedImage;

import java.util.Vector;
import java.io.File;


import java.awt.event.ActionListener;
import java.awt.event.ActionEvent;

//library imports
import applications.Settings;
import applications.ISettingsListener;
import applications.SettingsDialog;
import applications.LanguageProperties;
import gui.io.ImageDisplay;
import gui.io.IImageDisplayListener;




/**
 *   A panel that is used to modify an image property by loading an image from an external source.
 *
 **/
public class ImageLoadPanel extends JPanel implements ActionListener, IImageDisplayListener, ISettingsListener {

  /*** member variables ***/
  private Vector<IImageLoadPanelListener> m_pListeners = null;
  private BufferedImage m_pImage = null;
  private String m_pFileName = null;
  
  /*** gui members ***/
  private JLabel m_pFileNameLabel;
  private JButton m_pLoadButton;
  private ImageDisplay m_pImageDisplay;
  
  /*** private members ***/
  
  ///////////////////////////////
  // CONSTRUCTOR               //
  ///////////////////////////////
  /**
   *  Creates an {@code ImageLoadPanel} object with its gui components: a load button, an image display and a label for the file name.
   *
   */
  public ImageLoadPanel() {
    
    //0. get the language settings 
    Settings pSettings = SettingsDialog.getSettings(); 
    LanguageProperties pLanguageProperties = new LanguageProperties(pSettings.getProperty("Language")); 
    
    //1. construct the gui
    //1.1. set the layout and create a box
    setLayout(new FlowLayout(FlowLayout.LEFT));
    
    //1.2. the load button
    m_pLoadButton = new JButton(pLanguageProperties.getProperty("ImageLoadPanel.Label"));
    m_pLoadButton.addActionListener(this);
    add(m_pLoadButton);
    
    //1.3. to do: a panel that shows a small version of the image
    m_pImageDisplay = new ImageDisplay();
    m_pImageDisplay.addImageDisplayListener(this);
    add(m_pImageDisplay);
    
    //1.4. the file label
    m_pFileNameLabel = new JLabel(pLanguageProperties.getProperty("ImageLoadPanel.NoFileLabel"));
    add(m_pFileNameLabel);
    
   
    
   


    
  }
  
  
  ////////////////////////////////
  // SET METHODS                //
  ////////////////////////////////
  // TO DO: a set image method
  
  
  
  ////////////////////////////////
  // GET METHODS                //
  ////////////////////////////////
  /** 
   *  Returns the image that is currently loaded into this {@code ImageLoadPanel} object.
   *
   *  @return The image that is currently loaded into this {@code ImageLoadPanel} object.
   **/
  public BufferedImage getImage() {
    return m_pImage;
  }
  
  /** 
   *  Returns the file name of the image that is currently loaded into this {@code ImageLoadPanel} object.
   *
   *  @return The file name of the image that is currently loaded into this {@code ImageLoadPanel} object.
   **/
  public String getFileName() {
    return m_pFileName;
  }
  
  
  
  ////////////////////////////////
  // LISTENER METHODS           //
  ////////////////////////////////
  /** 
   *  Adds a listener to this {@code ImageLoadPanel} object.
   *
   *  @param pListener The {@code IImageLoadPanelListener} object that should be added as listener to this {@code ImageLoadPanel} object.
   **/
  public void addImageLoadPanelListener(IImageLoadPanelListener pListener) {
    //1. make some checks
    if (pListener == null) return;
    if (m_pListeners == null) m_pListeners = new Vector<IImageLoadPanelListener>();
    if (m_pListeners.contains(pListener)) return;
    
    //2. add it
    m_pListeners.add(pListener);
  }
  
  /** 
   *  Adds a listener from this {@code ImageLoadPanel} object.
   *
   *  @param pListener The {@code IImageLoadPanelListener} object that should be removed as listener from this {@code ImageLoadPanel} object.
   **/
  public void removeImageLoadPanelListener(IImageLoadPanelListener pListener) {
    //1. make some checks
    if (pListener == null) return;
    if (m_pListeners == null) return;
    
    //2. remove it
    m_pListeners.remove(pListener);
  }
  
  public void removeAllImageLoadPanelListener() {
    //1. make some checks
    if (m_pListeners == null) return;
    
    //2. clear it
    m_pListeners.clear();
  }
  
  
  
  ////////////////////////////////////////
  // ACTION LISTENER                    //
  ////////////////////////////////////////
  /** 
   *  Opens a file chooser to select a new image that should be loaded into this {@code ImageLoadPanel} object.
   *  Finally all listeners are notified, if the image property changed.
   *
   *  @param e The {@code ActionEvent} that triggers this action method.
   **/
  public void actionPerformed(ActionEvent e) {
    
    //1. open a file chooser to select an image file
    JFileChooser pChooser = new JFileChooser();
    FileNameExtensionFilter pFilter = new FileNameExtensionFilter("Image files", "png", "bmp", "gif", "jpg", "jpeg");
    pChooser.setFileFilter(pFilter);
    pChooser.setMultiSelectionEnabled(false);
    pChooser.setDialogTitle(m_pLoadButton.getText()); //-> TO DO: adapt the name -> Make the whole procedure generic (GuiHelper methods)
    File pCurrentDirectory = null;
    Settings pSettings = SettingsDialog.getSettings();
    try {
      pCurrentDirectory = new File(pSettings.getProperty("LoadImage.directory"));
    }
    catch (Exception pException) {}
    pChooser.setCurrentDirectory(pCurrentDirectory);
    
    int iReturnVal = pChooser.showOpenDialog(this);
    
    //2. if the file chooser has not been closed with the approve action -> return
    if(iReturnVal != JFileChooser.APPROVE_OPTION) return;
    
    //3. load the image
    String pFileName = null;
    try {
      File pSelectedFile = pChooser.getSelectedFile();
      pFileName = pSelectedFile.getName();
      m_pImage = ImageIO.read(pSelectedFile);
      pSettings.setProperty("LoadImage.directory", pChooser.getCurrentDirectory().getAbsolutePath());
      pSettings.store(SettingsDialog.SETTINGS_PROPERTY_FILE);
      
      m_pFileName = pFileName;
      m_pFileNameLabel.setText(pFileName);
      
      m_pImageDisplay.setImage(m_pImage);
    }
    catch (Exception pException) {
      String pMessage = "The image " + pFileName + " could not be loaded! " + pException.getMessage();
      JOptionPane.showMessageDialog(this, pMessage, "Image load error", JOptionPane.ERROR_MESSAGE);
      pException.printStackTrace();
      return;
    }
        
      
    
    
    //3. fire an event
    fireApplyChanges();

  }
  
  
  ////////////////////////////////////////
  // IMAGE DISPLAY LISTENER             //
  ////////////////////////////////////////
  /**
   *  Triggers the image load action. 
   *
   *  @param pImageDisplay An {@code ImageDisplay} object that is not used at the moment. (TO DO: Change this method?)
   **/
  public void imageLoadRequested(ImageDisplay pImageDisplay) {
    actionPerformed(null);
  }
  
  //////////////////////////////// 
  // SETTINGS LISTENER METHODS  // 
  //////////////////////////////// 
  /**
   *  Applies new settings (e.g. language settings) to this gui element.
   *
   *  @param pSettings An {@code Settings} object that provides the information that should be changed.
   **/
  public void applyNewSettings(Settings pSettings) { 
    //1. make an initial check and set the settings 
    if (pSettings == null) return; 

    //2. load new language settings 
    LanguageProperties pLanguageProperties = new LanguageProperties(pSettings.getProperty("Language")); 

    //3. set the new labels of the combobox 
    m_pLoadButton.setText(pLanguageProperties.getProperty("ImageLoadPanel.Label"));
    if (m_pImage == null) m_pFileNameLabel.setText(pLanguageProperties.getProperty("ImageLoadPanel.NoFileLabel")); 
  } 
  
  
  //////////////////////////////////
  // PRIVATE HELPER METHODS       //
  //////////////////////////////////
  
  
  private void fireApplyChanges() {
    //1. make some checks
    if (m_pListeners == null) return;
    
    //2. fire the event
    int iListenerNum = m_pListeners.size();
    for (int i = 0; i < iListenerNum; i++) {
      m_pListeners.get(i).applyChanges(this, m_pImage);
    }
    
    
  }
  
  
  

  
  
 
}