package gui;

/*
 *  Copyright (C) 2014 Juergen Platzer 
 * 
 *  This file is part of JFXLibrary.
 *  
 *  JFXLibrary is free software: you can redistribute it and/or modify
 *  it under the terms of the GNU Lesser General Public License as published by
 *  the Free Software Foundation, either version 3 of the License, or
 *  (at your option) any later version.
 *  
 *  JFXLibrary is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU Lesser General Public License for more details.
 *  
 *  You should have received a copy of the GNU Lesser General Public License
 *  along with JFXLibrary.  If not, see <http://www.gnu.org/licenses/>.
 */

//gui imports
import javax.swing.JPanel;
import javax.swing.JScrollPane;
import javax.swing.ImageIcon;
import javax.swing.JLabel;
import java.awt.image.BufferedImage;
import java.awt.Graphics2D;
import java.awt.Graphics;
import java.awt.RenderingHints;
import java.awt.Dimension;
import java.awt.BorderLayout;


import java.awt.event.ComponentListener;
import java.awt.event.ComponentEvent;


//library imports
import standard.EffectParameters;
import standard.ImageEffect;
import standard.IImageEffectListener;
import color.GrayscaleReduction;
import color.ColorTransition;
import color.NoiseGenerator;
import drawing.Network;
import drawing.BrickTexture;
import transformation.ImageResize;
import gui.color.GrayscaleReductionPropertiesPanel;
import gui.color.ColorTransitionPropertiesPanel;
import gui.color.NoiseGeneratorPropertiesPanel;
import gui.drawing.NetworkPropertiesPanel;
import gui.drawing.BrickTexturePropertiesPanel;
import gui.transformation.ImageResizePropertiesPanel;

import applications.Settings;

/**
 *  Implements an image effect panel that shows only the image of the effect. There is no interaction possible.
 *  
 **/
public class SimpleImageEffectPanel extends AbstractImageEffectPanel 
  implements IZoomPanelListener, ComponentListener, IImageEffectListener {

  /*** member variables ***/
  private ImageEffect m_pImageEffect;
  private BufferedImage m_pImage;
  private AbstractPropertiesPanel m_pPropertiesPanel;
  
  /*** zoom member variables ***/
  private ImageIcon m_pImageIcon;
  private JScrollPane m_pScrollPane;
  private ZoomPanel m_pZoomPanel;

  private boolean m_bFitWidth, m_bFitHeight;
  private double m_dZoomFactor;

  ////////////////////////////////////
  // CONSTRUCTOR                    //
  ////////////////////////////////////
  /**
   *  Creates a panel that shows the result of the specified image effect.
   *
   *  @param pImageEffect The image effect that should be shown by this panel. 
   **/
  public SimpleImageEffectPanel(ImageEffect pImageEffect) {
    
    //1. set the members
    m_pImageEffect = pImageEffect;
    m_pImage = null;
    
    if (pImageEffect instanceof GrayscaleReduction) {
      m_pPropertiesPanel = new GrayscaleReductionPropertiesPanel();
    }
    else if (pImageEffect instanceof Network) {
      m_pPropertiesPanel = new NetworkPropertiesPanel();
    }
    else if (pImageEffect instanceof BrickTexture) {
      m_pPropertiesPanel = new BrickTexturePropertiesPanel();
    }
    else if (pImageEffect instanceof ColorTransition) {
      m_pPropertiesPanel = new ColorTransitionPropertiesPanel();
    }
    else if (pImageEffect instanceof NoiseGenerator) {
      m_pPropertiesPanel = new NoiseGeneratorPropertiesPanel();
    }
    else if (pImageEffect instanceof ImageResize) {
      m_pPropertiesPanel = new ImageResizePropertiesPanel();
    }
    else {
      m_pPropertiesPanel = null;
    }
    
    if (m_pPropertiesPanel != null) m_pPropertiesPanel.addPropertiesPanelListener(this);
    
    //2. set the layout
    setLayout(new BorderLayout());

    //3. create the zoompanel
    m_pZoomPanel = new ZoomPanel();
    m_pZoomPanel.addZoomPanelListener(this);
    add(m_pZoomPanel, BorderLayout.SOUTH);

    //4. create the image icon and load the image
    m_pImageEffect.processAsThread(this);
    m_pImageIcon = new ImageIcon();

    //5. create a JScrollPane
    m_pScrollPane = new JScrollPane(new JLabel(m_pImageIcon));
    add(m_pScrollPane, BorderLayout.CENTER);

    
    //6. set zoom members and add the component listener
    m_bFitWidth = false;
    m_bFitHeight = false;
    m_dZoomFactor = 1;
    
    addComponentListener(this);
    
    
  }
  
  ////////////////////////////////
  // SET METHODS                //
  ////////////////////////////////
  /**
   *   Sets the parameters of the image effect that is shown by this panel. 
   *
   *   @param pEffectParameters The parameters of the image effect that is shown by this panel.
   **/
  public void setEffectParameters(EffectParameters pEffectParameters) {
    //1. make some checks
    if (pEffectParameters == null) return;
    if (m_pImageEffect == null) return;
    
    //2. set the effect parameters and init a repainting
    m_pImageEffect.setEffectParameters(pEffectParameters);
    m_pImageEffect.processAsThread(this);
    //repaintZoomedImage();
  }
  
  /**
   *  Sets the size of the result image of the image effect. Is not implemented at the moment.
   *
   *  @param iWidth The width of the result image.
   *  @param iHeight The height of the result image.
   **/
  public void setEffectSize(int iWidth, int iHeight) {
    //TO DO
  }
  
  
  ////////////////////////////////
  // GET METHODS                //
  ////////////////////////////////
  /**
   *  Returns the parameters of the image effect that is shown in this panel. 
   *
   *  @return The parameters of the image effect that is shown in this panel. 
   **/
  public EffectParameters getEffectParameters() {
    return m_pImageEffect.getEffectParameters();
  }
  
  /**
   *  Returns the properties panel of the image effect that is shown in this panel. 
   *
   *  @return The properties panel of the image effect that is shown in this panel. 
   **/
  public AbstractPropertiesPanel getPropertiesPanel() {
    return m_pPropertiesPanel;
  }
  
  /**
   *  Returns the result image of the image effect that is shown in this panel. 
   *
   *  @return The result image of the image effect that is shown in this panel. 
   **/
  public BufferedImage getEffectImage() {
    return m_pImage;
  }
  
  
 
  
  //////////////////////////////////////
  // DRAWING HELPER METHODS           //
  //////////////////////////////////////
  protected void drawInteraction(Graphics2D pGraphics) {
    //nothing to do at the moment
  }
  
  
  ///////////////////////////////////
  // COMPONENT LISTENER            //
  ///////////////////////////////////
  /**
   *  Performed when the panel is hidden. Not used at the moment.
   *
   *  @param e The event triggering this method.
   **/
  public void componentHidden(ComponentEvent e) {}
  
  /**
   *  Performed when the panel is moved. Not used at the moment.
   *
   *  @param e The event triggering this method.
   **/
  public void componentMoved(ComponentEvent e) {}
  
  /**
   *  Performed when the panel is resized. When the image effect is shown in the fit width or fit height mode, 
   *  this method triggers a repaint action to resize the result image correctly.
   *
   *  @param e The event triggering this method.
   **/
  public void componentResized(ComponentEvent e) {
    if (m_bFitWidth) setFitWidth();
    if (m_bFitHeight) setFitHeight();
  }
  
  /**
   *  Performed when the panel is shown. Not used at the moment.
   *
   *  @param e The event triggering this method.
   **/
  public void componentShown(ComponentEvent e)  {}


  ///////////////////////////////////
  // I ZOOM PANEL LISTENER         //
  ///////////////////////////////////
  /**
   *  Sets the zoom factor of the image or video. 
   *
   *  @param dZoomFactor The zoom factor of the image or video as double.
   **/
  public void setZoomFactor(double dZoomFactor) {
    zoomImage(dZoomFactor);

    m_bFitWidth = false;
    m_bFitHeight = false;
  }

  
  /**
   *  Sets the zoom factor so that it fits the width of the current display. 
   **/
  public void setFitWidth() {
    double dZoomFactor = ((double)m_pScrollPane.getViewport().getWidth())/((double)m_pImage.getWidth());
    zoomImage(dZoomFactor);

    m_bFitWidth = true;
    m_bFitHeight = false;
  }

  
  /**
   *  Sets the zoom factor so that it fits the height of the current display. 
   **/
  public void setFitHeight() {
    double dZoomFactor = ((double)m_pScrollPane.getViewport().getHeight())/((double)m_pImage.getHeight());
    zoomImage(dZoomFactor);

    m_bFitWidth = false;
    m_bFitHeight = true;
  }
  
  
  
  ///////////////////////////
  // IIMAGE EFFECT METHODS //
  ///////////////////////////
  /**
   *   The only method how the image that should be shown can be set. It triggers a repaint.
   *
   *   @param pImage    The rendered image.
   **/
  public void setEffectImage(BufferedImage pImage) {
    m_pImage = pImage;
    repaintZoomedImage();
  }
  
  //////////////////////////////// 
  // SETTINGS LISTENER METHODS  // 
  //////////////////////////////// 
  /**
   *  If the settings for the gui application have been changed, the changed settings are sent and the properties panel can adapt accordingly.
   *
   *  @param pSettings The modified settings of the gui application.
   **/
  public void applyNewSettings(Settings pSettings) { 
    m_pZoomPanel.applyNewSettings(pSettings);
  }

  

  ///////////////////////////////////////
  // HELPER METHODS                    //
  ///////////////////////////////////////
  private void zoomImage(double dZoomFactor) {
    //1. make some checks
    if (dZoomFactor <= 0) return;
    
    
    m_dZoomFactor = dZoomFactor;
    repaintZoomedImage();
    
    

  }
  
  
  private void repaintZoomedImage() {
    //1. make a simple check
    if (m_pImage == null) return;
  
    //2. create the zoom image
    int iNewWidth = (int)(m_pImage.getWidth()*m_dZoomFactor);
    int iNewHeight = (int)(m_pImage.getHeight()*m_dZoomFactor);
    //System.out.println("hallo: " + iNewWidth + ", " + iNewHeight);

    BufferedImage pZoomImage = new BufferedImage(iNewWidth, iNewHeight, m_pImage.getType());

    //3. draw the image in the best quality
    Graphics2D pGraphics = pZoomImage.createGraphics();
    pGraphics.setRenderingHint(RenderingHints.KEY_INTERPOLATION,
    RenderingHints.VALUE_INTERPOLATION_BILINEAR);
    pGraphics.setRenderingHint(RenderingHints.KEY_RENDERING,
    RenderingHints.VALUE_RENDER_QUALITY);
    pGraphics.setRenderingHint(RenderingHints.KEY_ANTIALIASING,
    RenderingHints.VALUE_ANTIALIAS_ON);
    pGraphics.drawImage(m_pImage, 0, 0, iNewWidth, iNewHeight, null);

    //4. display the image in the gui
    m_pImageIcon.setImage(pZoomImage);
    m_pScrollPane.setViewportView(new JLabel(m_pImageIcon));
  }

  

  




  
  
  
  

 

  
  
 
}