package gui;

/*
 *  Copyright (C) 2014 Juergen Platzer 
 * 
 *  This file is part of JFXLibrary.
 *  
 *  JFXLibrary is free software: you can redistribute it and/or modify
 *  it under the terms of the GNU Lesser General Public License as published by
 *  the Free Software Foundation, either version 3 of the License, or
 *  (at your option) any later version.
 *  
 *  JFXLibrary is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU Lesser General Public License for more details.
 *  
 *  You should have received a copy of the GNU Lesser General Public License
 *  along with JFXLibrary.  If not, see <http://www.gnu.org/licenses/>.
 */

//gui imports
import javax.swing.JPanel;
import javax.swing.JComboBox;
import javax.swing.JLabel;
import javax.swing.JSpinner;
import javax.swing.SpinnerNumberModel;
import javax.swing.border.EmptyBorder;

import java.awt.BorderLayout;
import java.awt.FlowLayout;
import java.awt.Dimension;
import java.awt.GridLayout;

import javax.swing.BoxLayout;
import javax.swing.Box;

import java.util.Vector;


import javax.swing.event.ChangeListener;
import javax.swing.event.ChangeEvent;
import java.awt.event.ActionListener;
import java.awt.event.ActionEvent;


/***
 *  A panel that steers the size mode for an image effect.
 **/
public class SizeModePanel extends JPanel implements ChangeListener, ActionListener {

  /*** constants ***/
  public static final int MINIMUM_FIXED_WIDTH = 10;
  public static final int MAXIMUM_FIXED_WIDTH = 10000;
  public static final int DEFAULT_FIXED_WIDTH = 400;
  
  public static final int MINIMUM_FIXED_HEIGHT = 10;
  public static final int MAXIMUM_FIXED_HEIGHT = 10000;
  public static final int DEFAULT_FIXED_HEIGHT = 400;
  
  public static final int RESIZING_MODE_PANELSIZE = 0;
  public static final int RESIZING_MODE_FIXEDSIZE = 1;
  
  /*** member variables ***/
  private Vector<ISizeModePanelListener> m_pListeners = null;
  
  /*** gui members ***/
  private JLabel m_pSizeModeLabel, m_pFixedWidthLabel, m_pFixedHeightLabel;
  private JComboBox<String> m_pSizeModes;
  private JSpinner m_pFixedWidthSpinner, m_pFixedHeightSpinner;
  
  ///////////////////////////////
  // CONSTRUCTOR               //
  ///////////////////////////////
  /**
   *  Creates the {@code SizeModePanel} object.
   **/
  public SizeModePanel() {
  
   
    //1. set layout
    setLayout(new GridLayout(3,2));
    
    //2. create the size mode selection
    m_pSizeModeLabel = new JLabel("Size mode:");
    Vector<String> pSizeModes = new Vector<String>();
    pSizeModes.add("Panel size");
    pSizeModes.add("Fixed size");
    m_pSizeModes = new JComboBox<String>(pSizeModes);
    m_pSizeModes.addActionListener(this);
    add(m_pSizeModeLabel);
    add(m_pSizeModes);
    
    //3. create the fixed width line
    m_pFixedWidthLabel = new JLabel("Fixed width:");
    
    Integer pWidthValue = new Integer(DEFAULT_FIXED_WIDTH);
    Integer pWidthMin = new Integer(MINIMUM_FIXED_WIDTH);
    Integer pWidthMax = new Integer(MAXIMUM_FIXED_WIDTH);
    Integer pWidthStep = new Integer(1);
    SpinnerNumberModel pWidthSpinnerModel = new SpinnerNumberModel(pWidthValue, pWidthMin, pWidthMax, pWidthStep); 
    m_pFixedWidthSpinner = new JSpinner(pWidthSpinnerModel);
    m_pFixedWidthSpinner.addChangeListener(this);
    
    add(m_pFixedWidthLabel);
    add(m_pFixedWidthSpinner);
    
    
    //4. create the fixed height line
    m_pFixedHeightLabel = new JLabel("Fixed height:");
    
    Integer pHeightValue = new Integer(DEFAULT_FIXED_HEIGHT);
    Integer pHeightMin = new Integer(MINIMUM_FIXED_HEIGHT);
    Integer pHeightMax = new Integer(MAXIMUM_FIXED_HEIGHT);
    Integer pHeightStep = new Integer(1);
    SpinnerNumberModel pHeightSpinnerModel = new SpinnerNumberModel(pHeightValue, pHeightMin, pHeightMax, pHeightStep); 
    m_pFixedHeightSpinner = new JSpinner(pHeightSpinnerModel);
    m_pFixedHeightSpinner.addChangeListener(this);
    
    add(m_pFixedHeightLabel);
    add(m_pFixedHeightSpinner);
    
    //5. set a border
    setBorder(new EmptyBorder(0, 5, 0, 5));
  
  
    
    
  }
  
  
  ////////////////////////////////
  // SET METHODS                //
  ////////////////////////////////
  
  
  
  ////////////////////////////////
  // GET METHODS                //
  ////////////////////////////////
  
  
  
  
  ////////////////////////////////
  // LISTENER METHODS           //
  ////////////////////////////////
  /**  
   *  Adds a listener to this {@code SizeModePanel} object. 
   *
   *  @param pListener The listener that should be added. 
   **/
  public void addSizeModePanelListener(ISizeModePanelListener pListener) {
    //1. make some checks
    if (pListener == null) return;
    if (m_pListeners == null) m_pListeners = new Vector<ISizeModePanelListener>();
    if (m_pListeners.contains(pListener)) return;
    
    //2. add it
    m_pListeners.add(pListener);
  }
  
  /**  
   *  Removes a listener from this {@code SizeModePanel} object. 
   *
   *  @param pListener The listener that should be removed. 
   **/
  public void removeSizeModePanelListener(ISizeModePanelListener pListener) {
    //1. make some checks
    if (pListener == null) return;
    if (m_pListeners == null) return;
    
    //2. remove it
    m_pListeners.remove(pListener);
  }
  
  
  /**  
   *  Removes all listeners from this {@code SizeModePanel} object. 
   *
   **/
  public void removeAllSizeModePanelListeners() {
    //1. make some checks
    if (m_pListeners == null) return;
    
    //2. clear it
    m_pListeners.clear();
  }
  
  
  
  ////////////////////////////////////////
  // CHANGE LISTENER - JSPINNER EVENTS  //
  ////////////////////////////////////////
  /**
   *  Applies the changes of the {@code JSpinner}.
   *
   *  @param e The event triggered by the {@code JSpinner}.
   *
   **/
  public void stateChanged(ChangeEvent e) {
    fireApplyChanges();
  }
  
  
  ////////////////////////////////////////
  // ACTION LISTENER - JCOMBOBOX EVENTS //
  ////////////////////////////////////////
  /**
   *  Applies the changes of the {@code JComboBox}.
   *
   *  @param e The event triggered by the {@code JComboBox}.
   *
   **/
  public void actionPerformed(ActionEvent e) {
    fireApplyChanges();
  }
  
  private void fireApplyChanges() {
    //1. make some checks
    if (m_pListeners == null) return;
    
    //2. fetch the new parameter values
    int iSizeMode = m_pSizeModes.getSelectedIndex();
    int iFixedWidth = ((Integer)m_pFixedWidthSpinner.getValue()).intValue();
    int iFixedHeight = ((Integer)m_pFixedHeightSpinner.getValue()).intValue();
    
    //3. fire the event
    int iListenerNum = m_pListeners.size();
    for (int i = 0; i < iListenerNum; i++) {
      m_pListeners.get(i).applySizeModeChanges(iSizeMode, iFixedWidth, iFixedHeight);
    }
    
  }

  
  
 
}