package gui;

/*
 *  Copyright (C) 2014 Juergen Platzer 
 * 
 *  This file is part of JFXLibrary.
 *  
 *  JFXLibrary is free software: you can redistribute it and/or modify
 *  it under the terms of the GNU Lesser General Public License as published by
 *  the Free Software Foundation, either version 3 of the License, or
 *  (at your option) any later version.
 *  
 *  JFXLibrary is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU Lesser General Public License for more details.
 *  
 *  You should have received a copy of the GNU Lesser General Public License
 *  along with JFXLibrary.  If not, see <http://www.gnu.org/licenses/>.
 */

//gui imports
import javax.swing.JPanel;
import javax.swing.JSlider;
import javax.swing.JLabel;
import javax.swing.JSpinner;
import javax.swing.SpinnerNumberModel;
import javax.swing.border.EmptyBorder;

import java.awt.BorderLayout;
import java.awt.FlowLayout;
import java.awt.Dimension;
import javax.swing.BoxLayout;
import javax.swing.Box;

import java.util.Vector;


import javax.swing.event.ChangeListener;
import javax.swing.event.ChangeEvent;


/**
 *  A panel for steering a numeric effect parameter with a slider and a spinner as main gui components.
 **/
public class SliderPanel extends JPanel implements ChangeListener {

  /*** member variables ***/
  private Vector<ISliderPanelListener> m_pListeners = null;
  
  /*** gui members ***/
  private JLabel m_pLabel, m_pMinLabel, m_pMaxLabel;
  private JSlider m_pSlider;
  private JSpinner m_pSpinner;
  private boolean m_bChangeListenerInProgress = false;
  
  ///////////////////////////////
  // CONSTRUCTOR               //
  ///////////////////////////////
  /**
   *  Creates the {@code SliderPanel} object with the specified label, the specified minimum and maximum value and the given current value.
   *
   *  @param pLabel The label for the {@code SliderPanel} object.
   *  @param iMin The minimum value of the effect parameter.
   *  @param iMax The maximum value of the effect parameter.
   *  @param iCurrentValue The current value of the effect parameter.
   **/
  public SliderPanel(String pLabel, int iMin, int iMax, int iCurrentValue) {
    //1. make some checks
    if (pLabel == null) pLabel = "";
    pLabel = pLabel.trim();
    
    if (iMin > iMax) {
      int iHelp = iMin;
      iMin = iMax;
      iMax = iHelp;
    }
    
    if (iCurrentValue < iMin) iCurrentValue = iMin;
    if (iCurrentValue > iMax) iCurrentValue = iMax;
    
    
    //2. construct the gui
    //2.1. set the layout and create a box
    setLayout(new BoxLayout(this, BoxLayout.Y_AXIS));
    Box pBox = new Box(BoxLayout.Y_AXIS);
    
    //2.2. create the label and the spinner and add it left and right in a border layout
    if (!pLabel.endsWith(":")) pLabel += ":";
    m_pLabel = new JLabel(pLabel);
    
    JPanel pHelpPanel = new JPanel();
    pHelpPanel.setLayout(new BorderLayout());
    pHelpPanel.setMaximumSize(new Dimension(300, 40));
    pHelpPanel.add(m_pLabel, BorderLayout.WEST);
    
    Integer pIntValue = new Integer(iCurrentValue);
    Integer pIntMin = new Integer(iMin);
    Integer pIntMax = new Integer(iMax);
    Integer pIntStep = new Integer(1);
    SpinnerNumberModel pSpinnerModel = new SpinnerNumberModel(pIntValue, pIntMin, pIntMax, pIntStep); 
    m_pSpinner = new JSpinner(pSpinnerModel);
    m_pSpinner.addChangeListener(this);
    pHelpPanel.add(m_pSpinner, BorderLayout.EAST);
    
    pBox.add(pHelpPanel);
    
    //2.3. sub panel with all the min and max label and the slider
    JPanel pSubPanel = new JPanel();
    pSubPanel.setLayout(new FlowLayout());
    pSubPanel.setMaximumSize(new Dimension(400, 100));
    
    m_pMinLabel = new JLabel(iMin + "");
    pSubPanel.add(m_pMinLabel);
    
    m_pSlider = new JSlider(iMin, iMax, iCurrentValue);
    m_pSlider.addChangeListener(this);
    pSubPanel.add(m_pSlider);
    
    m_pMaxLabel = new JLabel(iMax + "");
    pSubPanel.add(m_pMaxLabel);
    
    
    
    pBox.add(pSubPanel);
    
    //2.4. create some glue that takes the unnecessary space and set the box as only 
    //     element of this panel
    //pBox.add(pBox.createGlue());
    add(pBox);
    
    
    
  }
  
  
  ////////////////////////////////
  // SET METHODS                //
  ////////////////////////////////
  /**
   *  Sets the label text of this {@code SliderPanel} object.
   *
   *  @param pLabel The new label that should be displayed in this {@code SliderPanel} object.
   **/
  public void setLabel(String pLabel) {
    if (pLabel == null) pLabel = "";
    pLabel = pLabel.trim();
    
    if (!pLabel.endsWith(":")) pLabel += ":";
    m_pLabel.setText(pLabel);

  }
  
  /**
   *  Sets the label text of this {@code SliderPanel} object.
   *
   *  @param pText The new label that should be displayed in this {@code SliderPanel} object.
   **/
  public void setText(String pText) {
    setLabel(pText);
  }
  
  
  ////////////////////////////////
  // GET METHODS                //
  ////////////////////////////////
  
  
  
  
  ////////////////////////////////
  // LISTENER METHODS           //
  ////////////////////////////////
  /**
   *  Adds a listener to this {@code SliderPanel} object.
   *
   *  @param pListener The listener that should be added. 
   **/
  public void addSliderPanelListener(ISliderPanelListener pListener) {
    //1. make some checks
    if (pListener == null) return;
    if (m_pListeners == null) m_pListeners = new Vector<ISliderPanelListener>();
    if (m_pListeners.contains(pListener)) return;
    
    //2. add it
    m_pListeners.add(pListener);
  }
  
  /**
   *  Removes a listener from this {@code SliderPanel} object.
   *
   *  @param pListener The listener that should be removed. 
   **/
  public void removeSliderPanelListener(ISliderPanelListener pListener) {
    //1. make some checks
    if (pListener == null) return;
    if (m_pListeners == null) return;
    
    //2. remove it
    m_pListeners.remove(pListener);
  }
  
  
  /**
   *  Removes all listeners from this {@code SliderPanel} object.
   *
   **/
  public void removeAllSliderPanelListener() {
    //1. make some checks
    if (m_pListeners == null) return;
    
    //2. clear it
    m_pListeners.clear();
  }
  
  
  
  ////////////////////////////////////////
  // CHANGE LISTENER - SLIDER EVENTS    //
  ////////////////////////////////////////
  /**
   *  Applies the changes of the {@code JSpinner} and the {@code JSlider} object.
   *
   *  @param e The event triggered by the {@code JSpinner} or the {@code JSlider} object.
   *
   **/
  public void stateChanged(ChangeEvent e) {
    //1. make a simple check
    if (m_bChangeListenerInProgress) return;
    
    //2. set the change listener progress flag
    m_bChangeListenerInProgress = true;
    
    //3. fetch the new parameter value
    Object pSource = e.getSource();
    int iCurrentValue = 0;
    if (pSource.equals(m_pSlider)) {
      iCurrentValue = m_pSlider.getValue();
      m_pSpinner.setValue(new Integer(iCurrentValue));
    }
    else {
      iCurrentValue = ((Integer)m_pSpinner.getValue()).intValue();
      m_pSlider.setValue(iCurrentValue);
    }
  
    
    
    //4. unset the change listener progress flag and fire the apply changes event
    m_bChangeListenerInProgress = false;
    fireApplyChanges(iCurrentValue);

  }
  
  private void fireApplyChanges(int iValue) {
    //1. make some checks
    if (m_pListeners == null) return;
    
    //2. fire the event
    int iListenerNum = m_pListeners.size();
    for (int i = 0; i < iListenerNum; i++) {
      m_pListeners.get(i).applyChanges(this, iValue);
    }
    
    
  }

  
  
 
}