package gui;

/*
 *  Copyright (C) 2014 Juergen Platzer 
 * 
 *  This file is part of JFXLibrary.
 *  
 *  JFXLibrary is free software: you can redistribute it and/or modify
 *  it under the terms of the GNU Lesser General Public License as published by
 *  the Free Software Foundation, either version 3 of the License, or
 *  (at your option) any later version.
 *  
 *  JFXLibrary is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU Lesser General Public License for more details.
 *  
 *  You should have received a copy of the GNU Lesser General Public License
 *  along with JFXLibrary.  If not, see <http://www.gnu.org/licenses/>.
 */

//gui imports
import javax.swing.JPanel;
import javax.swing.JSlider;
import javax.swing.JLabel;
import javax.swing.JSpinner;
import javax.swing.SpinnerNumberModel;
import javax.swing.border.EmptyBorder;

import java.awt.FlowLayout;
import java.awt.Dimension;
import javax.swing.Box;

import java.util.Vector;


import javax.swing.event.ChangeListener;
import javax.swing.event.ChangeEvent;


/**
 *  A panel for steering a numeric effect parameter with a slider and a spinner as main gui components.
 **/
public class SlimSliderPanel extends JPanel implements ChangeListener {

  /*** member variables ***/
  private Vector<ISlimSliderPanelListener> m_pListeners = null;
  
  /*** gui members ***/
  private JLabel m_pMinLabel, m_pMaxLabel;
  private JSlider m_pSlider;
  private JSpinner m_pSpinner;
  private boolean m_bChangeListenerInProgress = false;
  
  ///////////////////////////////
  // CONSTRUCTOR               //
  ///////////////////////////////
  /**
   *  Creates the {@code SlimSliderPanel} object with the specified minimum and maximum value and the given current value.
   *
   *  @param iMin The minimum value of the effect parameter.
   *  @param iMax The maximum value of the effect parameter.
   *  @param iCurrentValue The current value of the effect parameter.
   **/
  public SlimSliderPanel(int iMin, int iMax, int iCurrentValue) {
    //1. make some checks    
    if (iMin > iMax) {
      int iHelp = iMin;
      iMin = iMax;
      iMax = iHelp;
    }
    
    if (iCurrentValue < iMin) iCurrentValue = iMin;
    if (iCurrentValue > iMax) iCurrentValue = iMax;
    
    
    //2. construct the gui
    //2.1. set the layout and create a box
    setLayout(new FlowLayout(FlowLayout.LEFT));
    
    //2.2. the min and max label and the slider
    m_pMinLabel = new JLabel(iMin + "");
    add(m_pMinLabel);
    
    m_pSlider = new JSlider(iMin, iMax, iCurrentValue);
    m_pSlider.addChangeListener(this);
    add(m_pSlider);
    
    m_pMaxLabel = new JLabel(iMax + "");
    add(m_pMaxLabel);
    
    
    //2.3. createthe spinner and add it left and right in a border layout
    Integer pIntValue = new Integer(iCurrentValue);
    Integer pIntMin = new Integer(iMin);
    Integer pIntMax = new Integer(iMax);
    Integer pIntStep = new Integer(1);
    SpinnerNumberModel pSpinnerModel = new SpinnerNumberModel(pIntValue, pIntMin, pIntMax, pIntStep); 
    m_pSpinner = new JSpinner(pSpinnerModel);
    m_pSpinner.addChangeListener(this);
    add(m_pSpinner);


    
  }
  
  
  ////////////////////////////////
  // SET METHODS                //
  ////////////////////////////////
  /**
   *  Sets the integer value of the effect parameter that is represented by this {@code SlimSliderPanel}.
   *  
   *  @param iValue The integer value of the effect parameter that should be used by this {@code SlimSliderPanel}.
   **/
  public void setIntValue(int iValue) {
    //1. make some checks
    if (iValue < m_pSlider.getMinimum()) iValue = m_pSlider.getMinimum();
    if (iValue > m_pSlider.getMaximum()) iValue = m_pSlider.getMaximum();
    
    //2. set the value but without reacting to the listeners
    m_bChangeListenerInProgress = true;
    m_pSlider.setValue(iValue);
    m_pSpinner.setValue(new Integer(iValue));
    m_bChangeListenerInProgress = false;
  }
  
  
  /**
   *  Sets the maximum value of the effect parameter that is represented by this {@code SlimSliderPanel}.
   *  
   *  @param iMax The maximum value of the effect parameter.
   **/
  public void setMaximum(int iMax) {
    //1. make some checks
    if (iMax <= m_pSlider.getMinimum()) return; // it is not valid
    
    //2. adapt current value, if necessary
    int iCurrentValue = getIntValue();
    boolean bResetCurrentValue = false;
    if (iMax < iCurrentValue) {
      iCurrentValue = iMax;
      bResetCurrentValue = true;
    }
    
    
    //2. set the max values on the gui elements
    m_bChangeListenerInProgress = true;
    
    m_pSlider.setValue(iCurrentValue);
    m_pSlider.setMaximum(iMax);
    
    Integer pIntValue = new Integer(iCurrentValue);
    Integer pIntMin = new Integer(m_pSlider.getMinimum());
    Integer pIntMax = new Integer(iMax);
    Integer pIntStep = new Integer(1);
    SpinnerNumberModel pSpinnerModel = new SpinnerNumberModel(pIntValue, pIntMin, pIntMax, pIntStep); 
    m_pSpinner.setModel(pSpinnerModel);
    
    m_pMaxLabel.setText(iMax + "");
    
    m_bChangeListenerInProgress = false;
    
    //3. if the current value has been adapted fire a change event
    if (bResetCurrentValue) {
      fireApplyChanges(iCurrentValue);
    }
  }
  
  ////////////////////////////////
  // GET METHODS                //
  ////////////////////////////////
  /**
   *  Returns the integer value of the effect parameter that is represented by this {@code SlimSliderPanel}.
   *  
   *  @return The integer value of the effect parameter that is represented by this {@code SlimSliderPanel}.
   **/
  public int getIntValue() {
    return ((Integer)m_pSpinner.getValue()).intValue();
  }
  
  
  
  ////////////////////////////////
  // LISTENER METHODS           //
  ////////////////////////////////
  /**
   *  Adds a listener to this {@code SlimSliderPanel} object.
   *
   *  @param pListener The listener that should be added. 
   **/
  public void addSlimSliderPanelListener(ISlimSliderPanelListener pListener) {
    //1. make some checks
    if (pListener == null) return;
    if (m_pListeners == null) m_pListeners = new Vector<ISlimSliderPanelListener>();
    if (m_pListeners.contains(pListener)) return;
    
    //2. add it
    m_pListeners.add(pListener);
  }
  
  /**
   *  Removes a listener from this {@code SlimSliderPanel} object.
   *
   *  @param pListener The listener that should be removed. 
   **/
  public void removeSlimSliderPanelListener(ISlimSliderPanelListener pListener) {
    //1. make some checks
    if (pListener == null) return;
    if (m_pListeners == null) return;
    
    //2. remove it
    m_pListeners.remove(pListener);
  }
  
  /**
   *  Removes all listeners from this {@code SlimSliderPanel} object.
   *
   **/
  public void removeAllSlimSliderPanelListener() {
    //1. make some checks
    if (m_pListeners == null) return;
    
    //2. clear it
    m_pListeners.clear();
  }
  
  
  
  ////////////////////////////////////////
  // CHANGE LISTENER - SLIDER EVENTS    //
  ////////////////////////////////////////
  /**
   *  Applies the changes of the {@code JSpinner} and the {@code JSlider} object.
   *
   *  @param e The event triggered by the {@code JSpinner} or the {@code JSlider} object.
   *
   **/
  public void stateChanged(ChangeEvent e) {
    //1. make a simple check
    if (m_bChangeListenerInProgress) return;
    
    //2. set the change listener progress flag
    m_bChangeListenerInProgress = true;
    
    //3. fetch the new parameter value
    Object pSource = e.getSource();
    int iCurrentValue = 0;
    if (pSource.equals(m_pSlider)) {
      iCurrentValue = m_pSlider.getValue();
      m_pSpinner.setValue(new Integer(iCurrentValue));
    }
    else {
      iCurrentValue = ((Integer)m_pSpinner.getValue()).intValue();
      m_pSlider.setValue(iCurrentValue);
    }
  
    
    
    //4. unset the change listener progress flag and fire the apply changes event
    m_bChangeListenerInProgress = false;
    fireApplyChanges(iCurrentValue);

  }
  
  private void fireApplyChanges(int iValue) {
    //1. make some checks
    if (m_pListeners == null) return;
    
    //2. fire the event
    int iListenerNum = m_pListeners.size();
    for (int i = 0; i < iListenerNum; i++) {
      m_pListeners.get(i).applyChanges(this, iValue);
    }
    
    
  }

  
  
 
}