package gui.color;

/*
 *  Copyright (C) 2014 Juergen Platzer 
 * 
 *  This file is part of JFXLibrary.
 *  
 *  JFXLibrary is free software: you can redistribute it and/or modify
 *  it under the terms of the GNU Lesser General Public License as published by
 *  the Free Software Foundation, either version 3 of the License, or
 *  (at your option) any later version.
 *  
 *  JFXLibrary is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU Lesser General Public License for more details.
 *  
 *  You should have received a copy of the GNU Lesser General Public License
 *  along with JFXLibrary.  If not, see <http://www.gnu.org/licenses/>.
 */

import javax.swing.JPanel;
import javax.swing.JColorChooser;
import java.awt.Color;
import java.awt.Dimension;
import java.awt.Graphics;
import java.awt.Graphics2D;

import java.awt.event.MouseListener;
import java.awt.event.MouseEvent;

import java.util.Vector;


/**
 *  A panel that shows a color and allows to modify it by clicking on it.
 **/
public class ColorDisplay extends JPanel implements MouseListener {
  
  /** private gui member variables **/
  private Color m_pColor;
  private String m_pColorChooserTitle;
  
  /*** listener member variables **/
  private Vector<IColorDisplayListener> m_pListeners;
  
  //////////////////////////
  // CONSTRUCTOR          //
  //////////////////////////
  /**
   *  Creates a default {@code ColorDisplay} object with the color Black and an empty title for the Color chooser dialog.
   **/
  public ColorDisplay() {
    //1. set members
    m_pColor = Color.BLACK;
    m_pColorChooserTitle = "";
    m_pListeners = new Vector<IColorDisplayListener>();
    
    //2. set size and tooltip
    setPreferredSize(new Dimension(50, 30));
    setToolTipText("Click to change the color!");
    
    //3. set listeners
    addMouseListener(this);
  }
  
  
  
  ////////////////////////////
  // GET METHODS            //
  ////////////////////////////
  /**
   *  Returns the color that is currently represented by this {@code ColorDisplay} object.
   *
   *  @return The color that is currently represented by this {@code ColorDisplay} object.
   **/
  public Color getColor() {
    return m_pColor;
  }
  
  ////////////////////////////
  // SET METHODS            //
  ////////////////////////////
  /**
   *  Sets the color that should be shown by this {@code ColorDisplay} object. 
   *  When the color is changed all the listeners of this {@code ColorDisplay} object are notified.
   *
   *  @param pColor The color that should be shown by this {@code ColorDisplay} object.
   **/
  public void setColor(Color pColor) {
    if (pColor == null) return;
    m_pColor = pColor;
    repaint();
    fireListenerEvent();
  }
  
  /**
   *  Sets the title of the color chooser dialog that is opened when the color should be changed.
   *
   *  @param pColorChooserTitle The title of the color chooser dialog that is opened when the color should be changed.
   **/
  public void setColorChooserTitle(String pColorChooserTitle) {
    if (pColorChooserTitle == null) pColorChooserTitle = "";
    m_pColorChooserTitle = pColorChooserTitle;
  }
  
  
  //////////////////////////
  // PAINTING             //
  //////////////////////////
  /**
   *  Paints the panel: The whole panel is filled with the color that is currently set to it.
   *
   *  @param g The graphics context that is used for the paint operations.
   **/
  public void paintComponent(Graphics g) {
    //set the color
    Graphics2D pGraphics = (Graphics2D)g;
    pGraphics.setColor(m_pColor);
    
    //fill out a colored rectangle
    int iWidth = getWidth();
    int iHeight = getHeight();
    
    pGraphics.fillRect(0, 0, iWidth, iHeight);
    
  }
  
  
  ///////////////////////////
  // MOUSE LISTENER        //
  ///////////////////////////
  /**
   *  When a click event is detected on the {@code ColorDisplay} object a color chooser dialog is opened.
   *
   *  @param e The mouse event that triggered this action.
   **/
  public void mouseClicked(MouseEvent e) {
    Color pNewColor = JColorChooser.showDialog(this, m_pColorChooserTitle, m_pColor); 
    setColor(pNewColor);
  }
  
  /**
   *  Not used at the moment.
   *
   *  @param e The mouse event that triggered this action.
   **/
  public void mouseEntered(MouseEvent e) {}
  
  /**
   *  Not used at the moment.
   *
   *  @param e The mouse event that triggered this action.
   **/
  public void mouseExited(MouseEvent e) {}
  
  /**
   *  Not used at the moment.
   *
   *  @param e The mouse event that triggered this action.
   **/
  public void mousePressed(MouseEvent e) {}
  
  /**
   *  Not used at the moment.
   *
   *  @param e The mouse event that triggered this action.
   **/
  public void mouseReleased(MouseEvent e) {}
  
  
  ////////////////////////////////
  // LISTENER METHODS           //
  ////////////////////////////////
  /** 
   *  Adds a listener to this {@code ColorDisplay} object.
   *
   *  @param pListener The listener that should be added to this {@code ColorDisplay} object.
   **/
  public void addColorDisplayListener(IColorDisplayListener pListener) {
    if (pListener == null) return;
    m_pListeners.add(pListener);
  }
  
  /** 
   *  Removes a listener from this {@code ColorDisplay} object.
   *
   *  @param pListener The listener that should be removed from this {@code ColorDisplay} object.
   **/
  public void removeColorDisplayListener(IColorDisplayListener pListener) {
    if (pListener == null) return;
    m_pListeners.remove(pListener);
  }
  
  /** 
   *  Removes all listeners from this {@code ColorDisplay} object.
   *
   **/
  public void removeAllColorDisplayListeners() {
    m_pListeners.clear();
  }
  
  
  ////////////////////////////////
  // PRIVATE METHODS            //
  ////////////////////////////////
  private void fireListenerEvent() {
    int iListenerNum = m_pListeners.size();
    for (int i = 0; i < iListenerNum; i++) {
      m_pListeners.get(i).applyNewColor(this, m_pColor);
    }
  }
  
  
  
  
  
}