package gui.color;

/*
 *  Copyright (C) 2014 Juergen Platzer 
 * 
 *  This file is part of JFXLibrary.
 *  
 *  JFXLibrary is free software: you can redistribute it and/or modify
 *  it under the terms of the GNU Lesser General Public License as published by
 *  the Free Software Foundation, either version 3 of the License, or
 *  (at your option) any later version.
 *  
 *  JFXLibrary is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU Lesser General Public License for more details.
 *  
 *  You should have received a copy of the GNU Lesser General Public License
 *  along with JFXLibrary.  If not, see <http://www.gnu.org/licenses/>.
 */


//gui imports
import javax.swing.JPanel;
import javax.swing.JColorChooser;

import java.awt.Graphics;
import java.awt.Graphics2D;
import java.awt.Color;
import java.awt.Dimension;
import java.awt.GradientPaint;

import java.awt.event.MouseEvent;
import java.awt.event.MouseListener;
import java.awt.event.MouseMotionListener;

import java.util.Vector;


//library imports 
import util.color.ColorGradient;


/**
 *  A panel to show and modify a color gradient.
 *
 **/
public class ColorGradientPanel extends JPanel implements MouseListener, MouseMotionListener {

  private ColorGradient m_pColorGradient;
  private int m_iDraggingColorPointId;
  private Color m_pDraggingColor; //helper for dragging a color point when it is dragged over another color point
  
  /** listeners **/
  private Vector<IColorGradientPanelListener> m_pListeners;
  
  
  //////////////////////////////////
  // CONSTRUCTOR                  //
  //////////////////////////////////
  /**
   *  Creates a {@code ColorGradientPanel} object with a default color gradient. 
   **/
  public ColorGradientPanel() {
    this(null);
  }
  
  /**
   *  Creates a {@code ColorGradientPanel} object showing the specified {@code ColorGradient} object. 
   *
   *  @param pColorGradient The {@code ColorGradient} object that should be visualized and modified by this panel.
   **/
  public ColorGradientPanel(ColorGradient pColorGradient) {
    if (pColorGradient == null) pColorGradient = new ColorGradient();
    m_pColorGradient = pColorGradient;
    addMouseListener(this);
    addMouseMotionListener(this);
    

    m_iDraggingColorPointId = -1;

    m_pListeners = new Vector<IColorGradientPanelListener>();

  }
  
  ////////////////////////////////
  // SET METHODS                //
  ////////////////////////////////
  /**
   *  Sets the {@code ColorGradient} object that should be shown and modified by this panel. 
   *
   *  @param pColorGradient The {@code ColorGradient} object that should be visualized and modified by this panel.
   **/
  public void setColorGradient(ColorGradient pColorGradient) {
    if (pColorGradient == null) return;
    m_pColorGradient = pColorGradient;
    repaint();
  }
  
  ///////////////////////////////
  // GET METHODS               //
  ///////////////////////////////
  /**
   *  Returns the {@code ColorGradient} object that is currently shown in this {@code ColorGradientPanel} object. 
   *
   *  @return The {@code ColorGradient} object that is currently shown in this {@code ColorGradientPanel} object.
   **/
  public ColorGradient getColorGradient() {
    return m_pColorGradient;
  }
  
  //////////////////////////////////
  // DRAWING METHODS              //
  //////////////////////////////////
  /**
   *  Paints the content of this panel: Each color point of the of the color gradient is drawn at its position with a 
   *  marker that is used for the interactions. The colors between the color points are linearly interpolated.
   *
   *  @param g The graphics context that is used to perform the paint operations. 
   **/
  protected void paintComponent(Graphics g) {
    //1. draw the color gradient
    Graphics2D pGraphics = (Graphics2D)g;    
    Dimension pSize = getSize();
    int iWidth = pSize.width;
    int iHeight = pSize.height;
    
    //first position
    Color pColor = m_pColorGradient.getColorOfColorPoint(0);
    double dPosition = m_pColorGradient.getPositionOfColorPoint(0);
    int iPosition = (int)(iWidth*dPosition);
    
    pGraphics.setColor(pColor);
    pGraphics.fillRect(0, 0, iPosition, iHeight);
    
    
    //positions in between
    int iColorPointNum = m_pColorGradient.getColorPointNum();
    for (int i = 1; i < iColorPointNum; i++) {
      Color pNextColor = m_pColorGradient.getColorOfColorPoint(i);
      dPosition = m_pColorGradient.getPositionOfColorPoint(i);
      int iNextPosition = (int)(iWidth*dPosition);
      
      
      GradientPaint pGradient = new GradientPaint((float)iPosition, 0, pColor, (float)iNextPosition, 0, pNextColor);
      pGraphics.setPaint(pGradient);
      pGraphics.fillRect(iPosition, 0, iNextPosition, iHeight);
      
      pColor = pNextColor;
      iPosition = iNextPosition;
    }
    
    
    //last position
    pGraphics.setColor(pColor);
    pGraphics.fillRect(iPosition, 0, iWidth - iPosition, iHeight);
    
 
    //2. draw the color points
    int iColorPointWidth = iHeight/2;   
    int iColorPointWidthHalf = iHeight/4;
    
    for (int i = 0; i < iColorPointNum; i++) {
      pColor = m_pColorGradient.getColorOfColorPoint(i);
      dPosition = m_pColorGradient.getPositionOfColorPoint(i);
      iPosition = (int)(iWidth*dPosition);
      
      pGraphics.setColor(pColor);
      pGraphics.fillRect(iPosition-iColorPointWidthHalf, iColorPointWidthHalf, 
      iColorPointWidth, iColorPointWidth);
      
      float[] pHSBValues = Color.RGBtoHSB(pColor.getRed(), pColor.getGreen(), pColor.getBlue(), null);
      Color pFrameColor = Color.BLACK;
      if (pHSBValues[2] < 0.2) {
        pFrameColor = Color.WHITE;
      }
      
      pGraphics.setColor(pFrameColor);
      pGraphics.drawRect(iPosition-iColorPointWidthHalf, iColorPointWidthHalf, 
      iColorPointWidth, iColorPointWidth);
      
      
    }
    
  }
  
  
  /////////////////////////////////////
  // MOUSE LISTENER METHODS          //
  /////////////////////////////////////
  /**
   *  When a right mouse click is detected on a color point marker, then a color chooser dialog is opened to modify the color. 
   *  When a double click is detected on a color point marker, the color point is removed. 
   *  When a double click is detected outside of all color point markers, then a new color point is inserted with the current color 
   *  at this position.   
   *
   *  @param e The mouse event that triggered this action.
   **/
  public void mouseClicked(MouseEvent e) {
    //1. fetch some information
    int iXPos = e.getX();
    int iYPos = e.getY();
    
    Dimension pSize = getSize();
    int iWidth = pSize.width;
    int iHeight = pSize.height;
    
    
    //2. check if the mouse click was in an existing color point
    int iColorPointId = findSelectedColorPoint(iXPos);
      
    
    //3. if an existing color is clicked 
    if (iColorPointId >= 0) {
      //if it is a double click and the color number is > 2 -> delete the color
      if ((e.getClickCount() == 2) && (m_pColorGradient.getColorPointNum() > 2)) {
        m_pColorGradient.removeColorPoint(iColorPointId);
        repaint();
        fireChangeEvent();
        return;
      }
      
      //if it is a right click -> open a color chooser
      if (e.getButton() != MouseEvent.BUTTON1) {
        Color pColor = m_pColorGradient.getColorOfColorPoint(iColorPointId);
        Color pNewColor = JColorChooser.showDialog(this, "", pColor);
        m_pColorGradient.setColorOfColorPoint(iColorPointId, pNewColor);
        repaint();
        fireChangeEvent();
        return;
      }
      
      //if it is an simple click prepare variables for moving the point
      
    
      
    
    }
    
    
    //4. if no existing color was clicked
    if ((iColorPointId == -1) && (e.getClickCount() == 2)) {
      double dPosition = (double)iXPos/(double)iWidth;
      Color pColor = m_pColorGradient.getColorAtPosition(dPosition);
      m_pColorGradient.addColorPoint(pColor, dPosition);
      repaint();
      // no fire change event necessary, because the color gradient it self has not changed yet
      return;
    }
    
    
  
  }
  
  /**
   *  Not used at the moment.
   *
   *  @param e The mouse event that triggered this action.
   **/
  public void mouseEntered(MouseEvent e) {}
  
  /**
   *  Not used at the moment.
   *
   *  @param e The mouse event that triggered this action.
   **/
  public void mouseExited(MouseEvent e) {}
  
  /**
   *  When the mouse is pressed, variables for dragging a color point are set.
   *
   *  @param e The mouse event that triggered this action.
   **/
  public void mousePressed(MouseEvent e) {
    m_iDraggingColorPointId = findSelectedColorPoint(e.getX());
    m_pDraggingColor = m_pColorGradient.getColorOfColorPoint(m_iDraggingColorPointId);
  }
  
  /**
   *  When the mouse is released, variables for dragging a color point are set back to 
   *  the defaults and the changes are communicated to the listeners.
   *
   *  @param e The mouse event that triggered this action.
   **/
  public void mouseReleased(MouseEvent e) {
    if (m_iDraggingColorPointId > -1) fireChangeEvent();
    m_iDraggingColorPointId = -1;
    m_pDraggingColor = null;
    
  } 
  
  
  ///////////////////////////////////////////
  // MOUSE MOTION LISTENER                 //
  ///////////////////////////////////////////
  /**
   *  When a color point has been selected and the drag event is activated, the new color point 
   *  position set and the color gradient visualization is recalculated.
   *
   *  @param e The mouse event that triggered this action.
   **/
  public void mouseDragged(MouseEvent e) {
    if (m_iDraggingColorPointId > -1) {
      int iXPos = e.getX();
    
      Dimension pSize = getSize();
      int iWidth = pSize.width;

      double dPosition = (double)iXPos/(double)iWidth;
           
      m_pColorGradient.setPositionOfColorPoint(m_iDraggingColorPointId, dPosition);
      
      m_iDraggingColorPointId = m_pColorGradient.getIdOfColorPointWithColor(m_pDraggingColor);
      
      repaint();
    }
  }
  
  /**
   *  Not used at the moment.
   *
   *  @param e The mouse event that triggered this action.
   **/
  public void mouseMoved(MouseEvent e) {
  
  }	
  

  //////////////////////////////////////////
  // PRIVATE HELPER METHODS               //
  //////////////////////////////////////////
  private int findSelectedColorPoint(int iXPos) {
    Dimension pSize = getSize();
    int iWidth = pSize.width;
    int iHeight = pSize.height;
     
    
    int iColorPointWidthHalf = iHeight/4;
    int iColorPointNum = m_pColorGradient.getColorPointNum();
    
    for (int i = 0; i < iColorPointNum; i++) {
      double dPosition = m_pColorGradient.getPositionOfColorPoint(i);
      int iPosition = (int)(iWidth*dPosition);
      
      if ((iXPos >= iPosition - iColorPointWidthHalf) && (iXPos <= iPosition + iColorPointWidthHalf)) {
        return i;
      }
    }
    
    return -1;
    
  }

  //////////////////////////////////////////////
  // LISTENER METHODS                         //
  //////////////////////////////////////////////
  /** 
   *  Adds a listener to this {@code ColorGradientPanel} object.
   *
   *  @param pListener The listener that should be added to this {@code ColorGradientPanel} object.
   **/
  public void addColorGradientPanelListener(IColorGradientPanelListener pListener) {
    if (pListener != null) m_pListeners.add(pListener);   
  }
  
  private void fireChangeEvent() {
    int iListenerNum = m_pListeners.size();
    for (int i = 0; i < iListenerNum; i++) {
      m_pListeners.get(i).colorGradientChanged(this);
    }
  }

}
