package gui.color;

/*
 *  Copyright (C) 2014 Juergen Platzer 
 * 
 *  This file is part of JFXLibrary.
 *  
 *  JFXLibrary is free software: you can redistribute it and/or modify
 *  it under the terms of the GNU Lesser General Public License as published by
 *  the Free Software Foundation, either version 3 of the License, or
 *  (at your option) any later version.
 *  
 *  JFXLibrary is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU Lesser General Public License for more details.
 *  
 *  You should have received a copy of the GNU Lesser General Public License
 *  along with JFXLibrary.  If not, see <http://www.gnu.org/licenses/>.
 */

//gui imports
import javax.swing.JPanel;
import javax.swing.JLabel;
import javax.swing.JSlider;
import javax.swing.JColorChooser;
import javax.swing.JCheckBox;
import javax.swing.JTabbedPane;
import javax.swing.JComboBox;

import javax.swing.event.ChangeListener;
import javax.swing.event.ChangeEvent;
import java.awt.event.ActionListener;
import java.awt.event.ActionEvent;

import java.awt.Color;
import java.awt.GridBagLayout;
import java.awt.GridBagConstraints;
import java.awt.FlowLayout;
import java.awt.Dimension;

import java.awt.image.BufferedImage;

import java.util.Vector;


//Library imports
import standard.EffectParameters;
import color.NoiseGeneratorParameters;
import gui.GridBagLayoutUtilities;
import gui.AbstractPropertiesPanel;
import gui.SlimSliderPanel;
import gui.ISlimSliderPanelListener;
import gui.ImageLoadPanel;
import gui.IImageLoadPanelListener;
import gui.color.ColorDisplay;
import gui.color.IColorDisplayListener;
import applications.Settings;
import applications.SettingsDialog;
import applications.LanguageProperties;

//static imports
import static color.NoiseGeneratorParameters.*;


/**
 *  A properties panel for the noise generator effect.
 **/
public class NoiseGeneratorPropertiesPanel extends AbstractPropertiesPanel 
  implements ISlimSliderPanelListener, IImageLoadPanelListener, ActionListener, ChangeListener {
  
  /*** member variables ***/
  private NoiseGeneratorParameters m_pNoiseGeneratorParameters;

  /*** gui member variables ***/
  private ImageLoadPanel m_pImageLoadPanel;
  private SlimSliderPanel m_pSaltAndPepperIntensitySliderPanel, m_pGaussianIntensitySliderPanel;
  private SlimSliderPanel m_pGaussianStandardDeviationSliderPanel, m_pRandomColorIntensitySliderPanel;
  private JPanel m_pNoiseTypesPanel;
  private JCheckBox m_pUseSaltAndPepperNoise, m_pUseGaussianNoise, m_pUseRandomColorNoise;
  
  
  private JPanel m_pCurrentPropertyPanel;
  
  ////////////////////////////////
  // CONSTRUCTOR                //
  ////////////////////////////////
  /**
   *  Creates a {@code NoiseGeneratorPropertiesPanel} object with the default parameters 
   *  of the {@code NoiseGenerator} effect.
   **/
  public NoiseGeneratorPropertiesPanel() {
    //0. init members
    m_pNoiseGeneratorParameters = new NoiseGeneratorParameters();
    
    //1. set the layout
    setLayout(new FlowLayout(FlowLayout.LEFT, 10, 0));
    
    
    
   
    
    //3. load image
    m_pImageLoadPanel = new ImageLoadPanel();
    m_pImageLoadPanel.addImageLoadPanelListener(this);
    m_pCurrentPropertyPanel = m_pImageLoadPanel;
    
    //4. noise types
    Settings pSettings = SettingsDialog.getSettings();
    LanguageProperties pLanguageProperties = new LanguageProperties(pSettings.getProperty("Language"));

    m_pNoiseTypesPanel = createNoiseTypesPanel(pLanguageProperties);
    
    
    //5. salt and pepper intensity
    m_pSaltAndPepperIntensitySliderPanel = new SlimSliderPanel(0, 100, 50);
    m_pSaltAndPepperIntensitySliderPanel.addSlimSliderPanelListener(this);
    
    //6. gaussian intensity
    m_pGaussianIntensitySliderPanel = new SlimSliderPanel(0, 100, 50);
    m_pGaussianIntensitySliderPanel.addSlimSliderPanelListener(this);
    
    //7. gaussian standard deviation
    m_pGaussianStandardDeviationSliderPanel = new SlimSliderPanel(0, 255, 128);
    m_pGaussianStandardDeviationSliderPanel.addSlimSliderPanelListener(this);
    
    //8. random color intensity
    m_pRandomColorIntensitySliderPanel = new SlimSliderPanel(0, 100, 50);
    m_pRandomColorIntensitySliderPanel.addSlimSliderPanelListener(this);

   
    
     //2. create the combo box with the properties
    
    
    JComboBox pPropertiesSelection = initPropertiesComboBox(this);
    add(pPropertiesSelection);
    populatePropertiesSelection(pLanguageProperties);
    
    
    

    
    //finally make new settings
    //applyNewSettings(SettingsDialog.getSettings());
    
  }
  
  //////////////////////////////////////
  // PRIVATE PANEL GENERATION METHODS //
  //////////////////////////////////////
  private JPanel createNoiseTypesPanel(LanguageProperties pLanguageProperties) {
    //1. create the panel and set the layout
    JPanel pPanel = new JPanel();
    pPanel.setLayout(new FlowLayout());
    
    //2. add the checkboxes
    //2.1. salt & pepper noise
    m_pUseSaltAndPepperNoise = new JCheckBox(pLanguageProperties.getProperty("NoiseGenerator.UseSaltAndPepperNoise"));
    m_pUseSaltAndPepperNoise.setSelected(true);
    m_pUseSaltAndPepperNoise.addChangeListener(this);
    pPanel.add(m_pUseSaltAndPepperNoise);
    
    //2.2. gaussian noise
    m_pUseGaussianNoise = new JCheckBox(pLanguageProperties.getProperty("NoiseGenerator.UseGaussianNoise"));
    m_pUseGaussianNoise.addChangeListener(this);
    pPanel.add(m_pUseGaussianNoise);
    
    //2.3. random color noise
    m_pUseRandomColorNoise = new JCheckBox(pLanguageProperties.getProperty("NoiseGenerator.UseRandomColorNoise"));
    m_pUseRandomColorNoise.addChangeListener(this);
    pPanel.add(m_pUseRandomColorNoise);
    
    
    return pPanel; 
  }
 
  ///////////////////////////
  // GET METHODS           //
  ///////////////////////////
  /**
   *  Returns the {@code NoiseGeneratorParameters} object representing the effect parameters of this panel.
   *
   *  @return The {@code NoiseGeneratorParameters} object representing the effect parameters of this panel.
   **/
  public EffectParameters getEffectParameters() {
    return m_pNoiseGeneratorParameters;
  }
  
  ///////////////////////////
  // SET METHODS           //
  ///////////////////////////
  /**
   *  Sets the effect parameters that should be represented by this {@code NoiseGeneratorPropertiesPanel} object. 
   *  The set operation is only successful if a {@code NoiseGeneratorParameters} object has been specified.
   *
   *  @param pParameters The {@code NoiseGeneratorParameters} object representing the effect parameters 
   *                     that should be represented by this {@code NoiseGeneratorPropertiesPanel} object.
   **/
  public void setEffectParameters(EffectParameters pParameters) {
    //1. make some checks
    if (pParameters == null) return;
    
    if (pParameters instanceof NoiseGeneratorParameters) {
      m_pNoiseGeneratorParameters = (NoiseGeneratorParameters)pParameters;
      
      updateControls();
    }
  }
  
  
  
  ////////////////////////////////////////
  // SLIM SLIDER PANEL LISTENER         //
  ////////////////////////////////////////
  /**
   *  If the intensities of the noises have been adapted in the corresponding slider panel, it is 
   *  communicated with this method, the effect parameters are adapted and the listeners are notified.
   *
   *  @param pSliderPanel The {@code SlimSliderPanel} object that has been modified.
   *  @param iValue The value of the slider panel that has been modified. 
   **/
  public void applyChanges(SlimSliderPanel pSliderPanel, int iValue) {
  
    //1. make some simple checks
    if (pSliderPanel == null) return;
    if (m_pNoiseGeneratorParameters == null) m_pNoiseGeneratorParameters = new NoiseGeneratorParameters();
    
    //2. set the new parameters
    if (pSliderPanel.equals(m_pSaltAndPepperIntensitySliderPanel)) {
      //double dValue = (double)iValue/100;
      //System.out.println(dValue);
      m_pNoiseGeneratorParameters.setDoubleParameterValue(PARAM_DOUBLE_SALT_AND_PEPPER_INTENSITY, (double)iValue/100);
    }
    
    if (pSliderPanel.equals(m_pGaussianIntensitySliderPanel)) {
      m_pNoiseGeneratorParameters.setDoubleParameterValue(PARAM_DOUBLE_GAUSSIAN_INTENSITY, (double)iValue/100);
    }
    
    if (pSliderPanel.equals(m_pGaussianStandardDeviationSliderPanel)) {
      m_pNoiseGeneratorParameters.setDoubleParameterValue(PARAM_DOUBLE_GAUSSIAN_STANDARD_DEVIATION, (double)iValue);
    }
    
    if (pSliderPanel.equals(m_pRandomColorIntensitySliderPanel)) {
      m_pNoiseGeneratorParameters.setDoubleParameterValue(PARAM_DOUBLE_RANDOM_COLOR_INTENSITY, (double)iValue/100);
    }
    
 
    
    //2. fire the apply changes event
    fireApplyChanges(m_pNoiseGeneratorParameters);
  }
  
  
  
  
  
  
  /////////////////////////////////////////
  // IMAGE LOAD PANEL LISTENER           //
  /////////////////////////////////////////
  /**
   *  If the image to which the effect should be applied is changed, this is communicated with this method, 
   *  the effect parameters are adapted and the listeners are notified.
   *
   *  @param pPanel The {@code ImageLoadPanel} object that steers the image to which the effect is applied.
   *  @param pImage The image to which the effect should be applied.  
   **/
  public void applyChanges(ImageLoadPanel pPanel, BufferedImage pImage) {
    //0. make a simple check
    if (pImage == null) return;
    if (m_pNoiseGeneratorParameters == null) m_pNoiseGeneratorParameters = new NoiseGeneratorParameters();
    
    //1. set the new colors
    m_pNoiseGeneratorParameters.setObjectParameterValue(PARAM_OBJECT_IMAGE, pImage);
    
    //2. fire the apply changes event
    fireApplyChanges(m_pNoiseGeneratorParameters);
  }
  
  /////////////////////////////////////////
  // ACTION LISTENER                     //
  /////////////////////////////////////////
  /**
   *  This sets the new parameter panel that should be modified. The parameter is chosen by a combo box. 
   *
   *  @param pEvent The action event triggering this method. 
   **/
  public void actionPerformed(ActionEvent pEvent) {
    
    //1. property selection
    //Object pSource = pEvent.getSource();
    propertiesSelectionActionPerformed();
    
    
    
  }
  
  
 
 
  
  /////////////////////////////////////////
  // CHANGE LISTENER                     //
  /////////////////////////////////////////
  /**
   *  If the checkboxes have been modified that steer which noise type is used, this is communicated with this method. 
   *  The effect parameters are adapted and the listeners are notified. 
   *
   *  @param e The action event triggering this method. 
   **/
  public void stateChanged(ChangeEvent e) {
    //0. make a simple check
    if (m_pNoiseGeneratorParameters == null) m_pNoiseGeneratorParameters = new NoiseGeneratorParameters();
  
    //1. set the checkbox flags
    boolean bRepaint = false;
    if (m_pUseSaltAndPepperNoise.isSelected() != m_pNoiseGeneratorParameters.getBooleanParameterValue(PARAM_BOOLEAN_CREATE_SALT_AND_PEPPER_NOISE)) {
      m_pNoiseGeneratorParameters.setBooleanParameterValue(PARAM_BOOLEAN_CREATE_SALT_AND_PEPPER_NOISE, m_pUseSaltAndPepperNoise.isSelected());
      bRepaint = true;
    }
    
    if (m_pUseGaussianNoise.isSelected() != m_pNoiseGeneratorParameters.getBooleanParameterValue(PARAM_BOOLEAN_CREATE_GAUSSIAN_NOISE)) {
      m_pNoiseGeneratorParameters.setBooleanParameterValue(PARAM_BOOLEAN_CREATE_GAUSSIAN_NOISE, m_pUseGaussianNoise.isSelected());
      bRepaint = true;
    }
      
    if (m_pUseRandomColorNoise.isSelected() != m_pNoiseGeneratorParameters.getBooleanParameterValue(PARAM_BOOLEAN_CREATE_RANDOM_COLOR_NOISE)) {
      m_pNoiseGeneratorParameters.setBooleanParameterValue(PARAM_BOOLEAN_CREATE_RANDOM_COLOR_NOISE, m_pUseRandomColorNoise.isSelected());
      bRepaint = true;
    }
    
    //2. fire the apply changes event
    if (bRepaint) fireApplyChanges(m_pNoiseGeneratorParameters);
  }
  
  
  
  ////////////////////////////////
  // SETTINGS LISTENER METHODS  //
  ////////////////////////////////
  /**
   *  Applies the changes of the settings to the application. For example when the language is changed, 
   *  all GUI elements are updated accordingly.
   *
   *  @param pSettings The new {@code Settings} object that is used to retrieve all labels. 
   **/
  public void applyNewSettings(Settings pSettings) {
  
    //1. make an initial check and set the settings
    if (pSettings == null) return;
    
    //2. load new language settings
    LanguageProperties pLanguageProperties = new LanguageProperties(pSettings.getProperty("Language"));
    populatePropertiesSelection(pLanguageProperties);
    
 
    
    //4. image load
    if (m_pImageLoadPanel != null) {
      m_pImageLoadPanel.applyNewSettings(pSettings);
    }
    
    //5. check boxes
    m_pUseSaltAndPepperNoise.setText(pLanguageProperties.getProperty("NoiseGenerator.UseSaltAndPepperNoise"));
    m_pUseGaussianNoise.setText(pLanguageProperties.getProperty("NoiseGenerator.UseGaussianNoise"));
    m_pUseRandomColorNoise.setText(pLanguageProperties.getProperty("NoiseGenerator.UseRandomColorNoise"));
    
    
  }
  
  
  
  //////////////////////////////////
  // PRIVATE HELPER METHODS       //
  //////////////////////////////////
  private void updateControls() {
    //1. make some checks
    if (m_pNoiseGeneratorParameters == null) return;
    
    //2. TO DO: update all controls according to the settings 
    //   in the effect parameters and make sure that no action is fired!
    //m_pBrickColorDisplay.setColor(m_pNoiseGeneratorParameters.getColorParameterValue(PARAM_COLOR_BRICK));
  }
  
  
  private void propertiesSelectionActionPerformed() {
  
    //1. get the selected property
    
    
    //2. remove the current property panel
    if (m_pCurrentPropertyPanel != null) remove(m_pCurrentPropertyPanel);
    
    //3. set the new property panel
    int iSelectedId = getSelectedPropertiesIndex();
    if (iSelectedId == 0) {
      m_pCurrentPropertyPanel = m_pImageLoadPanel;
    }
    else if (iSelectedId == 1) {
      m_pCurrentPropertyPanel = m_pNoiseTypesPanel;
    }
    else if (iSelectedId == 2) {
      m_pCurrentPropertyPanel = m_pSaltAndPepperIntensitySliderPanel;
    }
    else if (iSelectedId == 3) {
      m_pCurrentPropertyPanel = m_pGaussianIntensitySliderPanel;
    }
    else if (iSelectedId == 4) {
      m_pCurrentPropertyPanel = m_pGaussianStandardDeviationSliderPanel;
    }
    else if (iSelectedId == 5) {
      m_pCurrentPropertyPanel = m_pRandomColorIntensitySliderPanel;
    }
    
  
    add(m_pCurrentPropertyPanel);
    
    repaint();
    revalidate();
    
    
  }
  
  private void populatePropertiesSelection(LanguageProperties pLanguageProperties) {
    
    
    Vector<String> pPropertyKeys = new Vector<String>();
    pPropertyKeys.add("NoiseGenerator.Image");
    pPropertyKeys.add("NoiseGenerator.NoiseTypes");
    pPropertyKeys.add("NoiseGenerator.SaltAndPepperIntensity");
    pPropertyKeys.add("NoiseGenerator.GaussianIntensity");
    pPropertyKeys.add("NoiseGenerator.GaussianStandardDeviation");
    pPropertyKeys.add("NoiseGenerator.RandomColorIntensity");
    populatePropertiesSelection(pLanguageProperties, pPropertyKeys);
    
    
  }
  
  
  
  
  
}