package gui.drawing;

/*
 *  Copyright (C) 2014 Juergen Platzer 
 * 
 *  This file is part of JFXLibrary.
 *  
 *  JFXLibrary is free software: you can redistribute it and/or modify
 *  it under the terms of the GNU Lesser General Public License as published by
 *  the Free Software Foundation, either version 3 of the License, or
 *  (at your option) any later version.
 *  
 *  JFXLibrary is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU Lesser General Public License for more details.
 *  
 *  You should have received a copy of the GNU Lesser General Public License
 *  along with JFXLibrary.  If not, see <http://www.gnu.org/licenses/>.
 */

import javax.swing.JPanel;

import java.awt.event.MouseListener;
import java.awt.event.MouseMotionListener;
import java.awt.event.MouseEvent;

import java.awt.Dimension;
import java.awt.Graphics;
import java.awt.Graphics2D;
import java.awt.Point;
import java.awt.Color;

import java.awt.image.BufferedImage;

//library imports
import standard.EffectParameters;
import drawing.Stars;
import drawing.StarsParameters;
import gui.AbstractImageEffectPanel;
import gui.AbstractInteractionImageEffectPanel;
import gui.AbstractPropertiesPanel;

import static fractals.MandelbrotSetParameters.*;

/**
 *  Handles the interaction with a Star drawing.
 **/

public class StarsPanel extends AbstractInteractionImageEffectPanel implements MouseListener, MouseMotionListener {

  /** member variables **/
  private StarsParameters m_pParameters = null;
  private Stars m_pStars = null;  
  private BufferedImage m_pImage = null;
  private StarsPropertiesPanel m_pStarsPropertiesPanel = null;
  
  /*** star parameters that are steered in the panel ***/
  private int m_iWidth, m_iHeight;
  
  /** interaction members **/
  
  ////////////////////////////////////
  // CONSTRUCTOR                    //
  ////////////////////////////////////
  /**
   *  Creates a {@code StarsPanel} object with the default settings for the Stars effect.
   **/
  public StarsPanel() {
    
    m_pParameters = new StarsParameters();
    //setMandelbrotSetSize();
    //setDefaultMandelbrotSet();
    
    m_pStars = new Stars();
    initEffectRepaint();
    m_pStarsPropertiesPanel = null;
    
    m_iWidth = 400;
    m_iHeight = 400;
    
    addMouseListener(this);
    addMouseMotionListener(this);
  }

  
  ///////////////////////////////////////
  // SET METHODS                       //
  ///////////////////////////////////////
  /**
   *  Sets the effect parameters for the {@code Stars} effect that is shown by this {@code StarsPanel} object. 
   *
   *  @param pParameters The {@code StarsParameters} object representing the Stars effect parameters. 
   **/
  public void setStarsParameters(StarsParameters pParameters) {
    if (pParameters == null) return;
    
    m_pParameters = pParameters;
    
    m_pParameters.setIntegerParameterValue(PARAM_INT_WIDTH, m_iWidth);
    m_pParameters.setIntegerParameterValue(PARAM_INT_HEIGHT, m_iHeight);
    initEffectRepaint();
  }
  
  /**
   *  Sets the effect parameters for the {@code Stars} effect that is shown by this {@code StarsPanel} object. 
   *  The set operation is only successful if a {@code StarsParameters} object has been specified.
   *
   *  @param pEffectParameters The {@code StarsParameters} object representing the Stars effect parameters. 
   **/
  public void setEffectParameters(EffectParameters pEffectParameters) {
    if (pEffectParameters == null) return;
    
    if (pEffectParameters instanceof StarsParameters) {
      setStarsParameters((StarsParameters)pEffectParameters);
    }
    
  }
  
  /**
   *  Sets the size of the effect. (TO DO: Should this be made with the SizingPanel listener interface?)
   *
   *  @param iWidth The width in pixels of the effect image.
   *  @param iHeight The height in pixels of the effect image.
   **/
  public void setEffectSize(int iWidth, int iHeight) {
    m_pParameters.setIntegerParameterValue(PARAM_INT_WIDTH, iWidth);
    m_pParameters.setIntegerParameterValue(PARAM_INT_HEIGHT, iHeight);
    m_iWidth = iWidth;
    m_iHeight = iHeight;
    initEffectRepaint();
  }
  
  ///////////////////////////////////////
  // GET METHODS                       //
  ///////////////////////////////////////
  /**
   *  Returns the {@code StarsParameters} object representing the effect parameters of this panel.
   *
   *  @return The {@code StarsParameters} object representing the effect parameters of this panel.
   **/
  public StarsParameters getStarsParameters() {
    return m_pParameters;
  }
  
  /**
   *  Returns the {@code StarsParameters} object representing the effect parameters of this panel.
   *
   *  @return The {@code StarsParameters} object representing the effect parameters of this panel.
   **/
  public EffectParameters getEffectParameters() {
    return getStarsParameters();
  }
  
  /**
   *  Returns the {@code StarsPropertiesPanel} object representing the effect parameters and providing a GUI to modify them.
   *
   *  @return The {@code StarsPropertiesPanel} object representing the effect parameters and providing a GUI to modify them.
   **/
  public AbstractPropertiesPanel getPropertiesPanel() {
    if (m_pStarsPropertiesPanel == null) {
      m_pStarsPropertiesPanel = new StarsPropertiesPanel();
      m_pStarsPropertiesPanel.setEffectParameters(getStarsParameters());
      m_pStarsPropertiesPanel.addPropertiesPanelListener(this);
    }
    
    return m_pStarsPropertiesPanel;
  }
  
  
  //////////////////////////////////////
  // DRAWING HELPER METHODS           //
  //////////////////////////////////////
  protected void drawInteraction(Graphics2D pGraphics) {
    //nothing to do at the moment
  }


  
  
  
  

  /////////////////////////////////
  // MOUSE LISTENER INTERFACE    //
  /////////////////////////////////
  //TO DO: implement the stars interaction
  public void mouseClicked(MouseEvent e) {}
  public void mouseEntered(MouseEvent e) {}
  public void mouseExited(MouseEvent e) {}
  public void mousePressed(MouseEvent e) {}
  public void mouseReleased(MouseEvent e) {}
    
    
  /////////////////////////////////////
  // MOUSE MOTION LISTENER INTERFACE //
  /////////////////////////////////////
  
  public void mouseDragged(MouseEvent e) {}
  public void mouseMoved(MouseEvent e) {}
  
  
  /////////////////////////////////
  // PRIVATE HELPER METHODS      //
  /////////////////////////////////
  private void initEffectRepaint() {
    if (m_pStars == null) return;
    
    m_pStars.setEffectParameters(m_pParameters);
    m_pStars.processAsThread(this);
    
  }
}