package gui.drawing;

/*
 *  Copyright (C) 2014 Juergen Platzer 
 * 
 *  This file is part of JFXLibrary.
 *  
 *  JFXLibrary is free software: you can redistribute it and/or modify
 *  it under the terms of the GNU Lesser General Public License as published by
 *  the Free Software Foundation, either version 3 of the License, or
 *  (at your option) any later version.
 *  
 *  JFXLibrary is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU Lesser General Public License for more details.
 *  
 *  You should have received a copy of the GNU Lesser General Public License
 *  along with JFXLibrary.  If not, see <http://www.gnu.org/licenses/>.
 */

//gui imports
import javax.swing.JPanel;
import javax.swing.JLabel;
import javax.swing.JSlider;
import javax.swing.JColorChooser;
import javax.swing.JComboBox;

import javax.swing.event.ChangeListener;
import javax.swing.event.ChangeEvent;

import java.awt.Color;
import java.awt.GridBagLayout;
import java.awt.GridBagConstraints;
import java.awt.FlowLayout;
import java.awt.Dimension;

import java.awt.event.ActionListener;
import java.awt.event.ActionEvent;

import java.util.Vector;


//Library imports
import standard.EffectParameters;
import drawing.StarsParameters;
import gui.GridBagLayoutUtilities;
import gui.AbstractPropertiesPanel;
import gui.SlimSliderPanel;
import gui.ISlimSliderPanelListener;
import gui.color.ColorDisplay;
import gui.color.IColorDisplayListener;
import applications.Settings;
import applications.SettingsDialog;
import applications.LanguageProperties;

//static imports
import static drawing.StarsParameters.*;


/**
 *  A properties panel for the stars effect.
 **/
public class StarsPropertiesPanel extends AbstractPropertiesPanel 
  implements ISlimSliderPanelListener, IColorDisplayListener, ActionListener {
  
  /*** member variables ***/
  private StarsParameters m_pStarsParameters;

  /*** gui member variables ***/
  private SlimSliderPanel m_pPikeNumSliderPanel, m_pPikeLengthSliderPanel, m_pLineNumberSliderPanel;
  private SlimSliderPanel m_pXTranslateSliderPanel, m_pYTranslateSliderPanel, m_pRotationSliderPanel;
  private ColorDisplay m_pLineColorDisplay, m_pBackgroundColorDisplay;
  
  private JPanel m_pCurrentPropertyPanel;
  
  ////////////////////////////////
  // CONSTRUCTOR                //
  ////////////////////////////////
  /**
   *  Creates a {@code StarsPropertiesPanel} object with the default parameters 
   *  of the {@code Stars} effect.
   **/
  public StarsPropertiesPanel() {
    //0. init members
    m_pStarsParameters = null;
    
    //1. set the layout
    setLayout(new FlowLayout(FlowLayout.LEFT, 10, 0));
    
    
    
    
    
    
    //2. create the panel for the pike nums
    m_pPikeNumSliderPanel = new SlimSliderPanel(3, 20, 4);
    m_pPikeNumSliderPanel.addSlimSliderPanelListener(this);
    m_pCurrentPropertyPanel = m_pPikeNumSliderPanel;
    
    //add(m_pPikeNumSliderPanel, pGridBagConstraints);
    //pGridBagConstraints.gridy += 2; //increase the y cell coordinate (increase twice because slider panel takes two cells)
    
    
    //3. create the panel for the pike length
    m_pPikeLengthSliderPanel = new SlimSliderPanel(20, 1000, 180);
    m_pPikeLengthSliderPanel.addSlimSliderPanelListener(this);
    //add(m_pPikeLengthSliderPanel, pGridBagConstraints);
    //pGridBagConstraints.gridy += 2; //increase the y cell coordinate (increase twice because slider panel takes two cells)
    

    //3. create the line number panel
    m_pLineNumberSliderPanel = new SlimSliderPanel(5, 100, 10);
    m_pLineNumberSliderPanel.addSlimSliderPanelListener(this);
    //add(m_pLineNumberSliderPanel, pGridBagConstraints);
    //pGridBagConstraints.gridy += 2; //increase the y cell coordinate (increase twice because slider panel takes two cells)
    
      
      
    //4. create the line color panel  
    m_pLineColorDisplay = new ColorDisplay();
    m_pLineColorDisplay.addColorDisplayListener(this);
    //add(m_pLineColorDisplay, pGridBagConstraints);
    
    //5. create background color panel
    m_pBackgroundColorDisplay = new ColorDisplay();
    m_pBackgroundColorDisplay.addColorDisplayListener(this);
    
    //6. create the horizontal translation slider panel
    m_pXTranslateSliderPanel = new SlimSliderPanel(-1000, 1000, 0);
    m_pXTranslateSliderPanel.addSlimSliderPanelListener(this);
    
    //7. create the vertical translation slider panel
    m_pYTranslateSliderPanel = new SlimSliderPanel(-1000, 1000, 0);
    m_pYTranslateSliderPanel.addSlimSliderPanelListener(this);
    
    //8. create the rotation slider panel
    m_pRotationSliderPanel = new SlimSliderPanel(0, 360, 0);
    m_pRotationSliderPanel.addSlimSliderPanelListener(this);

    
    //finally make new settings
    //applyNewSettings(SettingsDialog.getSettings());
    
    Settings pSettings = SettingsDialog.getSettings();
    LanguageProperties pLanguageProperties = new LanguageProperties(pSettings.getProperty("Language"));
    
    
    JComboBox pPropertiesSelection = initPropertiesComboBox(this);
    add(pPropertiesSelection);
    populatePropertiesSelection(pLanguageProperties);
    
    //add(m_pPikeNumSliderPanel);
    
    
  }
  
  
  ///////////////////////////
  // GET METHODS           //
  ///////////////////////////
  /**
   *  Returns the {@code StarsParameters} object representing the effect parameters of this panel.
   *
   *  @return The {@code StarsParameters} object representing the effect parameters of this panel.
   **/
  public EffectParameters getEffectParameters() {
    return m_pStarsParameters;
  }
  
  ///////////////////////////
  // SET METHODS           //
  ///////////////////////////
  /**
   *  Sets the effect parameters that should be represented by this {@code StarsPropertiesPanel} object. 
   *  The set operation is only successful if a {@code StarsParameters} object has been specified.
   *
   *  @param pParameters The {@code StarsParameters} object representing the effect parameters 
   *                     that should be represented by this {@code StarsPropertiesPanel} object.
   **/
  public void setEffectParameters(EffectParameters pParameters) {
    //1. make some checks
    if (pParameters == null) return;
    
    if (pParameters instanceof StarsParameters) {
      m_pStarsParameters = (StarsParameters)pParameters;
      
      updateControls();
    }
  }
  

  
  
  ///////////////////////////////////////
  // SLIM SLIDER PANEL LISTENER        //
  ///////////////////////////////////////
  /**
   *  If the number of spikes, the spike length, the line number, the translation or rotation values have been adapted 
   *  in the corresponding slider panel, it is 
   *  communicated with this method, the effect parameters are adapted and the listeners are notified.
   *
   *  @param pSliderPanel The {@code SlimSliderPanel} object that has been modified.
   *  @param iValue The value of the slider panel that has been modified. 
   **/
  public void applyChanges(SlimSliderPanel pSliderPanel, int iValue) {
  
    //1. make some simple checks
    if (pSliderPanel == null) return;
    if (m_pStarsParameters == null) m_pStarsParameters = new StarsParameters();
    
    //1. set the new parameters
    if (pSliderPanel.equals(m_pPikeNumSliderPanel)) {
      m_pStarsParameters.setIntegerParameterValue(PARAM_INT_SPIKE_NUM, iValue);
    }
    
    
    if (pSliderPanel.equals(m_pPikeLengthSliderPanel)) {
      m_pStarsParameters.setIntegerParameterValue(PARAM_INT_SPIKE_LENGTH, iValue);
    }
    
    if (pSliderPanel.equals(m_pLineNumberSliderPanel)) {
      m_pStarsParameters.setIntegerParameterValue(PARAM_INT_LINE_NUM, iValue);
    } 
    
    if (pSliderPanel.equals(m_pXTranslateSliderPanel)) {
      m_pStarsParameters.setIntegerParameterValue(PARAM_INT_HORIZONTAL_TRANSLATION, iValue);
    }
    
    if (pSliderPanel.equals(m_pYTranslateSliderPanel)) {
      m_pStarsParameters.setIntegerParameterValue(PARAM_INT_VERTICAL_TRANSLATION, iValue);
    }
    
    if (pSliderPanel.equals(m_pRotationSliderPanel)) {
      m_pStarsParameters.setIntegerParameterValue(PARAM_INT_ROTATION, iValue);
    }
    
    
    //2. fire the apply changes event
    fireApplyChanges(m_pStarsParameters);
  }
  
  
  /////////////////////////////////////////
  // ICOLOR PANEL LISTENER               //
  /////////////////////////////////////////
  /**
   *  If the color of the lines has been changed, this is communicated with this method, 
   *  the effect parameters are adapted and the listeners are notified.
   *
   *  @param pColorDisplay The {@code ColorDisplay} object that has been changed.
   *  @param pColor The color that has been set for the {@code ColorDisplay} object. 
   **/
  public void applyNewColor(ColorDisplay pColorDisplay, Color pColor) {
    //0. make a simple check
    if (m_pStarsParameters == null) m_pStarsParameters = new StarsParameters();
    
    //1. set the new color
    if (pColorDisplay.equals(m_pLineColorDisplay)) {
      m_pStarsParameters.setColorParameterValue(PARAM_COLOR_LINE_COLOR, pColor);
    }
    else {
      m_pStarsParameters.setColorParameterValue(PARAM_COLOR_BACKGROUND, pColor);
    }
    
    
    
    //2. fire the apply changes event
    fireApplyChanges(m_pStarsParameters);
  }
  
  
  ////////////////////////////////
  // SETTINGS LISTENER METHODS  //
  ////////////////////////////////
  /**
   *  Applies the changes of the settings to the application. For example when the language is changed, 
   *  all GUI elements are updated accordingly.
   *
   *  @param pSettings The new {@code Settings} object that is used to retrieve all labels. 
   **/
  public void applyNewSettings(Settings pSettings) {
  
    //1. make an initial check and set the settings
    if (pSettings == null) return;
    
    //2. load new language settings
    LanguageProperties pLanguageProperties = new LanguageProperties(pSettings.getProperty("Language"));
    
    //3. set the text of the labels
    populatePropertiesSelection(pLanguageProperties);
    
  }
  
  
  ////////////////////////////////
  // ACTION LISTENER METHODS    //
  ////////////////////////////////
  /**
   *  Detects which parameter should be shown now in the properties panel.
   *
   *  @param pEvent The action event triggering this method. 
   **/
  public void actionPerformed(ActionEvent pEvent) {
  
    
    
    //2. remove the current property panel
    if (m_pCurrentPropertyPanel != null) remove(m_pCurrentPropertyPanel);
    
    //3. set the new property panel
    int iSelectedId = getSelectedPropertiesIndex();
    if (iSelectedId == 0) {
      m_pCurrentPropertyPanel = m_pPikeNumSliderPanel;
    }
    else if (iSelectedId == 1) {
      m_pCurrentPropertyPanel = m_pPikeLengthSliderPanel;
    }
    else if (iSelectedId == 2) {
      m_pCurrentPropertyPanel = m_pLineNumberSliderPanel;
    }
    else if (iSelectedId == 3) {
      m_pCurrentPropertyPanel = m_pLineColorDisplay;
    }
    else if (iSelectedId == 4) {
      m_pCurrentPropertyPanel = m_pBackgroundColorDisplay;
    }
    else if (iSelectedId == 5) {
      m_pCurrentPropertyPanel = m_pXTranslateSliderPanel;
    }
    else if (iSelectedId == 6) {
      m_pCurrentPropertyPanel = m_pYTranslateSliderPanel;
    }
    else if (iSelectedId == 7) {
      m_pCurrentPropertyPanel = m_pRotationSliderPanel;
    }
    
    add(m_pCurrentPropertyPanel);
    
    repaint();
    revalidate();
    
    
  }
  
  
  //////////////////////////////////
  // PRIVATE HELPER METHODS       //
  //////////////////////////////////
  private void updateControls() {
    //1. make some checks
    if (m_pStarsParameters == null) return;
    
    //2. TO DO: update all controls according to the settings 
    //   in the effect parameters and make sure that no action is fired!
    m_pLineColorDisplay.setColor(m_pStarsParameters.getColorParameterValue(PARAM_COLOR_LINE_COLOR));
    m_pBackgroundColorDisplay.setColor(m_pStarsParameters.getColorParameterValue(PARAM_COLOR_BACKGROUND));
  }
  
  private void populatePropertiesSelection(LanguageProperties pLanguageProperties) {
    
    
    Vector<String> pPropertyKeys = new Vector<String>();
    pPropertyKeys.add("Stars.PikeNum");
    pPropertyKeys.add("Stars.PikeLength");
    pPropertyKeys.add("Stars.LineNum");
    pPropertyKeys.add("Stars.LineColor");
    pPropertyKeys.add("Stars.BackgroundColor");
    pPropertyKeys.add("Stars.HorTrans");
    pPropertyKeys.add("Stars.VerTrans");
    pPropertyKeys.add("Stars.Rotation");
    populatePropertiesSelection(pLanguageProperties, pPropertyKeys);
    
    
  }
  
  
  
  
}