package gui.fractals;

/*
 *  Copyright (C) 2014 Juergen Platzer 
 * 
 *  This file is part of JFXLibrary.
 *  
 *  JFXLibrary is free software: you can redistribute it and/or modify
 *  it under the terms of the GNU Lesser General Public License as published by
 *  the Free Software Foundation, either version 3 of the License, or
 *  (at your option) any later version.
 *  
 *  JFXLibrary is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU Lesser General Public License for more details.
 *  
 *  You should have received a copy of the GNU Lesser General Public License
 *  along with JFXLibrary.  If not, see <http://www.gnu.org/licenses/>.
 */

import javax.swing.JPanel;

import java.awt.event.MouseListener;
import java.awt.event.MouseMotionListener;
import java.awt.event.MouseEvent;

import java.awt.Dimension;
import java.awt.Graphics;
import java.awt.Graphics2D;
import java.awt.Point;
import java.awt.Color;

import java.awt.image.BufferedImage;

//library imports
import standard.EffectParameters;
import fractals.MandelbrotSet;
import fractals.MandelbrotSetParameters;
import util.math.ComplexNumber;
import gui.AbstractInteractionImageEffectPanel;
import gui.AbstractPropertiesPanel;
import gui.ISizingPanelListener;
import gui.SizingPanel;

import static fractals.MandelbrotSetParameters.*;
import static gui.SizeModePanel.*;

/**
 *  Handles the interaction with a Mandelbrot set.
 **/

public class MandelbrotSetPanel extends AbstractInteractionImageEffectPanel 
  implements MouseListener, MouseMotionListener, ISizingPanelListener {

  
  
  /** member variables **/
  private MandelbrotSetParameters m_pParameters = null; //those parameters are shared with the properties panel
  private MandelbrotSet m_pMandelbrotSet = null;  
  private BufferedImage m_pImage = null;
  private int m_iWidth, m_iHeight;
  private MandelbrotSetPropertiesPanel m_pMandelbrotSetPropertiesPanel = null;
  
  /** interaction members **/
  private Point m_pStartPoint = null; 
  private Point m_pEndPoint = null;
  
  
  /** resizing members **/
  private boolean m_bUsePanelSize;
  private int m_iFixedWidth; //used when fixed size
  private int m_iFixedHeight;//used when fixed size
  
  ////////////////////////////////////
  // CONSTRUCTOR                    //
  ////////////////////////////////////
  /**
   *  Creates a {@code MandelbrotSetPanel} object with the default settings for the Mandelbrot set effect.
   **/
  public MandelbrotSetPanel() {
    
    //set resizing members
    m_bUsePanelSize = true;
    m_iFixedWidth = 1;
    m_iFixedHeight = 1;
    
    
    //set members
    m_pParameters = new MandelbrotSetParameters();
    setEffectSize(400, 400);
    setDefaultMandelbrotSet();
    
    m_pMandelbrotSet = new MandelbrotSet();
    initEffectRepaint();
    
    
    
    //add listeners
    addMouseListener(this);
    addMouseMotionListener(this);
  }

  
  ///////////////////////////////////////
  // SET METHODS                       //
  ///////////////////////////////////////
  /**
   *  Sets the effect parameters for the {@code MandelbrotSet} effect that is shown by this {@code MandelbrotSetPanel} object. 
   *
   *  @param pParameters The {@code MandelbrotSetParameters} object representing the Mandelbrot set effect parameters. 
   **/
  public void setMandelbrotSetParameters(MandelbrotSetParameters pParameters) {
    if (pParameters == null) return;
    
    m_pParameters = pParameters;
    initEffectRepaint();
  }
  
  
  /**
   *  Sets the effect parameters for the {@code MandelbrotSet} effect that is shown by this {@code MandelbrotSetPanel} object. 
   *  The set operation is only successful if a {@code MandelbrotSetParameters} object has been specified.
   *
   *  @param pEffectParameters The {@code MandelbrotSetParameters} object representing the Mandelbrot set effect parameters. 
   **/
  public void setEffectParameters(EffectParameters pEffectParameters) {
    if (pEffectParameters == null) return;
    
    if (pEffectParameters instanceof MandelbrotSetParameters) {
      setMandelbrotSetParameters((MandelbrotSetParameters)pEffectParameters);
    }
    
  }
  
  /**
   *  Sets the size of the effect. (TO DO: Should this be made with the SizingPanel listener interface?)
   *
   *  @param iWidth The width in pixels of the effect image.
   *  @param iHeight The height in pixels of the effect image.
   **/
  public void setEffectSize(int iWidth, int iHeight) {
  
    //1. check if we have to resize
    boolean bResize = false;
    if (m_bUsePanelSize) {
      bResize = true; //panel size
    }
    else { //fixed size
      //only if the size has changed
      if ((m_iWidth != m_iFixedWidth) || (m_iHeight != m_iFixedHeight)) {
        bResize = true;
      }
    }
    
    //if there is no resizing quit
    if (!bResize) return;
  
    
    //2. set the new size of the effect
    int iWidthToSet = iWidth;
    int iHeightToSet = iHeight;
    if (!m_bUsePanelSize) {
      iWidthToSet = m_iFixedWidth;
      iHeightToSet = m_iFixedHeight;
    }
    
    m_pParameters.setIntegerParameterValue(PARAM_INT_WIDTH, iWidthToSet);
    m_pParameters.setIntegerParameterValue(PARAM_INT_HEIGHT, iHeightToSet);
    m_iWidth = iWidthToSet;
    m_iHeight = iHeightToSet;
      
      
    //3. recalculate the rectangle of the effect (the window width is constant)
    double dAspectRatio = (double)iHeightToSet/(double)iWidthToSet;
    ComplexNumber pUpperLeft = (ComplexNumber)m_pParameters.getObjectParameterValue(PARAM_OBJECT_UPPER_LEFT_WINDOW_CORNER);
    ComplexNumber pLowerRight = (ComplexNumber)m_pParameters.getObjectParameterValue(PARAM_OBJECT_LOWER_RIGHT_WINDOW_CORNER);
      
    double dWindowWidth = pLowerRight.m_dReal - pUpperLeft.m_dReal;
    double dNewWindowHeight = dWindowWidth*dAspectRatio;
     
    ComplexNumber pNewLowerRight = new ComplexNumber(pLowerRight.m_dReal, pUpperLeft.m_dImaginary-dNewWindowHeight);
    m_pParameters.setObjectParameterValue(PARAM_OBJECT_LOWER_RIGHT_WINDOW_CORNER, pNewLowerRight);
      
      
    //3. clear the image and return
    initEffectRepaint();
 
  
  }
  
  /**
   *  Sets the effect back to the default settings.
   **/
  public void setDefaultMandelbrotSet() {
    double dAspectRatio = (double)m_iHeight/(double)m_iWidth;
    m_pParameters.setObjectParameterValue(PARAM_OBJECT_UPPER_LEFT_WINDOW_CORNER, new ComplexNumber(-2, 1.5));
    m_pParameters.setObjectParameterValue(PARAM_OBJECT_LOWER_RIGHT_WINDOW_CORNER, new ComplexNumber(1, 1.5 - 3*dAspectRatio));
    initEffectRepaint();
  }
  
  ///////////////////////////////////////
  // GET METHODS                       //
  ///////////////////////////////////////
  /**
   *  Returns the {@code MandelbrotSetParameters} object representing the effect parameters of this panel.
   *
   *  @return The {@code MandelbrotSetParameters} object representing the effect parameters of this panel.
   **/
  public MandelbrotSetParameters getMandelbrotSetParameters() {
    return m_pParameters;
  }
  
  /**
   *  Returns the {@code MandelbrotSetParameters} object representing the effect parameters of this panel.
   *
   *  @return The {@code MandelbrotSetParameters} object representing the effect parameters of this panel.
   **/
  public EffectParameters getEffectParameters() {
    return getMandelbrotSetParameters();
  }
  
  /**
   *  Returns the {@code MandelbrotSetPropertiesPanel} object representing the effect parameters and providing a GUI to modify them.
   *
   *  @return The {@code MandelbrotSetPropertiesPanel} object representing the effect parameters and providing a GUI to modify them.
   **/
  public AbstractPropertiesPanel getPropertiesPanel() {
    if (m_pMandelbrotSetPropertiesPanel == null) {
      m_pMandelbrotSetPropertiesPanel = new MandelbrotSetPropertiesPanel(this);
      m_pMandelbrotSetPropertiesPanel.addPropertiesPanelListener(this);
    }
    
    return m_pMandelbrotSetPropertiesPanel;
  }
  
  
 
  //////////////////////////////////////
  // DRAWING HELPER METHODS           //
  //////////////////////////////////////
  protected void drawInteraction(Graphics2D pGraphics) {
    if ((m_pStartPoint != null) && (m_pEndPoint != null)) {
      pGraphics.setColor(Color.WHITE);
      int iXSize = m_pEndPoint.x - m_pStartPoint.x;
      double dAspectRatio = (double)m_iHeight/(double)m_iWidth;
      int iYSize = (int)(iXSize*dAspectRatio);
      if (iXSize > 0) {
        pGraphics.drawRect(m_pStartPoint.x, m_pStartPoint.y, iXSize, iYSize);
      }
      else {
        pGraphics.drawRect(m_pStartPoint.x+iXSize, m_pStartPoint.y+iYSize, -iXSize, -iYSize);
      }
      
    }
  }

  
  
  /////////////////////////////////
  // PRIVATE HELPER METHODS      //
  /////////////////////////////////
  private void initEffectRepaint() {
    if (m_pMandelbrotSet == null) return;
    
    m_pMandelbrotSet.setMandelbrotSetParameters(m_pParameters);
    m_pMandelbrotSet.processAsThread(this);
    
  }
  
  
  

  /////////////////////////////////
  // MOUSE LISTENER INTERFACE    //
  /////////////////////////////////
  /**
   *  If the mouse is double clicked the default Mandelbrot set is shown again.
   *
   *  @param e The mouse event that triggers this method.
   **/
  public void mouseClicked(MouseEvent e) {
    //doubleclick reset to default
    if (e.getClickCount() == 2) {
      setDefaultMandelbrotSet();
    }
  }
  
  /**
   *  Not used at the moment.
   *
   *  @param e The mouse event that triggers this method.
   **/
  public void mouseEntered(MouseEvent e) {}
  
  /**
   *  Not used at the moment.
   *
   *  @param e The mouse event that triggers this method.
   **/
  public void mouseExited(MouseEvent e) {}
  
  /**
   *  Sets a start point for a zoom operation.
   *
   *  @param e The mouse event that triggers this method.
   **/
  public void mousePressed(MouseEvent e) {
    m_pStartPoint = new Point(e.getX(), e.getY());
  }
  
  /**
   *  Realizes the zooming.
   *
   *  @param e The mouse event that triggers this method.
   **/
  public void mouseReleased(MouseEvent e) {
    //set the new coordinates of the mandelbrot set
    if ((m_pStartPoint != null) && (m_pEndPoint != null)) {
      
      
      int iXSize = m_pEndPoint.x - m_pStartPoint.x;
      double dAspectRatio = (double)m_iHeight/(double)m_iWidth;
      int iYSize = (int)(iXSize*dAspectRatio);
      
      int iXPos1, iXPos2, iYPos1, iYPos2;
      if (iXSize > 0) {
        iXPos1 = m_pStartPoint.x;
        iYPos1 = m_pStartPoint.y;
        
        iXPos2 = iXPos1 + iXSize;
        iYPos2 = iYPos1 + iYSize;
        
      }
      else {
        iXPos1 = m_pStartPoint.x + iXSize;
        iYPos1 = m_pStartPoint.y + iYSize;
        
        iXPos2 = m_pStartPoint.x;
        iYPos2 = m_pStartPoint.y;
        
      }
      
      //correct the iXPos and the iYPos by the image offset
      int iXOffset = getImageXOffset();
      iXPos1 -= iXOffset;
      iXPos2 -= iXOffset;
      
      int iYOffset = getImageYOffset();
      iYPos1 -= iYOffset;
      iYPos2 -= iYOffset;
      
      ComplexNumber m_pUpperLeft = m_pMandelbrotSet.getComplexNumberAt(iXPos1, iYPos1);
      ComplexNumber m_pLowerRight = m_pMandelbrotSet.getComplexNumberAt(iXPos2, iYPos2);
      
      m_pParameters.setObjectParameterValue(PARAM_OBJECT_UPPER_LEFT_WINDOW_CORNER, m_pUpperLeft);
      m_pParameters.setObjectParameterValue(PARAM_OBJECT_LOWER_RIGHT_WINDOW_CORNER, m_pLowerRight);
      initEffectRepaint(); //the image has to be repainted
    }
    
    
    
    
    //reset the interaction variables
    m_pStartPoint = null;
    m_pEndPoint = null;
    
    //repaint
    repaint();
  }
  
  /////////////////////////////////////
  // MOUSE MOTION LISTENER INTERFACE //
  /////////////////////////////////////
  /**
   *  Sets an end point for the zooming operation.
   *
   *  @param e The mouse event that triggers this method.
   **/
  public void mouseDragged(MouseEvent e) {
    if (m_pStartPoint != null) {
      m_pEndPoint = new Point(e.getX(), e.getY());
      repaint();
    }
  }
  
  /**
   *  Not used at the moment.
   *
   *  @param e The mouse event that triggers this method.
   **/
  public void mouseMoved(MouseEvent e) {}
  
  
  ////////////////////////////////////////
  // SIZE MODE PANEL LISTENER           //
  ////////////////////////////////////////
  /*
  public void applySizeModeChanges(int iSizeMode, int iFixedWidth, int iFixedHeight) {
    m_iSizeMode = iSizeMode;
    m_iFixedWidth = iFixedWidth; //used when fixed size
    m_iFixedHeight = iFixedHeight; //used when fixed size
    
    setEffectSize(iFixedWidth, iFixedHeight);
    repaint();
    
  }
  */
  
  /////////////////////////////////
  // SIZING PANEL LISTENER       //
  /////////////////////////////////
  /**
   *  Applies the settings of the specified {@code SizingPanel} to the effect.
   *
   *  @param pSizingPanel The {@code SizingPanel} from which the size properties are taken. 
   **/
  public void applySizingChanges(SizingPanel pSizingPanel) {
    
    //2. set the new sizes
    m_bUsePanelSize = false;
    m_iFixedWidth = pSizingPanel.getEffectWidth(); //used when fixed size
    m_iFixedHeight = pSizingPanel.getEffectHeight(); //used when fixed size
    
    setEffectSize(m_iFixedWidth, m_iFixedHeight);
    repaint();
    
    
    
    
  }
  

}