package gui.fractals;

/*
 *  Copyright (C) 2014 Juergen Platzer 
 * 
 *  This file is part of JFXLibrary.
 *  
 *  JFXLibrary is free software: you can redistribute it and/or modify
 *  it under the terms of the GNU Lesser General Public License as published by
 *  the Free Software Foundation, either version 3 of the License, or
 *  (at your option) any later version.
 *  
 *  JFXLibrary is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU Lesser General Public License for more details.
 *  
 *  You should have received a copy of the GNU Lesser General Public License
 *  along with JFXLibrary.  If not, see <http://www.gnu.org/licenses/>.
 */

//gui imports
import javax.swing.JPanel;
import javax.swing.JComboBox;
import javax.swing.JSlider;
import javax.swing.JButton;
import javax.swing.JColorChooser;
import javax.swing.BoxLayout;
import javax.swing.Box;
import javax.swing.border.EmptyBorder;

import javax.swing.event.ChangeListener;
import javax.swing.event.ChangeEvent;


import java.awt.Color;
import java.awt.BorderLayout;
import java.awt.FlowLayout;
import java.awt.Dimension;

import java.awt.event.ActionListener;
import java.awt.event.ActionEvent;
import java.awt.event.ComponentListener;
import java.awt.event.ComponentEvent;

import java.util.Vector;

//Library imports
import standard.EffectParameters;
import fractals.MandelbrotSetParameters;
import gui.AbstractPropertiesPanel;
import gui.SlimSliderPanel;
import gui.ISlimSliderPanelListener;
import gui.SizingPanel;
import gui.ISizingPanelListener;

import gui.color.ColorGradientPanel;
import gui.color.IColorGradientPanelListener;
import util.color.ColorGradient;


import applications.Settings;
import applications.SettingsDialog;
import applications.LanguageProperties;

//static imports
import static fractals.MandelbrotSetParameters.*;


/**
 *  A properties panel for the mandelbrot effect.
 **/
public class MandelbrotSetPropertiesPanel 
  extends AbstractPropertiesPanel 
  implements 
    ISlimSliderPanelListener, IColorGradientPanelListener, 
    ActionListener, ISizingPanelListener, ComponentListener {
  
  /*** member variables ***/
  private MandelbrotSetPanel m_pMandelbrotSetPanel; 
  private MandelbrotSetParameters m_pMandelbrotSetParameters; 
  //those parameters are shared with the effect panel, which steers the window and the size

  /*** gui member variables ***/
  private SlimSliderPanel m_pIterationDepthSliderPanel;
  private ColorGradientPanel m_pColorGradientPanel;
  private JButton m_pResetMandelbrotSet;
  private SizingPanel m_pSizingPanel;
  
  private JPanel m_pCurrentPropertyPanel;
  
  ////////////////////////////////
  // CONSTRUCTOR                //
  ////////////////////////////////
  /**
   *  Creates a {@code MandelbrotSetPropertiesPanel} object with the parameters that are set to a given  {@code MandelbrotSetPanel}.
   *
   *  @param pMandelbrotSetPanel The {@code MandelbrotSetPanel} object from which the effect properties are taken.
   **/
  public MandelbrotSetPropertiesPanel(MandelbrotSetPanel pMandelbrotSetPanel) {
    //1. init members
    m_pMandelbrotSetPanel = pMandelbrotSetPanel;
    m_pMandelbrotSetParameters = pMandelbrotSetPanel.getMandelbrotSetParameters();
  
    //2. set the layout
    setLayout(new FlowLayout(FlowLayout.LEFT, 10, 0));
    //setLayout(new BorderLayout());
    
    
    
 
    //4. create the panel for the iteration depth
    m_pIterationDepthSliderPanel = new SlimSliderPanel(10, 1000, 100);
    m_pIterationDepthSliderPanel.addSlimSliderPanelListener(this);
    m_pCurrentPropertyPanel = m_pIterationDepthSliderPanel;
    add(m_pIterationDepthSliderPanel);
    //add(m_pIterationDepthSliderPanel, BorderLayout.CENTER);
        
    
    //5. color gradient    
    m_pColorGradientPanel = new ColorGradientPanel();
    m_pColorGradientPanel.addColorGradientPanelListener(this);
    m_pColorGradientPanel.setMinimumSize(new Dimension(400, 50));
    m_pColorGradientPanel.setPreferredSize(new Dimension(400, 50));
    //m_pColorGradientPanel.setMinimumSize(new Dimension(400, 50));
    

    
    
    
    //6. reset the mandelbrot set
    m_pResetMandelbrotSet = new JButton("Reset Mandelbrot set");
    m_pResetMandelbrotSet.addActionListener(this);
        
    
    
    //7. sizing
    m_pSizingPanel = new SizingPanel();
    //m_pSizingPanel.addSizingPanelListener(this);    // not necessary. handled in the mandelbrot set panel
    m_pSizingPanel.addSizingPanelListener(m_pMandelbrotSetPanel);    
    
    
    
    
    //finally make new settings and register the component listener
    //applyNewSettings(SettingsDialog.getSettings());
    addComponentListener(this);
    
    
    //8. create the properties panel and populate it
    Settings pSettings = SettingsDialog.getSettings();
    LanguageProperties pLanguageProperties = new LanguageProperties(pSettings.getProperty("Language"));
    
    
    JComboBox pPropertiesSelection = initPropertiesComboBox(this);
    add(pPropertiesSelection);
    populatePropertiesSelection(pLanguageProperties);
    
    
  }
  
  
  ///////////////////////////
  // GET METHODS           //
  ///////////////////////////
  /**
   *  Returns the {@code MandelbrotSetParameters} object representing the effect parameters of this panel.
   *
   *  @return The {@code MandelbrotSetParameters} object representing the effect parameters of this panel.
   **/
  public EffectParameters getEffectParameters() {
    return m_pMandelbrotSetParameters;
  }
  
  ///////////////////////////
  // SET METHODS           //
  ///////////////////////////
  /**
   *  Sets the effect parameters that should be represented by this {@code MandelbrotSetPropertiesPanel} object. 
   *  The set operation is only successful if a {@code MandelbrotSetParameters} object has been specified.
   *
   *  @param pParameters The {@code MandelbrotSetParameters} object representing the effect parameters 
   *                     that should be represented by this {@code MandelbrotSetPropertiesPanel} object.
   **/
  public void setEffectParameters(EffectParameters pParameters) {
    //1. make some checks
    if (pParameters == null) return;
    
    if (pParameters instanceof MandelbrotSetParameters) {
      m_pMandelbrotSetParameters = (MandelbrotSetParameters)pParameters;
      
      updateControls();
    }
  }
  
  
  
  ////////////////////////////////////////
  // SLIM SLIDER PANEL LISTENER         //
  ////////////////////////////////////////
  /**
   *  If the iteration depth has been adapted 
   *  in the corresponding slider panel, it is 
   *  communicated with this method, the effect parameters are adapted and the listeners are notified.
   *
   *  @param pSliderPanel The {@code SlimSliderPanel} object that has been modified.
   *  @param iValue The value of the slider panel that has been modified. 
   **/
  public void applyChanges(SlimSliderPanel pSliderPanel, int iValue) {
  
    //1. make some simple checks
    if (pSliderPanel == null) return;
    if (m_pMandelbrotSetParameters == null) m_pMandelbrotSetParameters = new MandelbrotSetParameters();
    
    //2. set the new parameters
    if (pSliderPanel.equals(m_pIterationDepthSliderPanel)) {
      m_pMandelbrotSetParameters.setIntegerParameterValue(PARAM_INT_ITERATION_DEPTH, iValue);
    }
    
    
    //3. fire the apply changes event
    fireApplyChanges(m_pMandelbrotSetParameters);
  }
  
  ////////////////////////////////////////
  // COLOR GRADIENT PANEL LISTENER      //
  ////////////////////////////////////////
  /**
   *  If the color gradient has been modified in the specified {@code ColorGradientPanel}, it is 
   *  communicated with this method, the effect parameters are adapted and the listeners are notified.
   *
   *  @param pColorGradientPanel The {@code ColorGradientPanel} object that has been modified.
   **/
  public void colorGradientChanged(ColorGradientPanel pColorGradientPanel) {
    //1. make some checks
    if (pColorGradientPanel == null) return;
    if (m_pMandelbrotSetParameters == null) return;

    //2. set the color gradient
    ColorGradient pColorGradient = pColorGradientPanel.getColorGradient();
    if (pColorGradient == null) return;
    
    m_pMandelbrotSetParameters.setObjectParameterValue(PARAM_OBJECT_COLOR_GRADIENT, pColorGradient);
    
    //3. fire the apply changes event
    fireApplyChanges(m_pMandelbrotSetParameters);
  }
  
  //////////////////////////////////////////
  // ACTION LISTENER - BUTTON EVENTS      //
  //////////////////////////////////////////
  /**
   *   Applies a reset of the Mandelbrot set effect to the default setting respectively detects 
   *   which parameter should be shown now in the properties panel.
   *
   *   @param e The action event triggering this method. 
   **/
  public void actionPerformed(ActionEvent e) {
    
    //1. get source
    Object pSource = e.getSource();
    if (pSource == null) return;
    
    //2. reset the mandelbrot set
    if ((m_pResetMandelbrotSet != null) && (pSource.equals(m_pResetMandelbrotSet))) {
      m_pMandelbrotSetPanel.setDefaultMandelbrotSet();
      fireApplyChanges(m_pMandelbrotSetParameters);
      return;
    }
    
    
    //3. properties selection
    //3.2. remove the current property panel
    if (m_pCurrentPropertyPanel != null) remove(m_pCurrentPropertyPanel);
    int iSelectedId = getSelectedPropertiesIndex();
    
    //3. set the new property panel
    if (iSelectedId == 0) {
      m_pCurrentPropertyPanel = m_pIterationDepthSliderPanel;
    }
    else if (iSelectedId == 1) {
      m_pColorGradientPanel.setPreferredSize(new Dimension(getColorGradientPanelWidth(), 50));
      m_pCurrentPropertyPanel = m_pColorGradientPanel;
    }
    else if (iSelectedId == 2) {
      m_pCurrentPropertyPanel = m_pSizingPanel;
    }
    
    
    
    add(m_pCurrentPropertyPanel);
    //add(m_pCurrentPropertyPanel, BorderLayout.CENTER);
    
    repaint();
    revalidate();
      
    
    
  }
  
  /////////////////////////////////
  // SIZING PANEL LISTENER       //
  /////////////////////////////////
  /**
   *  Applies the settings of the specified {@code SizingPanel} to the effect.
   *
   *  @param pSizingPanel The {@code SizingPanel} from which the size properties are taken. 
   **/
  public void applySizingChanges(SizingPanel pSizingPanel) {
    //1. make a simple check
    if (m_pMandelbrotSetParameters == null) m_pMandelbrotSetParameters = new MandelbrotSetParameters();
    
    //2. set the new sizes
    m_pMandelbrotSetParameters.setIntegerParameterValue(PARAM_INT_WIDTH, pSizingPanel.getEffectWidth());
    m_pMandelbrotSetParameters.setIntegerParameterValue(PARAM_INT_HEIGHT, pSizingPanel.getEffectHeight());
    
    //3. apply the changes
    fireApplyChanges(m_pMandelbrotSetParameters);
  }
  
  
  ////////////////////////////////
  // SETTINGS LISTENER METHODS  //
  ////////////////////////////////
  /**
   *  Applies the changes of the settings to the application. For example when the language is changed, 
   *  all GUI elements are updated accordingly.
   *
   *  @param pSettings The new {@code Settings} object that is used to retrieve all labels. 
   **/
  public void applyNewSettings(Settings pSettings) {
  
    //1. make an initial check and set the settings
    if (pSettings == null) return;
    
    //2. load new language settings
    LanguageProperties pLanguageProperties = new LanguageProperties(pSettings.getProperty("Language"));
    populatePropertiesSelection(pLanguageProperties);
    
    
    
    //4. sizing panel
    if (m_pSizingPanel != null) {
      m_pSizingPanel.applyNewSettings(pSettings);
    }
    
    
   
    
  }
  
  //////////////////////////////////
  // COMPONENT LISTENER           //
  //////////////////////////////////
  /** 
   *  Not used at the moment.
   *
   *  @param e The event triggering this method.
   **/
  public void  componentHidden(ComponentEvent e) {}

  /** 
   *  Not used at the moment.
   *
   *  @param e The event triggering this method.
   **/
  public void componentMoved(ComponentEvent e) {}

  /** 
   *  Resizes the color gradient panel when the size of this panel has been changed.
   *
   *  @param e The event triggering this method.
   **/
  public void componentResized(ComponentEvent e) {
    if (m_pCurrentPropertyPanel.equals(m_pColorGradientPanel)) {
      m_pColorGradientPanel.setPreferredSize(new Dimension(getColorGradientPanelWidth(), 50));
      m_pColorGradientPanel.repaint();
      m_pColorGradientPanel.revalidate();
      
    }
  }

  /** 
   *  Not used at the moment.
   *
   *  @param e The event triggering this method.
   **/
  public void componentShown(ComponentEvent e) {}

  
  
  
  //////////////////////////////////
  // PRIVATE HELPER METHODS       //
  //////////////////////////////////
  private void updateControls() {
    //1. make some checks
    if (m_pMandelbrotSetParameters == null) return;
    
    //2. TO DO: update all controls according to the settings 
    //   in the effect parameters and make sure that no action is fired!
  }
  
  private int getColorGradientPanelWidth() { 
    return getWidth() - 50 - getPropertiesSelectionWidth();
  }
  
  private void populatePropertiesSelection(LanguageProperties pLanguageProperties) {
    
    
    Vector<String> pPropertyKeys = new Vector<String>();
    pPropertyKeys.add("MandelbrotSet.IterationDepth");
    pPropertyKeys.add("MandelbrotSet.ColorGradient");
    pPropertyKeys.add("MandelbrotSet.Size");
    populatePropertiesSelection(pLanguageProperties, pPropertyKeys);
    

  }
  
  
}