package gui.io;

/*
 *  Copyright (C) 2014 Juergen Platzer 
 * 
 *  This file is part of JFXLibrary.
 *  
 *  JFXLibrary is free software: you can redistribute it and/or modify
 *  it under the terms of the GNU Lesser General Public License as published by
 *  the Free Software Foundation, either version 3 of the License, or
 *  (at your option) any later version.
 *  
 *  JFXLibrary is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU Lesser General Public License for more details.
 *  
 *  You should have received a copy of the GNU Lesser General Public License
 *  along with JFXLibrary.  If not, see <http://www.gnu.org/licenses/>.
 */

import javax.swing.JPanel;
import javax.swing.JFileChooser;
import javax.swing.filechooser.FileNameExtensionFilter;
import javax.swing.JOptionPane;

import javax.imageio.ImageIO;
import java.io.File;

import java.awt.Color;
import java.awt.Dimension;
import java.awt.Graphics;
import java.awt.Graphics2D;


import java.awt.image.BufferedImage;

import java.awt.event.MouseListener;
import java.awt.event.MouseEvent;

import java.util.Vector;


/**
 *  A gui element that handles an image parameter for effects. 
 **/
public class ImageDisplay extends JPanel implements MouseListener {
  
  /** private gui member variables **/
  private BufferedImage m_pImage;
  
  /*** listener member variables **/
  private Vector<IImageDisplayListener> m_pListeners;
  
  //////////////////////////
  // CONSTRUCTOR          //
  //////////////////////////
  /**
   *  Creates an {@code ImageDisplay} object holding no image and showing the default tooltip message.
   **/
  public ImageDisplay() {
    //1. set members
    m_pImage = null;
    m_pListeners = new Vector<IImageDisplayListener>();
    
    //2. set size and tooltip
    setPreferredSize(new Dimension(50, 30));
    setToolTipText("Click to change the image!");
    
    //3. set listeners
    addMouseListener(this);
  }
  
  
  
  ////////////////////////////
  // GET METHODS            //
  ////////////////////////////
  /**
   *  Returns the image that is represented by this object. If no image is set, null is returned.
   *
   *  @return The image that is represented by this object as {@code BufferedImage}.
   **/
  public BufferedImage getImage() {
    return m_pImage;
  }
  
  ////////////////////////////
  // SET METHODS            //
  ////////////////////////////
  /**
   *  Sets the image that should be represented by this gui element.
   *
   *  @param pImage The {@code BufferedImage} object that should be represented by this gui element.
   **/
  public void setImage(BufferedImage pImage) {
    m_pImage = pImage;
    repaint();
  }
  
 
  
  //////////////////////////
  // PAINTING             //
  //////////////////////////
  /**
   *  Painting method that draws the represented image on it, or a black rectangle, if no image is set.
   *
   *  @param g The graphics context that is used for the drawing commands.
   **/
  public void paintComponent(Graphics g) {
    //1. fetch the graphics context and fetch the size
    Graphics2D pGraphics = (Graphics2D)g;
    int iWidth = getWidth();
    int iHeight = getHeight();
    
    //2. handle if no image is set
    if (m_pImage == null) {
      pGraphics.setColor(Color.BLACK);
      pGraphics.fillRect(0, 0, iWidth, iHeight);
      return;
    }
    
    //3. draw the image in the correct size
    /* // rendering hints if necessary
    pGraphics.setRenderingHint(RenderingHints.KEY_INTERPOLATION,
    RenderingHints.VALUE_INTERPOLATION_BILINEAR);
    pGraphics.setRenderingHint(RenderingHints.KEY_RENDERING,
    RenderingHints.VALUE_RENDER_QUALITY);
    pGraphics.setRenderingHint(RenderingHints.KEY_ANTIALIASING,
    RenderingHints.VALUE_ANTIALIAS_ON);
    */
    pGraphics.drawImage(m_pImage, 0, 0, iWidth, iHeight, null);
    
  }
  
  
  ///////////////////////////
  // MOUSE LISTENER        //
  ///////////////////////////
  /**
   * Used to inform the listeners about a change in the image property.
   *
   * @param e The mouse event that triggers this method.
   **/
  public void mouseClicked(MouseEvent e) {
    fireListenerEvent();
  }
  
  /**
   * Not used at the moment.
   *
   * @param e The mouse event that triggers this method.
   **/
  public void mouseEntered(MouseEvent e) {}
  
  /**
   * Not used at the moment.
   *
   * @param e The mouse event that triggers this method.
   **/
  public void mouseExited(MouseEvent e) {}
  
  /**
   * Not used at the moment.
   *
   * @param e The mouse event that triggers this method.
   **/
  public void mousePressed(MouseEvent e) {}
  
  /**
   * Not used at the moment.
   *
   * @param e The mouse event that triggers this method.
   **/
  public void mouseReleased(MouseEvent e) {}
  
  
  ////////////////////////////////
  // LISTENER METHODS           //
  ////////////////////////////////
  /**
   *  Adds a listener for this {@code ImageDisplay} object. 
   *
   *  @param pListener The listener for this {@code ImageDisplay} object that should be added. 
   **/
  public void addImageDisplayListener(IImageDisplayListener pListener) {
    if (pListener == null) return;
    m_pListeners.add(pListener);
  }
  
  /**
   *  Removes the specified listener from this {@code ImageDisplay} object. 
   *
   *  @param pListener The listener that should be removed. 
   **/
  public void removeImageDisplayListener(IImageDisplayListener pListener) {
    if (pListener == null) return;
    m_pListeners.remove(pListener);
  }
  
  /**
   *  Removes all listeners from this {@code ImageDisplay} object. 
   *
   **/
  public void removeAllIImageDisplayListeners() {
    m_pListeners.clear();
  }
  
  
  /////////////////////////////////
  // PRIVATE HELPER METHODS      //
  /////////////////////////////////
  private void fireListenerEvent() {
    int iListenerNum = m_pListeners.size();
    for (int i = 0; i < iListenerNum; i++) {
      m_pListeners.get(i).imageLoadRequested(this);
    }
  }
  
  
  
  
  
  
}