package io;

/*
 *  Copyright (C) 2014 Juergen Platzer 
 * 
 *  This file is part of JFXLibrary.
 *  
 *  JFXLibrary is free software: you can redistribute it and/or modify
 *  it under the terms of the GNU Lesser General Public License as published by
 *  the Free Software Foundation, either version 3 of the License, or
 *  (at your option) any later version.
 *  
 *  JFXLibrary is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU Lesser General Public License for more details.
 *  
 *  You should have received a copy of the GNU Lesser General Public License
 *  along with JFXLibrary.  If not, see <http://www.gnu.org/licenses/>.
 */

//library imports
import standard.IVideo;

import motionanalysis.FrameDifferencing;
import motionanalysis.FrameDifferencingParameters;

import static motionanalysis.FrameDifferencingParameters.*;

/**
 *  Provides methods to read in configuration files for the frame differencing effect.
 **/

public class FrameDifferencingConfiguration {

  /*** the prefix for the frame differencing configuration parameters ***/
  public static final String FRAME_DIFF_PREFIX = "framedifferencing";
  /*** the prefix for the frame differencing history configuration parameters ***/
  public static final String HISTORY_PREFIX = "history";
  /*** the prefix for the frame differencing show configuration parameters ***/
  public static final String SHOW_PREFIX = "show";
  
  /*** private static constants defining the framedifferencing parameters ***/
  private static final String THRESHOLD = "threshold";
  private static final String MARKING_COLOR = "marking_color";
  private static final String CONVERT_TO_GRAY = "convert_to_gray";
  
  private static final String HISTORY_ON = "on";
  private static final String HISTORY_KEEP = "keep";
  private static final String HISTORY_FRAME_NUM = "frame_num";
  private static final String HISTORY_FADING_FRAME_NUM = "fading_frame_num";
  
  private static final String SHOW_ORIGINAL = "original";
  private static final String SHOW_FRAME_DIFF = "frame_diff";
  private static final String SHOW_THRESHOLD_MASK = "threshold_mask";
  private static final String SHOW_HISTORY = "history";
  private static final String SHOW_RESULT = "result";

  
  
  /**
   *  Creates a frame differencing effect from a configuration file. Exception messages by configuring the frame differencing effect 
   *  are written to the command line.
   *
   *  @param pConfiguration The configuration object that handles the direct access to the configuration file. 
   *
   *  @throws NullPointerException when no configuration object has been specified.
   *  @throws Exception when the video object could not be configured.
   *
   *  @return The frame differencing effect that is created by the settings of the specified configuration file.
   **/
  public static FrameDifferencing getEffect(Configuration pConfiguration) throws Exception {
    //1. make a simple check
    if (pConfiguration == null) throw new NullPointerException("No configuration was specified!");
    
    //2. fetch the video
    IVideo pVideo = VideoConfiguration.getVideo(pConfiguration);
    
       
    //3. fetch and set the configuration values in the frame differencing parameters
    FrameDifferencingParameters pFrameDiffParams = new FrameDifferencingParameters();
    
    //3.1. general frame differencing parameters
    try {
      String pParamName = FRAME_DIFF_PREFIX + "." + THRESHOLD;
      boolean bSet = pFrameDiffParams.setParameterValue(PARAM_INT_THRESHOLD, 
      pConfiguration.getParameterValue(pParamName));
      if (!bSet) throw new Exception("The value of the parameter '" + pParamName + " could not be set!");
    }
    catch (Exception pException) {
      System.out.println(pException.getMessage());
    }
  
  
  
    try {
      String pParamName = FRAME_DIFF_PREFIX + "." + MARKING_COLOR;
      boolean bSet = pFrameDiffParams.setParameterValue(PARAM_COLOR_HISTORY_COLOR, 
      pConfiguration.getParameterValue(pParamName));
      if (!bSet) throw new Exception("The value of the parameter '" + pParamName + " could not be set!");
    }
    catch (Exception pException) {
      System.out.println(pException.getMessage());
    }
    
    
    try {
      String pParamName = FRAME_DIFF_PREFIX + "." + CONVERT_TO_GRAY;
      boolean bSet = pFrameDiffParams.setParameterValue(PARAM_BOOL_CONVERT_DIFF_IM, 
      pConfiguration.getParameterValue(pParamName));
      if (!bSet) throw new Exception("The value of the parameter '" + pParamName + " could not be set!");
    }
    catch (Exception pException) {
      System.out.println(pException.getMessage());
    }
    
    //3.2. history parameters
    try {
      String pParamName = FRAME_DIFF_PREFIX + "." + HISTORY_PREFIX + "." + HISTORY_ON;
      boolean bSet = pFrameDiffParams.setParameterValue(PARAM_BOOL_HISTORY_ON, 
      pConfiguration.getParameterValue(pParamName));
      if (!bSet) throw new Exception("The value of the parameter '" + pParamName + " could not be set!");
    }
    catch (Exception pException) {
      System.out.println(pException.getMessage());
    }

    try {
      String pParamName = FRAME_DIFF_PREFIX + "." + HISTORY_PREFIX + "." + HISTORY_KEEP;
      boolean bSet = pFrameDiffParams.setParameterValue(PARAM_BOOL_HISTORY_KEEP, 
      pConfiguration.getParameterValue(pParamName));
      if (!bSet) throw new Exception("The value of the parameter '" + pParamName + " could not be set!");
    }
    catch (Exception pException) {
      System.out.println(pException.getMessage());
    }
    
    
    try {
      String pParamName = FRAME_DIFF_PREFIX + "." + HISTORY_PREFIX + "." + HISTORY_FRAME_NUM;
      boolean bSet = pFrameDiffParams.setParameterValue(PARAM_INT_HISTORY_FRAME_NUM, 
      pConfiguration.getParameterValue(pParamName));
      if (!bSet) throw new Exception("The value of the parameter '" + pParamName + " could not be set!");
    }
    catch (Exception pException) {
      System.out.println(pException.getMessage());
    }
    
     
    try {
      String pParamName = FRAME_DIFF_PREFIX + "." + HISTORY_PREFIX + "." + HISTORY_FADING_FRAME_NUM;
      boolean bSet = pFrameDiffParams.setParameterValue(PARAM_INT_HISTORY_FADE_FRAME_NUM, 
      pConfiguration.getParameterValue(pParamName));
      if (!bSet) throw new Exception("The value of the parameter '" + pParamName + " could not be set!");
    }
    catch (Exception pException) {
      System.out.println(pException.getMessage());
    }
    
    
    //3.3 show parameters
    try {
      String pParamName = FRAME_DIFF_PREFIX + "." + SHOW_PREFIX + "." + SHOW_ORIGINAL;
      boolean bSet = pFrameDiffParams.setParameterValue(PARAM_BOOL_SHOW_ORIG_FRAME, 
      pConfiguration.getParameterValue(pParamName));
      if (!bSet) throw new Exception("The value of the parameter '" + pParamName + " could not be set!");
    }
    catch (Exception pException) {
      System.out.println(pException.getMessage());
    }
  
  
    try {
      String pParamName = FRAME_DIFF_PREFIX + "." + SHOW_PREFIX + "." + SHOW_FRAME_DIFF;
      boolean bSet = pFrameDiffParams.setParameterValue(PARAM_BOOL_SHOW_DIFF_IM, 
      pConfiguration.getParameterValue(pParamName));
      if (!bSet) throw new Exception("The value of the parameter '" + pParamName + " could not be set!");
    }
    catch (Exception pException) {
      System.out.println(pException.getMessage());
    }
  
  
    try {
      String pParamName = FRAME_DIFF_PREFIX + "." + SHOW_PREFIX + "." + SHOW_THRESHOLD_MASK;
      boolean bSet = pFrameDiffParams.setParameterValue(PARAM_BOOL_SHOW_MOTION_MASK, 
      pConfiguration.getParameterValue(pParamName));
      if (!bSet) throw new Exception("The value of the parameter '" + pParamName + " could not be set!");
    }
    catch (Exception pException) {
      System.out.println(pException.getMessage());
    }
  

    try {
      String pParamName = FRAME_DIFF_PREFIX + "." + SHOW_PREFIX + "." + SHOW_HISTORY;
      boolean bSet = pFrameDiffParams.setParameterValue(PARAM_BOOL_SHOW_HISTORY, 
      pConfiguration.getParameterValue(pParamName));
      if (!bSet) throw new Exception("The value of the parameter '" + pParamName + " could not be set!");
    }
    catch (Exception pException) {
      System.out.println(pException.getMessage());
    }

    
    try {
      String pParamName = FRAME_DIFF_PREFIX + "." + SHOW_PREFIX + "." + SHOW_RESULT;
      boolean bSet = pFrameDiffParams.setParameterValue(PARAM_BOOL_SHOW_RESULT, 
      pConfiguration.getParameterValue(pParamName));
      if (!bSet) throw new Exception("The value of the parameter '" + pParamName + " could not be set!");
    }
    catch (Exception pException) {
      System.out.println(pException.getMessage());
    }


    
    //4. create the Framedifferencing effect
    try {
      FrameDifferencing pFrameDiff = new FrameDifferencing(pVideo, pFrameDiffParams);
      return pFrameDiff;
    }
    catch (NullPointerException pException) {
      return null;
    }
  }
  
}