package io;

/*
 *  Copyright (C) 2014 Juergen Platzer 
 * 
 *  This file is part of JFXLibrary.
 *  
 *  JFXLibrary is free software: you can redistribute it and/or modify
 *  it under the terms of the GNU Lesser General Public License as published by
 *  the Free Software Foundation, either version 3 of the License, or
 *  (at your option) any later version.
 *  
 *  JFXLibrary is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU Lesser General Public License for more details.
 *  
 *  You should have received a copy of the GNU Lesser General Public License
 *  along with JFXLibrary.  If not, see <http://www.gnu.org/licenses/>.
 */

//standard imports
import java.awt.image.BufferedImage;
import java.text.DecimalFormat;

//library imports
import standard.IVideoEffectListener;

/**
 *  Manages command line output for processing of a video.
 **/
public class ProgressOutput implements IVideoEffectListener {

  /*** timing member variables ***/
  private long m_lRenderStartTime;
  private long m_lCurrentFrameStartTime;
  private int  m_iTimingFrameId;
  
  
  
  /////////////////////////////////////
  // VIDEO EFFECT LISTENER INTERFACE //
  /////////////////////////////////////
  /**
   *  Inits the timing variables when the video processing starts. 
   **/
  public void processingStarted() {
    initTimingVariables();
  }
  
  /**
   *  Not used. 
   *
   *  @param dProgress The percentage of the progress as double value.
   *  @param pCurrentWork The description of the current task as string.
   **/
  public void setProgressStatus(double dProgress, String pCurrentWork) {}
  
  /**
   *  Prints a finish notification. 
   **/
  public void endedProperly() {
    System.out.println("Rendering finished successfully!");
    System.out.println("");
    System.out.println("");
  
  }


  /**
   *  Prints a notification that the processing has been interrupted. The reason for the interruption is mentioned.
   *
   *  @param pReason The reason for the interruption as string.
   **/
  public void interrupted(String pReason) {
    if (pReason == null) pReason = "";
    System.out.println("Rendering interrupted: " + pReason);
    System.out.println("");
    System.out.println("");
    
  
  }
  
  /**
   *  Prints a status notification with a progress percentag, rendering time and a remaining time.
   *
   *  @param pCurrentFrame The video frame that is currently processed as {@code BufferedImage} object.
   *  @param iFrameId The id of the current frame as integer.
   *  @param iFrameNumber The total number of frames as integer.
   **/
  public void setCurrentVideoFrame(BufferedImage pCurrentFrame, int iFrameId, int iFrameNumber) {
    //0. fetch general information
    long lNow = System.currentTimeMillis();
    int iProgress = iFrameId*100/iFrameNumber;

    //1. calculate the last frame's rendering time
    long lFrameRenderingTime = lNow - m_lCurrentFrameStartTime;
    double dFrameRenderingTime = (double)lFrameRenderingTime/1000;

    m_lCurrentFrameStartTime = lNow;


    //2. calculate the mean render time per frame
    long lOverallRenderTime = lNow - m_lRenderStartTime;
    String pOverallRenderTime = convertSecondsToTime(lOverallRenderTime/1000);
    double dMeanFrameRenderingTime = (double)lOverallRenderTime/(iFrameId*1000);
    

    //3. calculate the remaining rendertime
    int iRemainingFrameNumber = iFrameNumber - iFrameId;
    double dRemainingSeconds = iRemainingFrameNumber*dMeanFrameRenderingTime;
    String pRemainingTime = convertSecondsToTime(dRemainingSeconds);
    
    /*double dRemainingHours = dRemainingSeconds/3600;
    int iRemainingHours = (int)dRemainingHours;

    dRemainingSeconds -= iRemainingHours*3600;
    double dRemainingMinutes = dRemainingSeconds/60;
    int iRemainingMinutes = (int)dRemainingMinutes;
    String pRemainingMinutes = iRemainingMinutes+"";
    if (pRemainingMinutes.length() < 2) pRemainingMinutes = "0" + pRemainingMinutes;

    dRemainingSeconds -= iRemainingMinutes*60;
    int iRemainingSeconds = (int)dRemainingSeconds;
    String pRemainingSeconds = iRemainingSeconds+"";
    if (pRemainingSeconds.length() < 2) pRemainingSeconds = "0" + pRemainingSeconds;


    System.out.println("Remaining time: " + iRemainingHours + ":" + pRemainingMinutes + ":" + pRemainingSeconds);
    */
    
    //4. print everything
    String[] pLabels = new String[5];
    pLabels[0] = "Progress: ";
    pLabels[1] = "Last frame: ";
    pLabels[2] = "Mean frame rendering time: ";
    pLabels[3] = "Overall rendering time: ";
    pLabels[4] = "Remaining time: ";
    pLabels = fillWithSpaces(pLabels, true);
    
    DecimalFormat pDecimalFormat = new DecimalFormat("#0.000");
    String[] pValues = new String[5];
    pValues[0] = iFrameId + " of " + iFrameNumber + " frames rendered! (" + iProgress + " %)";
    pValues[1] = pDecimalFormat.format(dFrameRenderingTime) + " sec";
    pValues[2] = pDecimalFormat.format(dMeanFrameRenderingTime) + " sec";
    pValues[3] = pOverallRenderTime;
    pValues[4] = pRemainingTime;
    pLabels = fillWithSpaces(pLabels, false);
    
    /*int iProgress = (iFrameId+1)*100 / iFrameNumber;
    System.out.println("Progress: " + (iFrameId+1) + " of " + iFrameNumber + " frames rendered! (" + iProgress + " %)");
    
    DecimalFormat pDecimalFormat = new DecimalFormat("#0.000");
    System.out.println("Last frame:                " + pDecimalFormat.format(dFrameRenderingTime) + " sec");
    System.out.println("Mean frame rendering time: " + pDecimalFormat.format(dMeanFrameRenderingTime) + " sec");
    System.out.println("Overall rendering time:    " + "");
    System.out.println("Remaining time:            " + iRemainingHours + ":" + pRemainingMinutes + ":" + pRemainingSeconds);
    */
    
    for (int i = 0; i < 5; i++) {
      System.out.println(pLabels[i] + pValues[i]);
    }
    
    System.out.println("");
    System.out.println("");
    
  }

  
  ////////////////////////////////
  // PRIVATE HELPER METHODS     //
  ////////////////////////////////
  private void initTimingVariables() {
    m_lRenderStartTime = System.currentTimeMillis();
    m_lCurrentFrameStartTime = m_lRenderStartTime;
  }
  
  private String convertSecondsToTime(double dSeconds) {
  
    
  
    double dHours = dSeconds/3600;
    String pTime = (int)dHours + ":";

    dSeconds -= (int)dHours*3600;
    double dMinutes = dSeconds/60;
    String pMinutes = (int)dMinutes+"";
    if (pMinutes.length() < 2) pMinutes = "0" + pMinutes;
    pTime += pMinutes + ":";

    dSeconds -= (int)dMinutes*60;
    String pSeconds = (int)dSeconds+"";
    if (pSeconds.length() < 2) pSeconds = "0" + pSeconds;
    pTime += pSeconds;
    
    return pTime;
  }
  
  
  private String[] fillWithSpaces(String[] pData, boolean bRight) {
    //1. make a simple check
    if (pData == null) return null;
    int iEntryNum = pData.length;
    if (iEntryNum == 0) return pData;
    
    //2. calculate the maximum length
    int iMaxSize = pData[0].length();
    for (int i = 1; i < iEntryNum; i++) {
      int iCurrentSize = pData[i].length();
      if (iCurrentSize > iMaxSize) {
        iMaxSize = iCurrentSize;
      }
    }
    
    
    //3. fill with spaces right
    if (bRight) {
      for (int i = 0; i < iEntryNum; i++) {
        String pCurrentData = pData[i];
        
        while (pCurrentData.length() < iMaxSize) {
          pCurrentData += " ";
        }
        
        pData[i] = pCurrentData;
      }
    }
    
    //4. fill with spaces left
    else {
      for (int i = 0; i < iEntryNum; i++) {
        String pCurrentData = pData[i];
        
        while (pCurrentData.length() < iMaxSize) {
          pCurrentData = " " + pCurrentData;
        }
        
        pData[i] = pCurrentData;
      }
    }
    
    return pData;
  }
  

}