package io;

/*
 *  Copyright (C) 2014 Juergen Platzer 
 * 
 *  This file is part of JFXLibrary.
 *  
 *  JFXLibrary is free software: you can redistribute it and/or modify
 *  it under the terms of the GNU Lesser General Public License as published by
 *  the Free Software Foundation, either version 3 of the License, or
 *  (at your option) any later version.
 *  
 *  JFXLibrary is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU Lesser General Public License for more details.
 *  
 *  You should have received a copy of the GNU Lesser General Public License
 *  along with JFXLibrary.  If not, see <http://www.gnu.org/licenses/>.
 */

//library imports
import standard.ImageSequence;
import standard.BooleanParameter;



/**
 *  Provides methods to read in configuration files to configure the output settings.
 **/
public class RenderingOutputConfiguration {


  /*** constants for fetching parameters concerning the video ***/
  private static final String DIRECTORY = "renderingoutput.directory";
  private static final String FILEBASENAME = "renderingoutput.filebasename";
  private static final String FIRSTFRAME = "renderingoutput.firstframe";
  private static final String FILETYPE = "renderingoutput.filetype";
  private static final String LEADING_ZEROS = "renderingoutput.leadingZeros";
  private static final String CREATE_DIRECTORIES = "renderingoutput.createDirectories";
  private static final String OVERWRITE_IMAGE_FILES = "renderingoutput.overwriteImageFiles";
  

  /**
   *  Creates a {@code RenderingOutput} object from a configuration file. Exception messages by configuring the frame differencing effect 
   *  are written to the command line.
   *
   *  @param pConfiguration The configuration object that handles the direct access to the configuration file. 
   *
   *
   *  @return The {@code RenderingOutput} object that is created by the settings of the specified configuration file.
   **/

  public static RenderingOutput getRenderingOutput(Configuration pConfiguration) {
    
    //1. create the RenderingOutput and return it, if there is no configuration
    RenderingOutput pOutput = new RenderingOutput();
    if (pConfiguration == null) return pOutput;
    
    
    //2. fetch the configuration parameters and set them
    //2.1. string parameters
    pOutput.setDirectory(pConfiguration.getParameterValueWithoutException(DIRECTORY));
    pOutput.setFileBaseName(pConfiguration.getParameterValueWithoutException(FILEBASENAME));
    
    //2.2. integer parameters
    try {
      int iFirstFrame = Integer.parseInt(pConfiguration.getParameterValue(FIRSTFRAME));
      pOutput.setFirstFrame(iFirstFrame);
    }
    catch (Exception pException) {
    }
    
    
    int iFileType = ImageSequence.FILE_TYPE_PNG;
    String pFileType = pConfiguration.getParameterValueWithoutException(FILETYPE);
    if (pFileType != null) {
      if (pFileType.equalsIgnoreCase("jpeg") || pFileType.equalsIgnoreCase("jpg")) {
        iFileType = ImageSequence.FILE_TYPE_JPG;
      }
      else if (pFileType.equalsIgnoreCase("BMP")) {
        iFileType = ImageSequence.FILE_TYPE_BMP;
      }
    }
    pOutput.setFileType(iFileType);
    
    
    //2.3. boolean flags
    try {
      BooleanParameter pParameter = pConfiguration.getBooleanParameter(LEADING_ZEROS);
      pOutput.useLeadingZeros(pParameter.getValue());
    }
    catch (Exception pException) {}
    
    try {
      BooleanParameter pParameter = pConfiguration.getBooleanParameter(CREATE_DIRECTORIES);
      pOutput.createDirectories(pParameter.getValue());
    }
    catch (Exception pException) {}
    
    try {
      BooleanParameter pParameter = pConfiguration.getBooleanParameter(OVERWRITE_IMAGE_FILES);
      pOutput.overwriteImageFiles(pParameter.getValue());
    }
    catch (Exception pException) {}
    
    return pOutput;
    

    
    
  }
  
  
 


}