package io;

/*
 *  Copyright (C) 2014 Juergen Platzer 
 * 
 *  This file is part of JFXLibrary.
 *  
 *  JFXLibrary is free software: you can redistribute it and/or modify
 *  it under the terms of the GNU Lesser General Public License as published by
 *  the Free Software Foundation, either version 3 of the License, or
 *  (at your option) any later version.
 *  
 *  JFXLibrary is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU Lesser General Public License for more details.
 *  
 *  You should have received a copy of the GNU Lesser General Public License
 *  along with JFXLibrary.  If not, see <http://www.gnu.org/licenses/>.
 */

//library imports
import standard.IVideo;
import standard.ImageSequence;

/**
 *  Provides methods to read in configuration files to define a video object.
 **/

public class VideoConfiguration {


  /*** constants for fetching parameters concerning the video ***/
  private static final String FILEBASENAME = "video.filebasename";
  private static final String FIRSTFRAME = "video.firstframe";
  private static final String LASTFRAME = "video.lastframe";
  private static final String FILETYPE = "video.filetype";
  
  
  /**
   *  Creates a {@code IVideo} object from a configuration file. Exception messages by configuring the video object
   *  are written to the command line.
   *
   *  @param pConfiguration The configuration object that handles the direct access to the configuration file. 
   *
   *  @throws NullPointerException when no configuration object has been specified.
   *  @throws Exception when the video object could not be configured.
   *
   *  @return The {@code IVideo} object that is represents the video defined by the settings of the specified configuration file.
   **/  
  public static IVideo getVideo(Configuration pConfiguration) throws Exception {
    //1. make a simple check
    if (pConfiguration == null) throw new NullPointerException("No configuration was specified!");
    
    //2. fetch the video parameters 
    String pFileBaseName = null;
    String pFirstFrame = null;
    String pLastFrame = null;
    String pFileType = null;
    try {
      pFileBaseName = pConfiguration.getParameterValue(FILEBASENAME);
      pFirstFrame = pConfiguration.getParameterValue(FIRSTFRAME);
      pLastFrame = pConfiguration.getParameterValue(LASTFRAME);
      pFileType = pConfiguration.getParameterValue(FILETYPE);
    }
    catch (Exception pException) {
      throw new Exception("The configuration file is invalid. A video parameter is missing!", pException);
    }
    
    //3. prepare the parameters and create the image sequence object
    int iFirstFrame = -1;
    int iLastFrame = -1;
    
    try {
      iFirstFrame = Integer.parseInt(pFirstFrame);
      iLastFrame = Integer.parseInt(pLastFrame);
    }
    catch (Exception pException) {
      throw new Exception("The parameter values for '" + FIRSTFRAME + 
      "' or '" + LASTFRAME + "' could not be converted to an integer!", pException);
    }
    
    int iFileType;
    if (pFileType.equalsIgnoreCase("PNG")) {
      iFileType = ImageSequence.FILE_TYPE_PNG;
    }
    else if (pFileType.equalsIgnoreCase("BMP")) {
      iFileType = ImageSequence.FILE_TYPE_BMP;
    }
    else if (pFileType.equalsIgnoreCase("JPG") || pFileType.equalsIgnoreCase("JPEG")) {
      iFileType = ImageSequence.FILE_TYPE_JPG;
    }
    else {
      throw new Exception("The parameter value for '" + FILETYPE + "' is not valid!");
    }
    
    return new ImageSequence(pFileBaseName, iFirstFrame, iLastFrame, iFileType);
  
  }

}