package motionanalysis;

/*
 *  Copyright (C) 2014 Juergen Platzer 
 * 
 *  This file is part of JFXLibrary.
 *  
 *  JFXLibrary is free software: you can redistribute it and/or modify
 *  it under the terms of the GNU Lesser General Public License as published by
 *  the Free Software Foundation, either version 3 of the License, or
 *  (at your option) any later version.
 *  
 *  JFXLibrary is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU Lesser General Public License for more details.
 *  
 *  You should have received a copy of the GNU Lesser General Public License
 *  along with JFXLibrary.  If not, see <http://www.gnu.org/licenses/>.
 */

//java imports
import java.util.Hashtable;

import java.awt.Color;

//library imports
import standard.EffectParameters;
import standard.BooleanParameter;
import standard.IntegerParameter;
import standard.ColorParameter;



/**
 *   Encapsulates the parameters that can be specified for the frame differencing analysis. 
 **/
public class FrameDifferencingParameters extends EffectParameters {

  ////////////////////////////////
  // BOOLEAN PARAMETERS         //
  ////////////////////////////////

  /*** KEY for the boolean parameter: convert difference image. It defines whether the difference 
   *   difference image should be converted to a grayscale image before the thresholding is performed.
   ***/
  public static final String PARAM_BOOL_CONVERT_DIFF_IM = "CONVERT_DIFF_IM";

  /*** KEY for the boolean parameter: show original frame. It defines whether the original 
   *   frame should be shown in the result video. 
   ***/
  public static final String PARAM_BOOL_SHOW_ORIG_FRAME = "SHOW_ORIG_FRAME";

  /*** KEY for the boolean parameter: show difference image. It defines whether the difference image 
   *   should be shown in the result video. 
   ***/
  public static final String PARAM_BOOL_SHOW_DIFF_IM = "SHOW_DIFF_IM";

  /*** KEY for the boolean parameter: show motion mask. It defines whether the frame differencing 
   *   motion mask should be shown in the result video. 
   ***/
  public static final String PARAM_BOOL_SHOW_MOTION_MASK = "SHOW_MOTION_MASK";

  /*** KEY for the boolean parameter: show history. It defines whether the frame differencing 
   *   history should be shown in the result video. 
   ***/
  public static final String PARAM_BOOL_SHOW_HISTORY = "SHOW_HISTORY";

  /*** KEY for the boolean parameter: show result. It defines whether the frame differencing 
   *   result should be shown in the result video. 
   ***/
  public static final String PARAM_BOOL_SHOW_RESULT = "SHOW_RESULT";

  /*** KEY for the boolean parameter: history on. It defines whether the differencing history be shown. ***/
  public static final String PARAM_BOOL_HISTORY_ON = "HISTORY_ON";

  /*** KEY for the boolean parameter: keep history. It defines whether the differencing history should be 
   *   shown all the time for the rest of the video. 
   **/
  public static final String PARAM_BOOL_HISTORY_KEEP = "HISTORY_KEEP";

  /*** KEY for the boolean parameter: difference history fade. It decides whether a 
   *   fading of the difference history should be used. 
   ***/
  public static final String PARAM_BOOL_HISTORY_FADE = "HISTORY_FADE";


  ////////////////////////////////
  // INTEGER PARAMETERS         //
  ////////////////////////////////
  
  /*** KEY for the integer parameter: threshold. It defines the decision value at which differences in the frame are
       detected. If the difference value at a pixel is higher than the threshold it is marked. 
   **/
  public static final String PARAM_INT_THRESHOLD = "THRESHOLD";

  /*** KEY for the integer parameter: history frame number. It defines the number of frames that 
   * the difference history should be shown (exclusive fade out).
   **/
  public static final String PARAM_INT_HISTORY_FRAME_NUM = "HISTORY_FRAME_NUM";

  /*** KEY for the integer parameter: fade out frame number. 
   * It defines how many frames should be used to let the difference history color fade out.
   **/
  public static final String PARAM_INT_HISTORY_FADE_FRAME_NUM = "HISTORY_FADE_FRAME_NUM";


  ////////////////////////////////
  // COLOR PARAMETERS           //
  ////////////////////////////////

  /*** KEY for the color parameter: history color. It defines in which color the difference history is shown.**/
  public static final String PARAM_COLOR_HISTORY_COLOR = "HISTORY_COLOR";
  



  /***
   *   Creates the default frame differencing parameter set. The following values are used:
   *
   *   <ul>
   *    <li>Convert difference image to grayscale image: true</li>
   *    <li>Show original frame: false</li>
   *    <li>Show difference image: false</li>
   *    <li>Show motion mask: false</li>
   *    <li>Show history image: false</li>
   *    <li>Show result image: true</li>
   *    <li>History on: false</li>
   *    <li>Keep history: false</li>
   *    <li>History fading: false</li>
   *    <li>Threshold: 128 = the middle of the possible values (0, 255)</li>
   *    <li>History frame number: 1</li>
   *    <li>History fade out frame number: 1</li>
   *    <li>History color: red</li>
   *  </ul>
   **/
  public FrameDifferencingParameters() {

    //1. create the parameters and set them
    setBooleanParameters(createBooleanParameters());
    setIntegerParameters(createIntegerParameters());
    setColorParameters(createColorParameters());

  }


  ////////////////////////////////////////////
  // PRIVATE METHODS FOR PARAMETER CREATION //
  ////////////////////////////////////////////
  private Hashtable<String, BooleanParameter> createBooleanParameters() {

    //1. create the hashtable
    Hashtable<String, BooleanParameter> pParameters = new Hashtable<String, BooleanParameter>();

    //2. fill the hashtable
    pParameters.put(PARAM_BOOL_CONVERT_DIFF_IM, new BooleanParameter(true));
    pParameters.put(PARAM_BOOL_SHOW_ORIG_FRAME, new BooleanParameter(false));
    pParameters.put(PARAM_BOOL_SHOW_DIFF_IM, new BooleanParameter(false));
    pParameters.put(PARAM_BOOL_SHOW_MOTION_MASK, new BooleanParameter(false));
    pParameters.put(PARAM_BOOL_SHOW_HISTORY, new BooleanParameter(false));
    pParameters.put(PARAM_BOOL_SHOW_RESULT, new BooleanParameter(true));
    pParameters.put(PARAM_BOOL_HISTORY_ON, new BooleanParameter(false));
    pParameters.put(PARAM_BOOL_HISTORY_KEEP, new BooleanParameter(false));
    pParameters.put(PARAM_BOOL_HISTORY_FADE, new BooleanParameter(false));

    //3. return the hashtable
    return pParameters;
  }




  private Hashtable<String, IntegerParameter> createIntegerParameters() {
    //1. create the hashtable
    Hashtable<String, IntegerParameter> pParameters = new Hashtable<String, IntegerParameter>();

    //2. fill the hashtable
    pParameters.put(PARAM_INT_THRESHOLD, new IntegerParameter(128, 0, 255));
    pParameters.put(PARAM_INT_HISTORY_FRAME_NUM, new IntegerParameter(25, 1, 250));
    pParameters.put(PARAM_INT_HISTORY_FADE_FRAME_NUM, new IntegerParameter(25, 1, 250));

    //3. return the hashtable
    return pParameters;
  }
  
  
  private Hashtable<String, ColorParameter> createColorParameters() {
    //1. create the hashtable
    Hashtable<String, ColorParameter> pParameters = new Hashtable<String, ColorParameter>();

    //2. fill the hashtable
    pParameters.put(PARAM_COLOR_HISTORY_COLOR, new ColorParameter(Color.RED));
    
    //3. return the hashtable
    return pParameters;
  
  }



}
