package standard;

/*
 *  Copyright (C) 2014 Juergen Platzer 
 * 
 *  This file is part of JFXLibrary.
 *  
 *  JFXLibrary is free software: you can redistribute it and/or modify
 *  it under the terms of the GNU Lesser General Public License as published by
 *  the Free Software Foundation, either version 3 of the License, or
 *  (at your option) any later version.
 *  
 *  JFXLibrary is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU Lesser General Public License for more details.
 *  
 *  You should have received a copy of the GNU Lesser General Public License
 *  along with JFXLibrary.  If not, see <http://www.gnu.org/licenses/>.
 */

//standard imports
import java.awt.event.ActionListener;
import java.awt.event.ActionEvent;

import java.util.Vector;

//library imports
//import gui.util.DoubleSliderPanel;

/**
 *  Represents an image or video effect parameter that has a double value. 
 **/
public class DoubleParameter implements IParameter, ActionListener {


  /*** value variables ***/
  private double m_dMinValue;
  private double m_dMaxValue;
  private InterpolationContainer<InterpolatedDouble> m_pInterpolationContainer;

  /*** gui variables ***/
  //DoubleSliderPanel m_pSliderPanel;


  /////////////////////////
  // CONSTRUCTOR         //
  /////////////////////////
  /**
   *  Creates a {@code DoubleParameter} object with the specified double value and the allowed minimum and maximum value. 
   *
   * @param dValue The double value that should be represented by this {@code DoubleParameter} object.
   * @param dMinValue The minimum double value that can be represented by this {@code DoubleParameter} object.
   * @param dMaxValue The maximum double value that can be represented by this {@code DoubleParameter} object.
   **/
  public DoubleParameter(double dValue, double dMinValue, double dMaxValue) {

    //1. set the member variables
    m_pInterpolationContainer = new InterpolationContainer<InterpolatedDouble>();
    m_dMinValue = dMinValue;
    m_dMaxValue = dMaxValue;
    
    

    //2. set the first value
    setValue(dValue);
    
    //3. set the slider panel to null
    //m_pSliderPanel = null;
    
    
  }



  /////////////////////////
  // SET METHODS         //
  /////////////////////////
  /**
   * Sets the double value that should be represented by this {@code DoubleParameter} object.
   * The double value is representated by a string. 
   *
   * @param pValue The string value representing the double that should be represented by this {@code DoubleParameter} object.
   *
   * @return true, if the double value could be successfully set. False otherwise.
   **/   
  public boolean setValue (String pValue) {
    //1. make a simple check
    if (pValue == null) return false;
    
    //2. try to set the value
    try {
      setValue(Double.parseDouble(pValue));
      return true;
    }
    catch (Exception pException) {
      return false;
    }
    
  }

  
  /**
   * Sets the double value that should be represented by this {@code DoubleParameter} object.
   *
   * @param dValue The double value that should be represented by this {@code DoubleParameter} object.
   **/   
  public void setValue(double dValue) {

    //1. make some checks
    if (dValue < m_dMinValue) dValue = m_dMinValue;
    if (dValue > m_dMaxValue) dValue = m_dMaxValue;

    //2. set the value
    m_pInterpolationContainer.setValue(new InterpolatedDouble(dValue), 0);

    //3. slider panel
    /*if (m_pSliderPanel != null) {
      m_pSliderPanel.setValue(dValue);
    }*/

  }
  
  /**
   * Sets the double value that should be represented by this {@code DoubleParameter} object for a specified point in time of a video.
   *
   * @param dValue The double value that should be represented by this {@code DoubleParameter} object for a specified point in time of a video.
   * @param dTime The point in time of a video as double.
   **/   
  public void setValue(double dValue, double dTime) {
    //1. make some checks
    if (dValue < m_dMinValue) dValue = m_dMinValue;
    if (dValue > m_dMaxValue) dValue = m_dMaxValue;

    //2. set the value
    m_pInterpolationContainer.setValue(new InterpolatedDouble(dValue), dTime);
    
    //3. TO DO handle GUI Elements
    
  }

  /**
   * Sets the minimum double value that can be represented by this {@code DoubleParameter} object.
   *
   * @param dMinValue The minimum double value that can be represented by this {@code DoubleParameter} object.
   **/   
  public void setMinimum(double dMinValue) {

    //1. set the minimum value and validate the values
    m_dMinValue = dMinValue;
    validateTheVariables();


  }

  /**
   * Sets the maximum double value that can be represented by this {@code DoubleParameter} object.
   *
   * @param dMaxValue The maximum double value that can be represented by this {@code DoubleParameter} object.
   **/   
  public void setMaximum(double dMaxValue) {

    //2. set the minimum value and validate the values
    m_dMaxValue = dMaxValue;
    validateTheVariables();


  }



  /////////////////////////
  // GET METHODS         //
  /////////////////////////
  /**
   * Returns the double value that is represented by this {@code DoubleParameter} object.
   *
   * @return The double value that is represented by this {@code DoubleParameter} object.
   **/   
  public double getValue() {
    return m_pInterpolationContainer.getValue(0).doubleValue();
  }
  
  /**
   * Returns the double value that is represented by this {@code DoubleParameter} object for the specified point of time.
   *
   * @param dTime The point in time of a video as double.
   *
   * @return The double value that is represented by this {@code DoubleParameter} object for the specified point of time.
   **/   
  public double getValue(double dTime) {
    return m_pInterpolationContainer.getValue(dTime).doubleValue();
  }

  /**
   * Returns the minimum double value that can be represented by this {@code DoubleParameter} object.
   *
   * @return The minimum double value that can be represented by this {@code DoubleParameter} object.
   *
   **/  
  public double getMinimum() {
    return m_dMinValue;
  }

  /**
   * Returns the maximum double value that can be represented by this {@code DoubleParameter} object.
   *
   * @return The maximum double value that can be represented by this {@code DoubleParameter} object.
   *
   **/  
  public double getMaximum() {
    return m_dMaxValue;
  }



/*
  public DoubleSliderPanel getImageEffectGUIElement(String pLabelText) {

    //1. make a simple check
    if (pLabelText == null) pLabelText = "";

    //2. create and return the SliderPanel
    m_pSliderPanel = new DoubleSliderPanel(pLabelText, m_pInterpolationContainer.getValue(0).doubleValue(), m_dMinValue, m_dMaxValue, 0.01);
    m_pSliderPanel.addActionListener(this);

    return m_pSliderPanel;
    
  }

*/

  ////////////////////////////////
  // ACTION LISTENER INTERFACE  //
  ////////////////////////////////
  /**
   *  Not used at the moment. 
   *
   *  @param e event that triggers this method.
   **/
  public void actionPerformed(ActionEvent e) {

    /*
    //1. make some checks
    if (m_pSliderPanel == null) return;


    //2. set the value
    InterpolatedDouble pValue = new InterpolatedDouble(m_pSliderPanel.getValue());
    m_pInterpolationContainer.setValue(pValue, 0);

    */
    
  }



  ////////////////////////////////
  // PRIVATE METHODS            //
  ////////////////////////////////
  private void validateTheVariables() {

    //1. make sure that the minimum is the minimum
    if (m_dMinValue > m_dMaxValue) {
      double dHelp = m_dMinValue;
      m_dMinValue = m_dMaxValue;
      m_dMaxValue = dHelp;
    }

    //2. make sure that the parameter values are valid
    Vector<Double> pTimestamps = m_pInterpolationContainer.getTimestamps();
    int iEntryNum = pTimestamps.size();
    
    for (int i = 0; i < iEntryNum; i++) {
      double dTimestamp = pTimestamps.get(i).doubleValue();
      double dParameterValue = m_pInterpolationContainer.getValue(dTimestamp).doubleValue();
      boolean bModified = false;
      
      if (dParameterValue < m_dMinValue) {
        dParameterValue = m_dMinValue;
        bModified = true;
      }
      
      if (dParameterValue > m_dMaxValue) {
        dParameterValue = m_dMaxValue;
        bModified = true;
      }
      
      if (bModified) m_pInterpolationContainer.setValue(new InterpolatedDouble(dParameterValue), dTimestamp);
    }

    //3. handle the slider panel
    /*if (m_pSliderPanel != null) {
      m_pSliderPanel.reset(m_pInterpolationContainer.getValue(0).doubleValue(), m_dMinValue, m_dMaxValue);
    }*/

  }


}
