package standard;

/*
 *  Copyright (C) 2014 Juergen Platzer 
 * 
 *  This file is part of JFXLibrary.
 *  
 *  JFXLibrary is free software: you can redistribute it and/or modify
 *  it under the terms of the GNU Lesser General Public License as published by
 *  the Free Software Foundation, either version 3 of the License, or
 *  (at your option) any later version.
 *  
 *  JFXLibrary is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU Lesser General Public License for more details.
 *  
 *  You should have received a copy of the GNU Lesser General Public License
 *  along with JFXLibrary.  If not, see <http://www.gnu.org/licenses/>.
 */

//standard imports
import java.awt.event.ActionListener;
import java.awt.event.ActionEvent;

import java.util.Vector;

//library imports
/*import gui.util.IntegerSliderPanel;
import gui.util.IntegerInterpolationPanel;
import gui.util.InterpolationPanel;
*/

/**
 *  Represents an image or video effect parameter that has an integer value. 
 **/

public class IntegerParameter implements IParameter, ActionListener {


  /*** value variables ***/
  private int m_iMinValue;
  private int m_iMaxValue;
  private InterpolationContainer<InterpolatedInteger> m_pInterpolationContainer;

  /*** gui variables ***/
  //private IntegerSliderPanel m_pIntegerSliderPanel;
  //private IntegerInterpolationPanel m_pIntegerInterpolationPanel;


  /////////////////////////
  // CONSTRUCTOR         //
  /////////////////////////
  /**
   *  Creates an {@code IntegerParameter} object with the specified integer value and the allowed minimum and maximum value. 
   *
   * @param iValue The integer value that should be represented by this {@code IntegerParameter} object.
   * @param iMinValue The minimum integer value that can be represented by this {@code IntegerParameter} object.
   * @param iMaxValue The maximum integer value that can be represented by this {@code IntegerParameter} object.
   **/
public IntegerParameter(int iValue, int iMinValue, int iMaxValue) {

    //1. set the member variables
    m_pInterpolationContainer = new InterpolationContainer<InterpolatedInteger>();
    m_iMinValue = iMinValue;
    m_iMaxValue = iMaxValue;
    
    

    //2. set the first value
    setValue(iValue);
    
    //3. set gui elements to null
    //m_pIntegerSliderPanel = null;
    //m_pIntegerInterpolationPanel = null;
    
    
  }



  /////////////////////////
  // SET METHODS         //
  /////////////////////////
  /**
   * Sets the integer value that should be represented by this {@code IntegerParameter} object.
   * The integer value is representated by a string. 
   *
   * @param pValue The string value representing the integer that should be represented by this {@code IntegerParameter} object.
   *
   * @return true, if the integer value could be successfully set. False otherwise.
   **/   
  public boolean setValue (String pValue) {

    //1. make a simple check
    if (pValue == null) return false;
    
    //2. try to set the value
    try {
      setValue(Integer.parseInt(pValue));
      return true;
    }
    catch (Exception pException) {
      return false;
    }
    
    
  }

  
  /**
   * Sets the integer value that should be represented by this {@code IntegerParameter} object.
   *
   * @param iValue The integer value that should be represented by this {@code IntegerParameter} object.
   **/   
  public void setValue(int iValue) {

    //1. make some checks
    if (iValue < m_iMinValue) iValue = m_iMinValue;
    if (iValue > m_iMaxValue) iValue = m_iMaxValue;

    //2. set the value
    m_pInterpolationContainer.setValue(new InterpolatedInteger(iValue), 0);

    //3. slider panel
    /*if (m_pIntegerSliderPanel != null) {
      m_pIntegerSliderPanel.setValue(iValue);
    }*/


  }
  
  
  /**
   * Sets the integer value that should be represented by this {@code IntegerParameter} object for a specified point in time of a video.
   *
   * @param iValue The integer value that should be represented by this {@code IntegerParameter} object for a specified point in time of a video.
   * @param dTime The point in time of a video as double.
   **/   
  public void setValue(int iValue, double dTime) {
    //1. make some checks
    if (iValue < m_iMinValue) iValue = m_iMinValue;
    if (iValue > m_iMaxValue) iValue = m_iMaxValue;

    //2. set the value
    m_pInterpolationContainer.setValue(new InterpolatedInteger(iValue), dTime);
    
    //3. TO DO handle GUI Elements
    
  }


  /**
   * Sets the minimum integer value that can be represented by this {@code IntegerParameter} object.
   *
   * @param iMinValue The minimum integer value that can be represented by this {@code IntegerParameter} object.
   **/   
  public void setMinimum(int iMinValue) {

    //1. set the minimum value and validate the values
    m_iMinValue = iMinValue;
    validateTheVariables();


  }


  /**
   * Sets the maximum integer value that can be represented by this {@code IntegerParameter} object.
   *
   * @param iMaxValue The maximum integer value that can be represented by this {@code IntegerParameter} object.
   **/   
  public void setMaximum(int iMaxValue) {

    //2. set the minimum value and validate the values
    m_iMaxValue = iMaxValue;
    validateTheVariables();


  }



  /////////////////////////
  // GET METHODS         //
  /////////////////////////
  /**
   * Returns the integer value that is represented by this {@code IntegerParameter} object.
   *
   * @return The integer value that is represented by this {@code IntegerParameter} object.
   **/   
  public int getValue() {
    //System.out.println("GET: " + m_pInterpolationContainer.getValue(0).intValue());
    return m_pInterpolationContainer.getValue(0).intValue();
  }
  
  /**
   * Returns the integer value that is represented by this {@code IntegerParameter} object for the specified point of time.
   *
   * @param dTime The point in time of a video as double.
   *
   * @return The integer value that is represented by this {@code IntegerParameter} object for the specified point of time.
   **/   
  public int getValue(double dTime) {
    return m_pInterpolationContainer.getValue(dTime).intValue();
  }
  
  
  
  /**
   *  Returns the timestamps for which integer values have been defined.
   *  
   *  @return The timestamps for which integer values have been defined as vector of double values.
   **/
  public Vector<Double> getTimestamps() {
    return m_pInterpolationContainer.getTimestamps();
  }


  /**
   * Returns the minimum integer value that can be represented by this {@code IntegerParameter} object.
   *
   * @return The minimum integer value that can be represented by this {@code IntegerParameter} object.
   *
   **/  
  public int getMinimum() {
    return m_iMinValue;
  }


  /**
   * Returns the maximum integer value that can be represented by this {@code IntegerParameter} object.
   *
   * @return The maximum integer value that can be represented by this {@code IntegerParameter} object.
   *
   **/  
  public int getMaximum() {
    return m_iMaxValue;
  }



  /*
  public IntegerSliderPanel getImageEffectGUIElement(String pLabelText) {

    //1. make a simple check
    if (pLabelText == null) pLabelText = "";

    //2. create and return the IntegerSliderPanel
    if (m_pIntegerSliderPanel == null) {
      m_pIntegerSliderPanel = new IntegerSliderPanel(pLabelText, m_pInterpolationContainer.getValue(0).intValue(), m_iMinValue, m_iMaxValue);
      m_pIntegerSliderPanel.addActionListener(this);
    }

    return m_pIntegerSliderPanel;
  }
  */


  /*
  public InterpolationPanel getVideoEffectGUIElement(String pLabelText) {
    //1. make a simple check
    if (pLabelText == null) pLabelText = "";
    
    //2. create and return the IntegerInterpolationPanel
    if (m_pIntegerInterpolationPanel == null) {
      m_pIntegerInterpolationPanel = new IntegerInterpolationPanel(this);
    }

    return new InterpolationPanel(pLabelText, m_pIntegerInterpolationPanel);
  }
  */
  

  ////////////////////////////////
  // ACTION LISTENER INTERFACE  //
  ////////////////////////////////
  /**
   * Not used at the moment. 
   *
   * @param e The event that triggers this method.
   **/
  public void actionPerformed(ActionEvent e) {
    /*
    //1. make some checks
    if (m_pIntegerSliderPanel == null) return;


    //2. set the value
    InterpolatedInteger pValue = new InterpolatedInteger(m_pIntegerSliderPanel.getValue());
    m_pInterpolationContainer.setValue(pValue, 0);
    */
  }



  ////////////////////////////////
  // PRIVATE METHODS            //
  ////////////////////////////////
  private void validateTheVariables() {

    //1. make sure that the minimum is the minimum
    if (m_iMinValue > m_iMaxValue) {
      int iHelp = m_iMinValue;
      m_iMinValue = m_iMaxValue;
      m_iMaxValue = iHelp;
    }

    //2. make sure that the parameter values are valid
    Vector<Double> pTimestamps = m_pInterpolationContainer.getTimestamps();
    int iEntryNum = pTimestamps.size();
    
    for (int i = 0; i < iEntryNum; i++) {
      double dTimestamp = pTimestamps.get(i).doubleValue();
      int iParameterValue = m_pInterpolationContainer.getValue(dTimestamp).intValue();
      boolean bModified = false;
      
      if (iParameterValue < m_iMinValue) {
        iParameterValue = m_iMinValue;
        bModified = true;
      }
      
      if (iParameterValue > m_iMaxValue) {
        iParameterValue = m_iMaxValue;
        bModified = true;
      }
      
      if (bModified) m_pInterpolationContainer.setValue(new InterpolatedInteger(iParameterValue), dTimestamp);
    }

    //3. handle the slider panel
    /*if (m_pIntegerSliderPanel != null) {
      m_pIntegerSliderPanel.reset(m_pInterpolationContainer.getValue(0).intValue(), m_iMinValue, m_iMaxValue);
    }*/

  }


}
