package standard;

/*
 *  Copyright (C) 2014 Juergen Platzer 
 * 
 *  This file is part of JFXLibrary.
 *  
 *  JFXLibrary is free software: you can redistribute it and/or modify
 *  it under the terms of the GNU Lesser General Public License as published by
 *  the Free Software Foundation, either version 3 of the License, or
 *  (at your option) any later version.
 *  
 *  JFXLibrary is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU Lesser General Public License for more details.
 *  
 *  You should have received a copy of the GNU Lesser General Public License
 *  along with JFXLibrary.  If not, see <http://www.gnu.org/licenses/>.
 */

//standard imports
import java.awt.Color;


/**
 *  An extension of the {@code Color} class that allows to calculate interpolated color values. 
 **/
public class InterpolatedColor extends Color implements IInterpolated<InterpolatedColor> {

  //////////////////////////////////
  // CONSTRUCTORS                 //
  //////////////////////////////////
  /**
   *  Creates an opaque sRGB color with the specified combined RGB value consisting of the 
   *  red component in bits 16-23, the green component in bits 8-15, and the blue component 
   *  in bits 0-7. The actual color used in rendering depends on finding the best match 
   *  given the color space available for a particular output device. Alpha is defaulted to 255.
   *
   *  @param iRGB the combined RGB components
   **/
  public InterpolatedColor(int iRGB) {
    super(iRGB);
  }
  
  /**
   *  Creates an sRGB color with the specified red, green, blue, and alpha values in the range (0 - 255).
   *
   *  @param iRed The red component. 
   *  @param iGreen The green component. 
   *  @param iBlue The blue component.
   *  @param iAlpha The alpha component.
   **/
  public InterpolatedColor(int iRed, int iGreen, int iBlue, int iAlpha) {
    super(iRed, iGreen, iBlue, iAlpha);
  }
  
  /**
   *  Creates an opaque sRGB color with the specified red, green, and blue values in the range (0 - 255). 
   *  The actual color used in rendering depends on finding the best match given the color space 
   *  available for a given output device. Alpha is defaulted to 255.
   *
   *  @param iRed The red component. 
   *  @param iGreen The green component. 
   *  @param iBlue The blue component.
   **/
  public InterpolatedColor(int iRed, int iGreen, int iBlue) {
    super(iRed, iGreen, iBlue);
  }
  
  /**
   *  Creates an interpolated color object from a given color. 
   *
   *  @param pColor The color that should be represented.
   **/
  public InterpolatedColor(Color pColor) {
    super(pColor.getRed(), pColor.getGreen(), pColor.getBlue(), pColor.getAlpha());
  }



  /////////////////////////////////
  // IINTERPOLATED INTERFACE     //
  /////////////////////////////////
  /**
   *  Returns an {@code InterpolatedColor} object representing the color which is an interpolation between the color of this object, 
   *  and the color that has been passed on as parameter. 
   *
   *  @param pOtherObject The second color that is needed to calculate the interpolated color. 
   *  @param dLambda Steers the weight of the color. A value of 1 means that this color gets fully considered. 
   *                 A value of 0 means that the parameter color gets fully considered. (TO DO: Should not it be the other way round?)
   *
   *  @return The resulting color of the interpolation.
   **/
  public InterpolatedColor interpolateLinearly(InterpolatedColor pOtherObject, double dLambda) {
    //1. make a simple check
    if (pOtherObject == null) return new InterpolatedColor(getRGB());
  
    //2. fetch values
    int iRGB1 = getRGB();
    int iRGB2 = pOtherObject.getRGB();
    
    int iRed1 = (iRGB1 >> 16) & 0xff;
    int iRed2 = (iRGB2 >> 16) & 0xff;
    
    int iGreen1 = (iRGB1 >> 8) & 0xff;
    int iGreen2 = (iRGB2 >> 8) & 0xff;
    
    int iBlue1 = (iRGB1 >> 0) & 0xff;
    int iBlue2 = (iRGB2 >> 0) & 0xff;
    
    int iAlpha1 = (iRGB1 >> 24) & 0xff;
    int iAlpha2 = (iRGB2 >> 24) & 0xff;
    
    //3. interpolate
    double dRed = (dLambda)*((double)iRed1) + (1 -dLambda)*((double)iRed2);
    double dGreen = (dLambda)*((double)iGreen1) + (1 -dLambda)*((double)iGreen2);
    double dBlue = (dLambda)*((double)iBlue1) + (1 -dLambda)*((double)iBlue2);
    double dAlpha = (dLambda)*((double)iAlpha1) + (1 -dLambda)*((double)iAlpha2);
    
    if (dRed > 255) dRed = 255;
    if (dRed < 0) dRed = 0;
    
    if (dGreen > 255) dGreen = 255;
    if (dGreen < 0) dGreen = 0;
    
    if (dBlue > 255) dBlue = 255;
    if (dBlue < 0) dBlue = 0;
    
    if (dAlpha > 255) dAlpha = 255;
    if (dAlpha < 0) dAlpha = 0;
   
    return new InterpolatedColor((int)dRed, (int)dGreen, (int)dBlue, (int)dAlpha); 
  }
  

}