package standard;

/*
 *  Copyright (C) 2014 Juergen Platzer 
 * 
 *  This file is part of JFXLibrary.
 *  
 *  JFXLibrary is free software: you can redistribute it and/or modify
 *  it under the terms of the GNU Lesser General Public License as published by
 *  the Free Software Foundation, either version 3 of the License, or
 *  (at your option) any later version.
 *  
 *  JFXLibrary is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU Lesser General Public License for more details.
 *  
 *  You should have received a copy of the GNU Lesser General Public License
 *  along with JFXLibrary.  If not, see <http://www.gnu.org/licenses/>.
 */

import java.util.Vector;

/**
 *  Manages a list of objects that can be interpolated. 
 **/
public class InterpolationContainer<E extends IInterpolated<E>> {

  //member variables
  private Vector<E> m_pValues;
  private Vector<Double> m_pTimestamps;
  
  
  /////////////////////////
  // CONSTRUCTORS        //
  /////////////////////////
  /**
   *  Creates an empty {@code InterpolationContainer} object. 
   **/
  public InterpolationContainer() {
    m_pValues = new Vector<E>(5);
    m_pTimestamps = new Vector<Double>(5);
  }
  
  
  //////////////////////////
  // MANIPULATION METHODS //
  //////////////////////////
  /**
   *  Sets a value at the specified point in time. 
   *
   *  @param pValue The value object that resides at the specified time. 
   *  @param dTime The time a double value between 0 and 1. 
   **/
  public void setValue(E pValue, double dTime) {
    //1. make some checks
    if (dTime < 0) dTime = 0;
    if (dTime > 1) dTime = 1;
    
    //2. if no value has been set -> just add the values
    int iValueNum = m_pValues.size();
    if (iValueNum == 0) {
      m_pValues.add(pValue);
      m_pTimestamps.add(new Double(dTime));
      return;
    }
    
    
    //3. check if there is already such a time entry, if yes set the value
    for (int i = 0; i < iValueNum; i++) {
      if (dTime == m_pTimestamps.get(i).doubleValue()) {
        m_pValues.remove(i);
        m_pValues.add(i, pValue);
        return;
      }
    }
    
    //4. if no valid position was found -> insert the value    
    //4.1. find the insertion position
    int iInsertPosition = -1;
    if (dTime < m_pTimestamps.get(0).doubleValue()) {
      iInsertPosition = 0;
    }
    else if (dTime > m_pTimestamps.get(iValueNum-1).doubleValue()) {
      iInsertPosition = iValueNum;
    }
    else {
      for (int i = 1; i < iValueNum; i++) {
        if ((dTime > m_pTimestamps.get(i-1).doubleValue()) && (dTime < m_pTimestamps.get(i).doubleValue())) {
          iInsertPosition = i;
          break;
        }
      }
    }
    
    //4.2. insert
    m_pValues.add(iInsertPosition, pValue);
    m_pTimestamps.add(iInsertPosition, new Double(dTime));
  }


  /**
   *  Removes the value object that resides at the specified time. 
   *
   *  @param dTime The point in time as double where the object can be found that should be removed. 
   *
   *  @return The value object that has been removed. If no object could be found at the specified time, null is returned. 
   **/
  public E removeValue(double dTime) {
  
    //1. make initial checks
    if (dTime < 0) dTime = 0;
    if (dTime > 1) dTime = 1;
    
    //2. try to remove the object
    int iValueNum = m_pValues.size();
    for (int i = 0; i < iValueNum; i++) {
      if (dTime == m_pTimestamps.get(i).doubleValue()) {
        m_pTimestamps.remove(i);
        return m_pValues.remove(i);
      }
    }
    
    //3. return null if no value could be removed
    return null;
  
  }
  
  
  /**
   *  Removes all value objects from this container. 
   **/
  public void removeAllValues() {
    m_pValues.removeAllElements();
    m_pTimestamps.removeAllElements();
  }
  
  
  
  //////////////////////////////////
  // GET METHODS                  //
  //////////////////////////////////
  /**
   *  Returns the value object that is at the specified point in time. 
   *  If there is an object at that position it is returned, otherwise an interpolated value is created. 
   *
   *  @param dTime The point in time as double for which the value object should be retrieved. 
   *
   *  @return The value object that is at the specified point in time.  
   **/
  public E getValue(double dTime) {
    
    //1. make some checks
    if (dTime < 0) dTime = 0;
    if (dTime > 1) dTime = 1;

    //2. if no value is set return null
    int iValueNum = m_pValues.size();
    if (iValueNum == 0) return null;
    
    //3. if only one value was set return this value
    if (iValueNum == 1) return m_pValues.get(0);
    
    //4. check if there is a value exactly at this time
    for (int i = 0; i < iValueNum; i++) {
      if (m_pTimestamps.get(i).doubleValue() == dTime) {
        return m_pValues.get(i);
      }
    }
    
    
    //5. find preceding and succeding elements for interpolation
    E pPreceding = null;
    E pSucceeding = null;
    double dLambda = 0;
    
    for (int i = 0; i < iValueNum-1; i++) {
      double dPrecedingTimestamp = m_pTimestamps.get(i).doubleValue();
      double dSucceedingTimestamp = m_pTimestamps.get(i+1).doubleValue();
      
      if ((dPrecedingTimestamp < dTime) && (dSucceedingTimestamp > dTime)) {
        pPreceding = m_pValues.get(i);
        pSucceeding = m_pValues.get(i+1);
        dLambda = (dSucceedingTimestamp-dTime)/(dSucceedingTimestamp - dPrecedingTimestamp); //= 1 if dTime == dPrecedingTimestamp
        break;
      }
    }
    
    //TO DO Interpolate

    
    return null;
  }
  
  
  /**
   *  Returns the list of points in time where value objects reside. 
   *
   *  @return The list of points in time where value objects reside.
   **/
  public Vector<Double> getTimestamps() {
    return m_pTimestamps;
  }
  
  
  
  


}