package standard;

/*
 *  Copyright (C) 2014 Juergen Platzer 
 * 
 *  This file is part of JFXLibrary.
 *  
 *  JFXLibrary is free software: you can redistribute it and/or modify
 *  it under the terms of the GNU Lesser General Public License as published by
 *  the Free Software Foundation, either version 3 of the License, or
 *  (at your option) any later version.
 *  
 *  JFXLibrary is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU Lesser General Public License for more details.
 *  
 *  You should have received a copy of the GNU Lesser General Public License
 *  along with JFXLibrary.  If not, see <http://www.gnu.org/licenses/>.
 */

//standard imports
import java.util.Vector;


/**
 *  All classes that need to notify thread listener objects should extend this class and use its functionality.
 **/

public abstract class ThreadEventManager {

  //List of ThreadListener objects
  private Vector<IThreadListener>   m_pThreadListeners  = null;


  //////////////////////////////////////////
  // METHODS FOR HANDLING THREADLISTENERS //
  //////////////////////////////////////////



  /**
   *  Adds a thread listener object. 
   *
   *  @param pThreadListener The thread listener object that should listen to this thread. 
   **/
  public void addThreadListener(IThreadListener pThreadListener) {

    //1. make a simple check
    if (pThreadListener == null) return;

    //2. create the list of IThreadListener objects
    if (m_pThreadListeners == null) {
      m_pThreadListeners = new Vector<IThreadListener>();
    }

    //3. add the IThreadListener object
    m_pThreadListeners.add(pThreadListener);

  }

  
  /**
   *  Adds a thread listener object at a specified position. TO DO: Is this really necessary? Also the positioning does not seem to work correct.
   *
   *  @param pThreadListener The thread listener object that should listen to this thread. 
   *  @param iPosition The position in the list of thread listeners where the listener should be added. 
   **/
  public void addThreadListener(IThreadListener pThreadListener, int iPosition) {

    //1. make a simple check
    if (pThreadListener == null) return;

    //2. create the list of IThreadListener objects
    if (m_pThreadListeners == null) {
      m_pThreadListeners = new Vector<IThreadListener>();
    }

    //3. add the IThreadListener object
    if (iPosition < 0) addThreadListener(pThreadListener);
    if (iPosition > m_pThreadListeners.size()) addThreadListener(pThreadListener);
    m_pThreadListeners.add(pThreadListener);

  }


  /**
   *  Removes the specified thread listener object. 
   *
   *  @param pThreadListener The thread listener that should no longer listen to this Thread. 
   **/
  public void removeThreadListener(IThreadListener pThreadListener) {
    //1. make a simple check
    if (m_pThreadListeners == null) return;
    if (pThreadListener == null) return;

    //2. remove the IThreadListener object, as soon as no pending operation is executed anymore
    m_pThreadListeners.remove(pThreadListener);
  }

  /**
   *  Removes all thread listener objects. 
   **/
  public void removeAllThreadListeners() {
    //1. make a simple check
    if (m_pThreadListeners == null) return;

    //2. remove all IThreadListener objects, as soon as no pending operation is executed anymore
    m_pThreadListeners.removeAllElements();
  }



  ////////////////////////////////////////////
  // METHODS FOR COMMUNICATING THREADEVENTS //
  ////////////////////////////////////////////
  /** 
   *  Fires a status event, that informs all thread listeners about the current status of the thread. 
   *  This method should only be called by classes that extend this class. 
   *
   *  @param dProgress The progress as a double between 0 and 1. 
   *  @param pCurrentWork A description of the current work that is processed. 
   **/
  protected void fireProgressStatusEvent(double dProgress, String pCurrentWork) {
    //1. make a simple value check
    if (dProgress < 0) dProgress = 0;
    if (dProgress > 1) dProgress = 1;
    if (m_pThreadListeners == null) return;


    //2. fire the progress status events
    IThreadListener pListener;

    for (int i = 0; i < m_pThreadListeners.size(); i++) {
      pListener = m_pThreadListeners.get(i);
      pListener.setProgressStatus(dProgress, pCurrentWork);

    }

  }


  /** 
   *  Informs all thread listeners that the thread ended properly. 
   *  This method should only be called by classes that extend this class. 
   **/
  protected void fireEndedProperlyEvent() {

    //1. make a simple check
    if (m_pThreadListeners == null) return;

    //2. fire the ended properly events
    IThreadListener pListener;

    for (int i = 0; i < m_pThreadListeners.size(); i++) {
      pListener = m_pThreadListeners.get(i);
      pListener.endedProperly();
    }


  }


  
  /** 
   *  Informs all thread listeners that the thread got interrupted and could not be ended properly. 
   *  This method should only be called by classes that extend this class. 
   *
   *  @param pReason The reason why the thread got interrupted. 
   **/
  protected void fireInterruptedEvent(String pReason) {
    //1. make a simple check
    if (m_pThreadListeners == null) return;

    //2. fire the interrupted events
    IThreadListener pListener;

    for (int i = 0; i < m_pThreadListeners.size(); i++) {
      pListener = m_pThreadListeners.get(i);
      pListener.interrupted(pReason);
    }

  }



}
