package transformation;

/*
 *  Copyright (C) 2014 Juergen Platzer 
 * 
 *  This file is part of JFXLibrary.
 *  
 *  JFXLibrary is free software: you can redistribute it and/or modify
 *  it under the terms of the GNU Lesser General Public License as published by
 *  the Free Software Foundation, either version 3 of the License, or
 *  (at your option) any later version.
 *  
 *  JFXLibrary is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU Lesser General Public License for more details.
 *  
 *  You should have received a copy of the GNU Lesser General Public License
 *  along with JFXLibrary.  If not, see <http://www.gnu.org/licenses/>.
 */

//standard imports
import java.awt.image.BufferedImage;
import java.awt.Color;
import java.awt.Graphics2D;
import java.awt.RenderingHints;

//library imports
import standard.ImageEffect;
import standard.EffectParameters;
import util.image.ImageTransformations;

import static transformation.ImageResizeParameters.*;

/**
 *  Implements resizing of images.
 **/
public class ImageResize extends ImageEffect {

  /*** private member variables ***/
  private ImageResizeParameters m_pParameters;
  
  private BufferedImage m_pResizedImage;
  private BufferedImage m_pImage;
  private int m_iNewWidth, m_iNewHeight;

  
  //////////////////////////////
  // CONSTRUCTOR              //
  //////////////////////////////
  /**
   *   Creates a {@code ImageResize} object with the default parameters.
   *
   **/
  public ImageResize() {
    this(new ImageResizeParameters());
  }
  

  /**
   *   Creates a {@code ImageResize} object with the specified parameters..
   *
   *   @param pParameters The parameters that configure the effect.
   **/
  public ImageResize(ImageResizeParameters pParameters) {
    //1. Make an initial check
    if (pParameters == null) pParameters = new ImageResizeParameters();
    
    //2. set the member variables
    m_pParameters = pParameters;
    fetchParameters();
  }
  
  
  //////////////////////////////
  // SET METHODS              //
  //////////////////////////////
  /**
   *   Sets the parameters for the resize effect. 
   *
   *   @param pParameters The parameters that configure the effect. 
   **/
  public void setImageResizeParameters(ImageResizeParameters pParameters) {
    //1. Make an initial check
    if (pParameters == null) return;
    
    
    //2. set the member variables
    m_pParameters = pParameters;
    fetchParameters();
     
  }
  
  ////////////////////////////////
  // SET METHODS                //
  ////////////////////////////////
  /**
   *   Sets the parameters for the resize effect. The parameters are only set, 
   *   if the object is of the type {@code ImageResizeParameters}.
   *
   *   @param pEffectParameters The parameters that configure the effect. 
   **/
  public void setEffectParameters(EffectParameters pEffectParameters) {
    if (pEffectParameters == null) return;
    
    if (pEffectParameters instanceof ImageResizeParameters) {
      setImageResizeParameters((ImageResizeParameters)pEffectParameters);
    }
  }
  
  
  //////////////////////////////
  // GET METHODS              //
  //////////////////////////////
  /**
   *   Returns the parameters that are currently used for the resizing effect.
   *
   *   @return The parameters that are currently used for the effect.
   **/
  public EffectParameters getEffectParameters() {
    return m_pParameters;
  }
  
  
    
  
  //////////////////////////////
  // PROCESS METHODS          //
  //////////////////////////////
  /**
   *  Returns the resized image.
   *
   *  @return The resized image.
   *
   */
  public BufferedImage process() throws Exception {
   
    //1. make some checks
    if (m_pImage == null) return null;
    
    int iOrigWidth = m_pImage.getWidth();
    int iOrigHeight = m_pImage.getHeight();
    if ((iOrigWidth == m_iNewWidth) && (iOrigHeight == m_iNewHeight)) return m_pImage;
    
    /* // does not work, when the image has been changed
    if (m_pResizedImage != null) {
      int iOldWidth = m_pResizedImage.getWidth();
      int iOldHeight = m_pResizedImage.getHeight();
      if ((iOldWidth == m_iNewWidth) && (iOldHeight == m_iNewHeight)) return m_pResizedImage;
    }
    */
    
    //2. calculate the scale
    double dScaleX = (double)m_iNewWidth/(double)iOrigWidth;
    double dScaleY = (double)m_iNewHeight/(double)iOrigHeight;
    
    
    //improve the down scaling quality: 
    /*
     http://stackoverflow.com/questions/4964979/why-does-this-image-look-so-bad-after-being-scaled-down-in-java
     There are three ways to fix the downscaling problem. First is to do it in multiple steps, with no more than 
     a 75% reduction in each step. The second is to blur the image before resizing; the more it shrinks, the 
     more you'll have to blur. The third way is to use a method that filters using more than the 2x2 to 4x4 
     pixel blocks used by a naive bilinear or bicubic interpolation method. As the shrink factor grows larger, 
     so should the pixel block used by the filter, otherwise you get aliasing artifacts as you've seen here.
    */
    
    
    //3. handle if the scale is bigger than 0.75 (here we need not care about the quality)
    if ((dScaleX > 0.75) && (dScaleY > 0.75)) {
      m_pResizedImage = scaleImage(m_iNewWidth, m_iNewHeight, dScaleX, dScaleY, m_pImage);
    }
    else {
      int iNewWidth = iOrigWidth;
      int iNewHeight = iOrigHeight;
      
      if (dScaleX < 0.75) dScaleX = 0.75;
      if (dScaleY < 0.75) dScaleY = 0.75;
      
      iNewWidth = (int)(dScaleX*iNewWidth);
      iNewHeight = (int)(dScaleY*iNewHeight);
      
      BufferedImage pResizedImage = scaleImage(iNewWidth, iNewHeight, dScaleX, dScaleY, m_pImage);
      
      while ((iNewWidth > m_iNewWidth) || (iNewHeight > m_iNewHeight)) {
        
        dScaleX = (double)m_iNewWidth/(double)iNewWidth;
        dScaleY = (double)m_iNewHeight/(double)iNewHeight;
        
        if (dScaleX < 0.75) dScaleX = 0.75;
        if (dScaleY < 0.75) dScaleY = 0.75;
        
        iNewWidth = (int)(dScaleX*iNewWidth);
        iNewHeight = (int)(dScaleY*iNewHeight);
        
        pResizedImage = scaleImage(iNewWidth, iNewHeight, dScaleX, dScaleY, pResizedImage);
      }
      
      m_pResizedImage = pResizedImage;
      
      
    }
   
    
    
   
    
    return m_pResizedImage;
    
    
  }
  
  /////////////////////////////////////////
  // PRIVATE HELPER METHODS              //
  /////////////////////////////////////////
  private void fetchParameters() {
    
    m_pImage = (BufferedImage)m_pParameters.getObjectParameterValue(PARAM_OBJECT_IMAGE);
    m_iNewWidth = m_pParameters.getIntegerParameterValue(PARAM_INT_NEW_WIDTH);
    m_iNewHeight = m_pParameters.getIntegerParameterValue(PARAM_INT_NEW_HEIGHT);
    
    
  }
  
  
  private BufferedImage scaleImage(int iNewWidth, int iNewHeight, double dScaleX, double dScaleY, BufferedImage pImage) {
    //1. create the resized image and configure the graphics context
    BufferedImage pResizedImage = new BufferedImage(iNewWidth, iNewHeight, BufferedImage.TYPE_INT_RGB);
    Graphics2D pGraphics = pResizedImage.createGraphics();
    pGraphics.setRenderingHint(RenderingHints.KEY_INTERPOLATION,RenderingHints.VALUE_INTERPOLATION_BICUBIC);
    //VALUE_INTERPOLATION_BICUBIC , VALUE_INTERPOLATION_BILINEAR, VALUE_INTERPOLATION_NEAREST_NEIGHBOR
    pGraphics.setRenderingHint(RenderingHints.KEY_RENDERING,RenderingHints.VALUE_RENDER_QUALITY);
    pGraphics.setRenderingHint(RenderingHints.KEY_COLOR_RENDERING,RenderingHints.VALUE_COLOR_RENDER_QUALITY);
    pGraphics.setRenderingHint(RenderingHints.KEY_ANTIALIASING,RenderingHints.VALUE_ANTIALIAS_ON);
    
    //2. apply the scale transform and draw the original image
    pGraphics.scale(dScaleX, dScaleY);
    pGraphics.drawImage(pImage, 0, 0, null);
    
    return pResizedImage;
  }
  
 
  
  
  

}
