package util.color;

/*
 *  Copyright (C) 2014 Juergen Platzer 
 * 
 *  This file is part of JFXLibrary.
 *  
 *  JFXLibrary is free software: you can redistribute it and/or modify
 *  it under the terms of the GNU Lesser General Public License as published by
 *  the Free Software Foundation, either version 3 of the License, or
 *  (at your option) any later version.
 *  
 *  JFXLibrary is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU Lesser General Public License for more details.
 *  
 *  You should have received a copy of the GNU Lesser General Public License
 *  along with JFXLibrary.  If not, see <http://www.gnu.org/licenses/>.
 */

import java.awt.Color;


/** 
 * Represents a positioned color holding a color value and a one dimensional position as double. 
 *
 **/
public class ColorPoint1D implements Comparable<ColorPoint1D> {

  /** private member variables ***/
  private Color m_pColor;
  private double m_dPosition;
  
  
  ////////////////////////////////
  // CONSTRUCTOR                //
  ////////////////////////////////
  /** 
   * Creates a color point object with the specified color and position. 
   *
   * @param pColor The color of the color point.
   * @param dPosition The position of the color point as double value.
   *
   **/
  public ColorPoint1D(Color pColor, double dPosition) {
    if (pColor == null) pColor = Color.BLACK;
    
    m_pColor = pColor;
    m_dPosition = dPosition;
  }
  
  
  /////////////////////////
  // SET METHODS         //
  /////////////////////////
  /** 
   * Sets the color of the color point.
   *
   * @param pColor The color of the color point.
   *
   **/
  public void setColor(Color pColor) {
    if (pColor == null) pColor = Color.BLACK;
    
    m_pColor = pColor;
  }
  
  /** 
   * Sets the position of the color point.
   *
   * @param dPosition The position of the color point as double value.
   *
   **/
  public void setPosition(double dPosition) {
    m_dPosition = dPosition;
  }
  
  //////////////////////////
  // GET METHODS          //
  //////////////////////////
  /**
   *  Returns the color of the color point. 
   *
   *  @return The color of the color point.
   **/
  public Color getColor() {
    return m_pColor;
  }
  
  /**
   *  Returns the position of the color point. 
   *
   *  @return The position of the color point.
   **/
  public double getPosition() {
    return m_dPosition;
  }
  
  //////////////////////////
  // COMPARABLE INTERFACE //
  //////////////////////////
  /**
   *  Implementation of the comparable interface to sort color points according to their position. 
   *
   *  @param pOtherPoint The other color point which is needed for the comparison. 
   *
   *  @return 1 if the position is greater than the position of the other point. -1 if the position is lower than the position of the other point. 
   *          0 if the positions are equal.
   **/
  public int compareTo(ColorPoint1D pOtherPoint) {
    double dOtherPosition = pOtherPoint.getPosition();
    if (dOtherPosition < m_dPosition) return 1;
    if (dOtherPosition > m_dPosition) return -1;
    
    return 0;
  }

}
