package util.image;

/*
 *  Copyright (C) 2014 Juergen Platzer 
 * 
 *  This file is part of JFXLibrary.
 *  
 *  JFXLibrary is free software: you can redistribute it and/or modify
 *  it under the terms of the GNU Lesser General Public License as published by
 *  the Free Software Foundation, either version 3 of the License, or
 *  (at your option) any later version.
 *  
 *  JFXLibrary is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU Lesser General Public License for more details.
 *  
 *  You should have received a copy of the GNU Lesser General Public License
 *  along with JFXLibrary.  If not, see <http://www.gnu.org/licenses/>.
 */

//standard imports
import java.awt.image.BufferedImage;
import java.awt.Color;


/**
 * Provides methods to fill areas of images with a specific color. TO DO: Get rid of static methods and use object oriented style?
 **/

public class FillOperations {



  /**
   *  Returns a copy of the given image, where the pixels indicated by the mask are filled with the given color.
   *
   *  @param pImage    BufferedImage representing the image from which should be filled with a color at certain pixels.
   *  @param pColor    The color that should be used for the filling operation.
   *  @param pMask     The mask indicating for each pixel if it should be colored with the given color.
   *
   *  @return The copy of the given image, where the pixels indicated by the mask are filled with the given color.
   */
  public static BufferedImage fillMaskCopy(BufferedImage pImage, Color pColor, BufferedImage pMask) {

    //1. make some checks
    if (pImage == null) return null;
    if (pColor == null) return null;
    if (pMask == null) return null; //or return a copy of the original image


    //2. create a copy of the original image
    BufferedImage pFilledImage = ImageTransformations.copyImage(pImage);

    //3. fill the image with the fill color, if the mask indicates that
    int iWidth = Math.min(pImage.getWidth(), pMask.getWidth());
    int iHeight = Math.min(pImage.getHeight(), pMask.getHeight());

    int iFillColorRGB = pColor.getRGB();
    int iFillColorRed = pColor.getRed();
    int iFillColorGreen = pColor.getGreen();
    int iFillColorBlue = pColor.getBlue();


    int iBlackRGB = Color.BLACK.getRGB();
    int iWhiteRGB = Color.WHITE.getRGB();

    int iMaskRGB, iMaskRed, iMaskGreen, iMaskBlue;
    int iRGB, iRed, iGreen, iBlue;
    double dLambdaRed, dLambdaGreen, dLambdaBlue;

    for (int x = 0; x < iWidth; x++) {
      for (int y = 0; y < iHeight; y++) {

        //I. do nothing, if the mask color is black
        iMaskRGB = pMask.getRGB(x,y);

        if (iMaskRGB == iBlackRGB) continue;


        //II. set the fill color if the mask has the color white
        if (iMaskRGB == iWhiteRGB) {
          pFilledImage.setRGB(x,y,iFillColorRGB);
          continue;
        }

        //III. fill the image with a combination of the original image color and the fill color
        iRGB = pFilledImage.getRGB(x,y);
        iRed = (iRGB >> 16) & 0xff;
        iGreen = (iRGB >> 8) & 0xff;
        iBlue = (iRGB >> 0) & 0xff;

        iMaskRed = (iMaskRGB >> 16) & 0xff;
        iMaskGreen = (iMaskRGB >> 8) & 0xff;
        iMaskBlue = (iMaskRGB >> 0) & 0xff;

        dLambdaRed = (double)iMaskRed/255;
        dLambdaGreen = (double)iMaskGreen/255;
        dLambdaBlue = (double)iMaskBlue/255;

        iRed = (int)(dLambdaRed*iFillColorRed + (1-dLambdaRed)*iRed);
        iGreen = (int)(dLambdaGreen*iFillColorGreen + (1-dLambdaGreen)*iGreen);
        iBlue = (int)(dLambdaBlue*iFillColorBlue + (1-dLambdaBlue)*iBlue);

        iRGB = (255 << 24) | (iRed << 16) | (iGreen << 8) | iBlue;
        pFilledImage.setRGB(x,y, iRGB);




      }
    }

    return pFilledImage;



  }

}