package util.image;


/*
 *  Copyright (C) 2014 Juergen Platzer 
 * 
 *  This file is part of JFXLibrary.
 *  
 *  JFXLibrary is free software: you can redistribute it and/or modify
 *  it under the terms of the GNU Lesser General Public License as published by
 *  the Free Software Foundation, either version 3 of the License, or
 *  (at your option) any later version.
 *  
 *  JFXLibrary is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU Lesser General Public License for more details.
 *  
 *  You should have received a copy of the GNU Lesser General Public License
 *  along with JFXLibrary.  If not, see <http://www.gnu.org/licenses/>.
 */

//standard imports
import java.awt.image.BufferedImage;
import java.awt.image.ColorConvertOp;
import java.awt.image.LookupOp;
import java.awt.image.ShortLookupTable;

import java.awt.Graphics2D;

import java.awt.color.ColorSpace;

//library imports
import standard.ImageEffect;

/** 
 * Provides basic image color transformation methods. TO DO: Get rid of static methods and use object oriented style?
 *
 **/

public class ImageTransformations {


  /**
   *  Returns a copy of the specified image. 
   *
   *  @param pImage The image that should be copied.
   * 
   *  @return A copy of the specified image.
   **/
  public static BufferedImage copyImage(BufferedImage pImage) {
    //1. make a simple check
    if (pImage == null) return null;

    //2. create a new image object
    int iWidth = pImage.getWidth();
    int iHeight = pImage.getHeight();
    int iType = pImage.getType();
    if (iType == BufferedImage.TYPE_CUSTOM) iType = BufferedImage.TYPE_INT_RGB;

    BufferedImage pNewImage = new BufferedImage(iWidth, iHeight, iType);
    Graphics2D pGraphics = pNewImage.createGraphics();

    //3. copy the image
    pGraphics.drawImage(pImage, 0, 0, null);

    return pNewImage;
  }



  /**
   *  Inverts the specified image. TO DO: Use only one inversion worker method with a source and a target image. If the image should be directly inverted, 
   *  the source and the target should be the same. If a copy should be generated, create a copy first with the copy method and use the copy as target image.
   *
   *  @param pImage The image that should be inverted.
   * 
   **/
  public static void invertImage(BufferedImage pImage) {
    //1. make a simple check
    if (pImage == null) return;

    //2. invert the image
    int iWidth = pImage.getWidth();
    int iHeight = pImage.getHeight();

    int iRGB, iAlpha, iRed, iGreen, iBlue;

    for (int x = 0; x < iWidth; x++) {
      for (int y = 0; y < iHeight; y++) {

        iRGB = pImage.getRGB(x,y);

        iAlpha = (iRGB >> 24) & 0xff;
        iRed = (iRGB >> 16) & 0xff;
        iGreen = (iRGB >> 8) & 0xff;
        iBlue = (iRGB >> 0) & 0xff;

        iRed = 255 - iRed;
        iGreen = 255 - iGreen;
        iBlue = 255 - iBlue;

        iRGB = (iAlpha << 24) | (iRed << 16) | (iGreen << 8) | iBlue;

        pImage.setRGB(x,y, iRGB);

      }
    }
  }
  
  /**
   *  Returns the inverted version of the speicified image. TO DO: Use only one inversion worker method with a source and a target image. If the image should be directly inverted, 
   *  the source and the target should be the same. If a copy should be generated, create a copy first with the copy method and use the copy as target image.
   *
   *  @param pImage The image that should be inverted.
   * 
   *  @return An inverted copy of the speicified image.
   **/
  public static BufferedImage invertImageCopy(BufferedImage pImage) {
    //1. make a simple check
    if (pImage == null) return null;

    //2. create the inverted the image
    int iWidth = pImage.getWidth();
    int iHeight = pImage.getHeight();
    BufferedImage pInvImage = new BufferedImage(iWidth, iHeight, BufferedImage.TYPE_INT_RGB);

    int iRGB, iAlpha, iRed, iGreen, iBlue;

    for (int x = 0; x < iWidth; x++) {
      for (int y = 0; y < iHeight; y++) {

        iRGB = pImage.getRGB(x,y);

        iAlpha = (iRGB >> 24) & 0xff;
        iRed = (iRGB >> 16) & 0xff;
        iGreen = (iRGB >> 8) & 0xff;
        iBlue = (iRGB >> 0) & 0xff;

        iRed = 255 - iRed;
        iGreen = 255 - iGreen;
        iBlue = 255 - iBlue;

        iRGB = (iAlpha << 24) | (iRed << 16) | (iGreen << 8) | iBlue;

        pInvImage.setRGB(x,y, iRGB);

      }
    }
    
    return pInvImage;
  }


  /**
   *  Converts the specified image to a gray scale image. TO DO: Use only one conversion worker method with a source and a target image. If the image should be directly converted, 
   *  the source and the target should be the same. If a copy should be generated, create a copy first with the copy method and use the copy as target image.
   *
   *  @param pImage The image that should be converted to a gray scale image.
   * 
   **/
  public static void convertToGrayscaleImage(BufferedImage pImage) {
    //1. make a simple check
    if (pImage == null) return;

    //2. invert the image
    int iWidth = pImage.getWidth();
    int iHeight = pImage.getHeight();

    int iRGB, iAlpha, iRed, iGreen, iBlue, iGray;

    for (int x = 0; x < iWidth; x++) {
      for (int y = 0; y < iHeight; y++) {

        iRGB = pImage.getRGB(x,y);

        iAlpha = (iRGB >> 24) & 0xff;
        iRed = (iRGB >> 16) & 0xff;
        iGreen = (iRGB >> 8) & 0xff;
        iBlue = (iRGB >> 0) & 0xff;


        iGray = (int)((double)(iRed + iGreen + iBlue)/3);
        if (iGray > 255) iGray = 255;
        if (iGray < 0) iGray = 0;


        iRGB = (iAlpha << 24) | (iGray << 16) | (iGray << 8) | iGray;

        pImage.setRGB(x,y, iRGB);

      }
    }
  }


  /**
   *  Returns a gray scale converted copy of the specified image. TO DO: Use only one conversion worker method with a source and a target image. If the image should be directly converted, 
   *  the source and the target should be the same. If a copy should be generated, create a copy first with the copy method and use the copy as target image.
   *
   *  @param pImage The image that should be converted to a gray scale image.
   *  @param pEffect The image effect object that is used to check if the process is still running. (TO DO: switch to a listener interface!)
   *
   *  @return The grayscale version of the specified image.
   * 
   **/
  public static BufferedImage convertToGrayscaleImageCopy(BufferedImage pImage, ImageEffect pEffect) {
    //1. make a simple check
    if (pImage == null) return null;

    //2. invert the image
    int iWidth = pImage.getWidth();
    int iHeight = pImage.getHeight();
    BufferedImage pNewImage = new BufferedImage(iWidth, iHeight, BufferedImage.TYPE_INT_RGB);

    int iRGB, iAlpha, iRed, iGreen, iBlue, iGray;

    for (int x = 0; x < iWidth; x++) {
      for (int y = 0; y < iHeight; y++) {

        iRGB = pImage.getRGB(x,y);

        iAlpha = (iRGB >> 24) & 0xff;
        iRed = (iRGB >> 16) & 0xff;
        iGreen = (iRGB >> 8) & 0xff;
        iBlue = (iRGB >> 0) & 0xff;


        iGray = (int)((double)(iRed + iGreen + iBlue)/3);
        if (iGray > 255) iGray = 255;
        if (iGray < 0) iGray = 0;

        iRGB = (iAlpha << 24) | (iGray << 16) | (iGray << 8) | iGray;

        pNewImage.setRGB(x,y, iRGB);

      }
      if (pEffect != null) {
        if (!pEffect.isRunning()) return null;
      }
    }

     return pNewImage;
  }


  /**
   *  Converts the specified image to a gray scale image by using conversion methods of the java API. TO DO: Use only one conversion worker 
   *  method with a source and a target image. 
   *
   *  @param pImage The image that should be converted to a gray scale image.
   * 
   **/
  public static void convertToGrayscaleImage2(BufferedImage pImage) {
    //1. make a simple check
    if (pImage == null) return;

    //2. create a color convert op
    ColorSpace pRGBSpace = ColorSpace.getInstance(ColorSpace.CS_sRGB);
    ColorSpace pGraySpace = ColorSpace.getInstance(ColorSpace.CS_GRAY);
    ColorConvertOp pColorConvertOp = new ColorConvertOp(pRGBSpace, pGraySpace, null);

    //3. convert the image
    pColorConvertOp.filter(pImage, pImage);

  }

   /**
   *  Returns a gray scale converted copy of the specified image by using conversion methods of the java API. TO DO: Use only one conversion worker method .
   *
   *  @param pImage The image that should be converted to a gray scale image.
   *
   *  @return The grayscale version of the specified image.
   * 
   **/
  public static BufferedImage convertToGrayscaleImage2Copy(BufferedImage pImage) {
    //1. make a simple check
    if (pImage == null) return null;

    //2. create a color convert op
    ColorSpace pRGBSpace = ColorSpace.getInstance(ColorSpace.CS_sRGB);
    ColorSpace pGraySpace = ColorSpace.getInstance(ColorSpace.CS_GRAY);
    ColorConvertOp pColorConvertOp = new ColorConvertOp(pRGBSpace, pGraySpace, null);

    //3. convert the image
    return pColorConvertOp.filter(pImage, null);

  }

  /**
   *  Converts the specified image to a gray scale image by using the luminance formula. TO DO: Use only one conversion worker method with a source and a target image. 
   *  If the image should be directly converted, 
   *  the source and the target should be the same. If a copy should be generated, create a copy first with the copy method and use the copy as target image.
   *  The luminance formula should only be coded at one place!!!
   *
   *  @param pImage The image that should be converted to a gray scale image.
   * 
   **/
  public static void convertToLuminanceImage(BufferedImage pImage) {
    //1. make a simple check
    if (pImage == null) return;

    //2. invert the image
    int iWidth = pImage.getWidth();
    int iHeight = pImage.getHeight();

    int iRGB, iAlpha, iRed, iGreen, iBlue, iLuminance;

    for (int x = 0; x < iWidth; x++) {
      for (int y = 0; y < iHeight; y++) {

        iRGB = pImage.getRGB(x,y);

        iAlpha = (iRGB >> 24) & 0xff;
        iRed = (iRGB >> 16) & 0xff;
        iGreen = (iRGB >> 8) & 0xff;
        iBlue = (iRGB >> 0) & 0xff;


        iLuminance = (int)(0.299*iRed + 0.587*iGreen + 0.114*iBlue);
        if (iLuminance > 255) iLuminance = 255;
        if (iLuminance < 0) iLuminance = 0;


        iRGB = (iAlpha << 24) | (iLuminance << 16) | (iLuminance << 8) | iLuminance;

        pImage.setRGB(x,y, iRGB);

      }
    }
  }



  ////////////////////////////
  // THRESHOLD OPERATIONS   //
  ////////////////////////////
  /**
   *  Converts the specified image to a binary image by using a treshold that decides at which value a pixel will be set to white. 
   *  TO DO: Use only one conversion worker method with a source and a target image. 
   *  If the image should be directly converted, 
   *  the source and the target should be the same. If a copy should be generated, create a copy first with the copy method and use the copy as target image.
   *
   *  @param pImage The image that should be converted to a binary image.
   *  @param iThreshold The threshold as integer deciding if the pixel should be set to white or black.
   * 
   **/
  public static void convertToBinaryImage(BufferedImage pImage, int iThreshold) {

    //1. make some checks
    if (pImage == null) return;
    if (iThreshold < 0) iThreshold = 0;
    if (iThreshold > 255) iThreshold = 255;

    //2. create a simple lookuptable
    short[] pLookupTable = new short[256];

    for (int i = 0; i < 256; i++) {
      pLookupTable[i] = (i < iThreshold) ? (short)0 : (short)255;
    }

    //3. create an operation
    LookupOp pLookupOp = new LookupOp(new ShortLookupTable(0, pLookupTable), null);

    //4. transform the image
    pLookupOp.filter(pImage, pImage);
  }

  /**
   *  Returns a binary image copy of the specified image by using a treshold that decides at which value a pixel will be set to white. 
   *  TO DO: Use only one conversion worker method with a source and a target image. 
   *  If the image should be directly converted, 
   *  the source and the target should be the same. If a copy should be generated, create a copy first with the copy method and use the copy as target image.
   *
   *  @param pImage The image that should be converted to a binary image.
   *  @param iThreshold The threshold as integer deciding if the pixel should be set to white or black.
   * 
   *  @return A binary image copy of the specified image.
   **/
  public static BufferedImage convertToBinaryImageCopy(BufferedImage pImage, int iThreshold) {

    //1. make some checks
    if (pImage == null) return null;
    if (iThreshold < 0) iThreshold = 0;
    if (iThreshold > 255) iThreshold = 255;

    //2. create a simple lookuptable
    short[] pLookupTable = new short[256];

    for (int i = 0; i < 256; i++) {
      pLookupTable[i] = (i < iThreshold) ? (short)0 : (short)255;
    }

    //3. create an operation
    LookupOp pLookupOp = new LookupOp(new ShortLookupTable(0, pLookupTable), null);

    //4. transform the image
    return pLookupOp.filter(pImage, null);
  }




  /////////////////////////////////
  // COLOR REDUCTION             //
  /////////////////////////////////
  /**
   *  Returns a copy of the specified image with the specified number of colors. 
   *  The colors are calculated by using equal distances on each color channel. 
   *  TO DO: Use only one worker method that handles the number of colors by channel. 
   *  Pass on the same number of colors for each channel.
   *  
   *
   *  @param pImage The image that should be converted to a binary image.
   *  @param iColorNumPerChannel The number of colors that should be used in the result image.
   *
   *  @return A copy of the specified image with the specified number of colors. 
   * 
   **/
  public static BufferedImage getColorReducedImageCopy(BufferedImage pImage, int iColorNumPerChannel) {

    //1. make some chekcs
    if (pImage == null) return null;
    if (iColorNumPerChannel < 2) iColorNumPerChannel = 2;
    if (iColorNumPerChannel > 255) iColorNumPerChannel = 255;

    //2. create a lookuptable operation
    short[] pLookupTable = new short[256];

    //256/16 = 32
    int iStepSize = 256 / iColorNumPerChannel;

    for (int i = 0; i < 256; i++) {
      //255 - (255 % 32) = 255 - 31 = 224
      //254 - (254 % 32) = 253 - 30 = 224
      //...
      //1 - (1 % 32) = 0
      //0 - (0 % 32) = 0
      // -> maps each value to the lower interval limit
      pLookupTable[i] = (short)(i - (i % iStepSize));
      //pLookupTable[i] = (short)(i - (i % 32));
    }

    LookupOp pLookupOp = new LookupOp(new ShortLookupTable(0, pLookupTable), null);


    //3. transform the image
    //BufferedImage pHelp = new BufferedImage(pImage.getWidth(), pImage.getHeight(), BufferedImage.TYPE_INT_RGB);
    BufferedImage pHelp = copyImage(pImage);
    pLookupOp.filter(pHelp, pHelp);
    return pHelp;

  }

  /**
   *  Returns a copy of the specified image with the specified number of colors per channel. 
   *  
   *
   *  @param pImage The image that should be converted to a binary image.
   *  @param pColorNumPerChannel The number of colors per channel that should be used in the result image.
   * 
   *  @return A copy of the specified image with the specified number of colors per channel. 
   **/
  public static BufferedImage getColorReducedImageCopy(BufferedImage pImage, int[] pColorNumPerChannel) {

    //1. make some chekcs
    //1.1. null checks
    if (pImage == null) return null;
    if (pColorNumPerChannel == null) return null;

    //1.2. check the correct length of the color number array
    if (pColorNumPerChannel.length < 3) {
      int[] pNewColorNumPerChannel = new int[3];

      for (int i = 0; i < 3; i++) {
        if (i < pColorNumPerChannel.length) {
          pNewColorNumPerChannel[i] = pColorNumPerChannel[i];
        }
        else {
          pNewColorNumPerChannel[i] = pColorNumPerChannel[pColorNumPerChannel.length-1];
        }
      }

      pColorNumPerChannel = pNewColorNumPerChannel;
    }

    //1.3. check the correct value range of all entries of the color number array
    for (int i = 0; i < pColorNumPerChannel.length; i++) {
      if (pColorNumPerChannel[i] < 2) pColorNumPerChannel[i] = 2;
      if (pColorNumPerChannel[i] > 255) pColorNumPerChannel[i] = 255;
    }


    //2. create a lookuptable operation
    short[][] pLookupTable = new short[3][256];


    for (int iChannelId = 0; iChannelId < 3; iChannelId++) {
      //256/16 = 32
      int iStepSize = 256 / pColorNumPerChannel[iChannelId];

      for (int i = 0; i < 256; i++) {
        //255 - (255 % 32) = 255 - 31 = 224
        //254 - (254 % 32) = 253 - 30 = 224
        //...
        //1 - (1 % 32) = 0
        //0 - (0 % 32) = 0
        // -> maps each value to the lower interval limit
        pLookupTable[iChannelId][i] = (short)(i - (i % iStepSize));
      }
    }

    LookupOp pLookupOp = new LookupOp(new ShortLookupTable(0, pLookupTable), null);


    //3. transform the image
    //return pLookupOp.filter(pImage, null);

		BufferedImage pHelp = copyImage(pImage);
    pLookupOp.filter(pHelp, pHelp);
    return pHelp;



  }





}